/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    SUN_RAWfile.c (SUN .au file and RAW file read/write)
 *
 * Date:    Jan, 4, 1994
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#if defined(SVR4)
# include <sys/audioio.h>
# include "/usr/demo/SOUND/include/multimedia/audio_device.h"
# include "/usr/demo/SOUND/include/multimedia/audio_filehdr.h"
# include "/usr/demo/SOUND/include/multimedia/libaudio.h"
#else
# include <sun/audioio.h>
# include <multimedia/audio_device.h>
# include <multimedia/audio_filehdr.h>
# include <multimedia/libaudio.h>
#endif /* SVR4 */

#define NOX
#include "xplay.h"

#define MAGIC   ((unsigned long)0x2e736e64)

/*****  Write File Header for SUN and My header  *****/
int SUN_write_filehdr (rec_obj)
Audio_obj *rec_obj;
{
    int       err = False, hsize = 24;
    char      buff[BUFSIZ];
    unsigned  long HEADER[6];

    hsize  = strlen(rec_obj -> Infostr);
    hsize += hsize % 8;
    memset (buff, 0, sizeof (buff));
    sprintf (buff, "%s", rec_obj -> Infostr);
    HEADER[0] = MAGIC;
    HEADER[1] = 24 + hsize;
    HEADER[2] = 0;
    switch (rec_obj -> Dev_hdr.encoding) {
      case MULAW8  : HEADER[3] = AUDIO_FILE_ENCODING_MULAW_8;   break;
      case PCM8    : HEADER[3] = AUDIO_FILE_ENCODING_LINEAR_8;  break;
      case PCM16   : HEADER[3] = AUDIO_FILE_ENCODING_LINEAR_16; break;
      case PCM24   : HEADER[3] = AUDIO_FILE_ENCODING_LINEAR_24; break;
      case PCM32   : HEADER[3] = AUDIO_FILE_ENCODING_LINEAR_32; break;
      case FLOAT32 : HEADER[3] = AUDIO_FILE_ENCODING_FLOAT;     break;
      case DOUBLE64: HEADER[3] = AUDIO_FILE_ENCODING_DOUBLE;    break;
      default      : HEADER[3] = UNKNOWNENC;                    break;
    }; 

    HEADER[4] = (unsigned long) rec_obj -> Dev_hdr.sample_rate;
    HEADER[5] = (unsigned long) rec_obj -> Dev_hdr.channels;
    if (write (rec_obj -> ofd, HEADER, sizeof(long)*6) != sizeof(long)*6)
      err = True;
    if (write (rec_obj -> ofd, buff, hsize) != hsize)
      err = True;

    rec_obj -> suffix = SUN;
    rec_obj -> File_hdr.endian = BIGENDIAN;
    memset (buff, 0, sizeof (buff));
    sprintf (buff, "xplay %c", (unsigned char)rec_obj -> volume_scale);

    return write (rec_obj -> ofd, buff, EXTRAHEADERSIZE);
/*
 *  return Extra header size, not true header size.
 */
}

/*****  Rewrite filesize for SUN  *****/
int SUN_rewrite_filesize (fd, rw_size)
int fd;
unsigned long rw_size;
{
    unsigned long data_size = (unsigned long)rw_size;

    lseek (fd, (long)sizeof(long)*2L, SEEK_SET);
    return write (fd, &data_size, sizeof(long));
}

/*** Read File Header for SUN ***/
Boolean SUN_read_filehdr (play_obj)
Audio_obj *play_obj;
{
    int       err = False;
    int       bytes_per_unit, encoding;
    char      buff[BUFSIZ];
    unsigned  long HEADER[6];
    int       fd = play_obj -> ifd;
/*
    HEADER[6] =
    unsigned long magic;
    unsigned long hdr_size;
    unsigned long data_size;
    unsigned long encoding;
    unsigned long sample_rate;
    unsigned long channels;
*/
    if (fd < 3) 
      err = True;
    else {
      if (read (fd, HEADER, sizeof(long) * 6) != sizeof(long) * 6) err = True;
      if (MAGIC != HEADER[0])                       err = True;
    }
    if (err == True) {
      Put_Message (Basename (play_obj -> filename), 
                   " is not a valid Sun/NeXT .au audio file.");
      return False;
    }
    if (HEADER[1] == 24)
      buff[0] = Null;
    else
      if (HEADER[1] > MAX_INFO + 24) 
        err = True;
      else
        if (read (fd, buff, HEADER[1] - 24) != HEADER[1] - 24) err = True;
    if (err == True) {
      Put_Message (Basename (play_obj -> filename), 
                   " is not a valid Sun/NeXT .au audio file.");
      return False;
    }

    strncpy (play_obj -> Infostr, buff, MAX_INFO);
    play_obj -> Infostr[MAX_INFO] = Null;
    play_obj -> header_size = tell (fd);
/*** read volume scale for adjust volume  ***/

    play_obj -> volume_scale = 100;
    if (STATE(ADJUST)) {
       int volume_scale = 100;

       memset (buff, 0, sizeof (buff));
       if (read (fd, buff, EXTRAHEADERSIZE) != EXTRAHEADERSIZE) {
         /***  Is this audio file ?  ***/
         Put_Message (Basename (play_obj -> filename)," has not enough data.");
         return False;
       }
       if (memcmp (buff, XPLAYID, strlen (XPLAYID)) == 0) {
         /***  Found XPlay MAGIC  ***/
         volume_scale = (unsigned char) buff[strlen (XPLAYID)];
       }
       play_obj -> volume_scale = volume_scale;
    }

    play_obj -> File_hdr.header_size = HEADER[1];
    play_obj -> File_hdr.data_size   = HEADER[2];
/*
 *  Change Sun .au file encoding define to 
 *  my encoding define (See xplay.h)  
 */
    switch (HEADER[3]) {
      case AUDIO_FILE_ENCODING_MULAW_8:
                                  encoding = MULAW8;
                                  bytes_per_unit = 1;
                                  break;
      case AUDIO_FILE_ENCODING_LINEAR_8:
                                  encoding = PCM8;
                                  bytes_per_unit = 1;
                                  break;
      case AUDIO_FILE_ENCODING_LINEAR_16:
                                  encoding = PCM16;
                                  bytes_per_unit = 2;
                                  break;
      case AUDIO_FILE_ENCODING_LINEAR_24:
                                  encoding = PCM24;
                                  bytes_per_unit = 3;
                                  break;
      case AUDIO_FILE_ENCODING_LINEAR_32:
                                  encoding = PCM32;
                                  bytes_per_unit = 4;
                                  break;
      case AUDIO_FILE_ENCODING_FLOAT:
                                  encoding = FLOAT32;
                                  bytes_per_unit = 4;
                                  break;
      case AUDIO_FILE_ENCODING_DOUBLE:
                                  encoding = DOUBLE64;
                                  bytes_per_unit = 8;
                                  break;
/* a-law is not supported 
      case AUDIO_FILE_ENCODING_ALAW:
                                  encoding = ALAW8;
                                  bytes_per_unit = 1;
                                  break;
*/
      default:                    encoding = UNKNOWNENC;
                                  bytes_per_unit = 0;
                                  break;
    };
    play_obj -> File_hdr.encoding    = encoding;
    play_obj -> File_hdr.unit_size   = bytes_per_unit;
    play_obj -> File_hdr.sample_rate = HEADER[4];
    play_obj -> File_hdr.channels    = HEADER[5];
    play_obj -> File_hdr.samples_per_unit = 1;
    play_obj -> File_hdr.magicid     = SUN;
    play_obj -> File_hdr.endian      = BIGENDIAN;
    play_obj -> suffix               = SUN;
    return ((HEADER[0] <= 0 || HEADER[1] <= 0 || HEADER[2] <= 0 ||
             HEADER[3] <= 0 || HEADER[4] <= 0 || HEADER[5] <= 0)
                            ? False : True);
}

/*****  Read u-law byte raw file open  *****/
Boolean ULRAW_read_filehdr (play_obj)
Audio_obj *play_obj;
{
    struct stat st;
    
    if (stat (play_obj -> filename, &st) < 0) {
      Put_Message ("Can't stat ", Basename (play_obj -> filename));
      return False;
    }

    if (SUN_read_filehdr (play_obj))
      return True;
    Put_Message ("Try ulaw encoding.", " ");
    lseek (play_obj -> ifd, 0, SEEK_SET);

    play_obj -> Infostr[0] = Null;
    play_obj -> header_size = 0;
    play_obj -> volume_scale = 100;
    play_obj -> File_hdr.header_size = 0;
    play_obj -> File_hdr.data_size   = st.st_size;
    play_obj -> File_hdr.encoding    = MULAW8;
    play_obj -> File_hdr.unit_size   = 1;
    play_obj -> File_hdr.sample_rate = 8000;
    play_obj -> File_hdr.channels    = MONO;
    play_obj -> File_hdr.samples_per_unit = 1;
    play_obj -> File_hdr.magicid     = ULRAW;
    play_obj -> File_hdr.endian      = BIGENDIAN;
    play_obj -> suffix               = ULRAW;
    return True;
}

