/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    SUNaudio.c   (for SUN's audio control)
 *
 * Date:    Dec, 9, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#if defined(SVR4)
# include <sys/audioio.h>
# include "/usr/demo/SOUND/include/multimedia/audio_device.h"
# include "/usr/demo/SOUND/include/multimedia/audio_filehdr.h"
# include "/usr/demo/SOUND/include/multimedia/libaudio.h"
#else
# include <sun/audioio.h>
# include <multimedia/audio_device.h>
# include <multimedia/audio_filehdr.h>
# include <multimedia/libaudio.h>
#endif /* SVR4 */

#define NOX
#include "xplay.h"

static int    ACFD = -1;              /*  audio control fd        */
static int    PPFD = -1;              /*  auido play pause fd     */
static int    RPFD = -1;              /*  auido record pause fd   */

static audio_info_t CONFIGSAVE;       /*  backup for configuration*/
static Gain_obj     gain_obj = {-1, -1, -1};/* Gain Control Object */

extern Audio_obj *export_play_obj;    /*  Inport from audio.c     */
/*
 *  Prototypes
 */

int      set_inputport ();
int      invert_inputport ();
int      set_outputport ();
int      invert_outputport ();
int      set_playpause ();
int      invert_playpause ();
int      set_recpause ();
int      invert_recpause ();
int      set_playgain ();
int      set_recordgain ();
int      set_monitorgain ();
Boolean  set_audio_config ();
void     CheckAudioDevice ();
void     GetAudioConfiguration ();
void     Push_Configuration ();
void     Pop_Configuration ();
Boolean  SUN_read_filehdr ();
Boolean  SetAudioConfiguration ();
extern   void Put_Message ();
extern   int STATE();
Audio_hdr Convert_Sun_Header ();
File_obj  Convert_File_Header ();
File_obj  To_File_obj ();
File_obj  Audio_Device_to_File_obj ();
Audio_Device File_obj_to_Audio_Device ();
int       device_drain ();

/******  Set Input Port (Microphone / Line in)  ******/
int set_inputport (value)
int value;
{
    audio_info_t  ACnew;

    AUDIO_INITINFO (&ACnew);
    ACnew.record.port = value;
     if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0)
       Put_Message (" Error : ", "Set output port.");
    return value;
}

/*****  Change Input Port (Microphone / Line in) *****/
int invert_inputport (AD)
Audio_Device AD;
{
    if (AD.port == AUDIO_MICROPHONE)
      return set_inputport (AUDIO_LINE_IN);
    else
      return set_inputport (AUDIO_MICROPHONE);
}

/*****  Change Output Port (Speaker / Headphone / Line Out)  *****/
int set_outputport (value)
int value;
{
    char *p;
    audio_info_t  ACnew;

#ifndef AUDIO_MID_BALANCE
    p = (char *)&ACnew.monitor_gain;
#endif
    AUDIO_INITINFO (&ACnew);
    if (value == OUTPUT_MUTE)
#ifndef AUDIO_MID_BALANCE
      *(p + sizeof(int)) = (unsigned char)(STATE(MUTED) ? False : True);
#else
      ACnew.output_muted = (unsigned char)(STATE(MUTED) ? False : True);
#endif
    else {
      if (STATE(PROCESS) == DBRI) {
#ifndef AUDIO_MID_BALANCE
        *(p + sizeof(int)) = (unsigned char) False;
#else
        ACnew.output_muted = (unsigned char) False;
#endif
      }
      ACnew.play.port = value;
    }
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0)
       Put_Message (" Error : ", "Set output port.");

    return value;
}

/*****  Change Output Port (Speaker / Headphone / Line out)  *****/
int invert_outputport (AD)
Audio_Device AD;
{
    if (AD.port == AUDIO_SPEAKER)
      return set_outputport (AUDIO_HEADPHONE);
    else
    if (AD.port == AUDIO_HEADPHONE)
      return set_outputport (AUDIO_LINE_OUT);
    else
      return set_outputport (AUDIO_SPEAKER);
}

/******  Set Channel  (STEREO / MONO)  ******/
int set_channel (value)
int value;
{
    Audio_Device newPlay;
    Audio_Device newRecord;
    Audio_Gain   newGain;

    GetAudioConfiguration (&newPlay, &newRecord, &newGain);
    set_audio_config (newPlay.sample_rate, newPlay.encoding,
                      newPlay.bits, value);
    return value;
}

/*****  Change Channel  *****/
int invert_channel (AD)
Audio_Device AD;
{
    if (AD.channel == MONO)
      return set_channel (STEREO);
    else
      return set_channel (MONO);
}

/*****  Set Play Pause  *****/
int set_playpause (flag)
int flag;
{
    audio_info_t ACnew;
    Audio_Device newPlay;
    Audio_Device newRecord;
    Audio_Gain   newGain;

    if ( flag && (PPFD < 0)) { /**  If audio dev not open, open for pause  **/
      GetAudioConfiguration (&newPlay, &newRecord, &newGain);
      if ((PPFD = open (AUDIO_IODEV, O_WRONLY | O_NDELAY)) > 0)
        SetAudioConfiguration (PPFD, newPlay, newRecord, newGain);
    }

    AUDIO_INITINFO (&ACnew);
    ACnew.play.pause = flag;
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : Play ", "Pause/Resume");
    }
    return flag;
}

/*****  Change Play Pause  *****/
int invert_playpause (AD)
Audio_Device AD;
{
    if (AD.pause)
      return set_playpause (False);
    else
      return set_playpause (True);
}

/*****  Set Record Pause  *****/
int set_recpause (flag)
int flag;
{
    audio_info_t ACnew;
    Audio_Device newPlay;
    Audio_Device newRecord;
    Audio_Gain   newGain;

    if ( flag && (RPFD < 0)) { /** If audio dev not open, open for pause  **/
      GetAudioConfiguration (&newPlay, &newRecord, &newGain);
      if ((RPFD = open (AUDIO_IODEV, O_RDONLY | O_NDELAY)) > 0)
        SetAudioConfiguration (RPFD, newPlay, newRecord, newGain);
    }

    AUDIO_INITINFO (&ACnew);
    ACnew.record.pause = flag;
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : Record ", "Pause/Resume");
    }
    return flag;
}

/*****  Change Record Pause  *****/
int invert_recpause (AD)
Audio_Device AD;
{
    if (AD.pause)
      return set_recpause (False);
    else
      return set_recpause (True);
}

/*****  Set Play Gain  (0-100)*****/
int set_playgain (gain)
int gain;
{
    audio_info_t  ACnew;

    AUDIO_INITINFO (&ACnew);
    ACnew.play.gain = scale_gain ((unsigned) gain); /* 100 -> 255 */
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : ", "Set play volume.");
    }
    return gain;
}

/*****  Set Record Gain  (0-100)*****/
int set_recordgain (gain)
int gain;
{
    audio_info_t  ACnew;

    AUDIO_INITINFO (&ACnew);
    ACnew.record.gain = scale_gain ((unsigned) gain) - 1; /* 100 ->255 */
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : ", "Set record volume.");
    }
    return gain;
}

/*****  Set Monitor Gain  (0-100)*****/
int set_monitorgain(gain)
int gain;
{
    audio_info_t  ACnew;

    AUDIO_INITINFO (&ACnew);
    ACnew.monitor_gain = scale_gain ((unsigned) gain);  /* 100 -> 255 */
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : ", "Set monitor volume.");
    }
    return gain;
}

/*****  Set Play Balance  (0-100)*****/
int set_playbalance (balance)
int balance;
{
    audio_info_t  ACnew;

    AUDIO_INITINFO (&ACnew);                 /* 100 -> 64 */
#ifdef AUDIO_MID_BALANCE
    ACnew.play.balance = scale_balance (balance);
#else
    *(&ACnew.play.waiting + sizeof (char)) = scale_balance (balance);
#endif
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : ", "Set play balance.");
    }
    return balance;
}

/*****  Set Record Balance  *****/
int set_recordbalance (balance)
int balance;
{
    audio_info_t  ACnew;

    AUDIO_INITINFO (&ACnew);
#ifdef AUDIO_MID_BALANCE
    ACnew.record.balance = scale_balance (balance);
#else
    *(&ACnew.record.waiting + sizeof (char)) = scale_balance (balance);
#endif
    if (ioctl (ACFD, AUDIO_SETINFO, &ACnew) < 0) {
      Put_Message ("Error : ", "Set record balance.");
    }
    return balance;
}

/*****  Set Audio Configuration (Sample Rate, Encoding, Channels)  *****/
Boolean set_audio_config (sample_rate, encoding, precision, channels)
Cardinal sample_rate, encoding, precision, channels;
{
    audio_info_t   AI;
    int  fd = export_play_obj -> ofd, sfd, flag = True;

#ifndef LOCKCONFIG     /* Lock configuration while playing/recording */
    sfd = fd;         /* save fd */
    if (fd < 0 || ! (STATE(PRFLAG) & PLAY) )
#else
    sfd = -1;
#endif
      if ((fd = open (AUDIO_IODEV, O_WRONLY | O_NDELAY)) < 0)
        return False;

    AUDIO_INITINFO (&AI);
    AI.play.sample_rate = sample_rate;
    AI.play.encoding    = encoding;
    AI.play.precision   = precision;
    AI.play.channels    = channels;

    if (ioctl (fd, AUDIO_SETINFO, &AI) < 0) {
      Put_Message ("Can't change audio configuration.", Non);
      flag = False;
    } else {
      export_play_obj -> Dev_hdr.sample_rate = sample_rate;
      export_play_obj -> Dev_hdr.encoding    = encoding;
      export_play_obj -> Dev_hdr.channels    = channels;
      export_play_obj -> Dev_hdr.unit_size   = precision;
    }
    if (sfd != fd) 
      close (fd);
    else
      Flush_Play ();
    SetConfigMessage (export_play_obj);
    return flag;
}

/*****  Check SUN Audio Device at first  *****/
void CheckAudioDevice ()
{
    int    err, device;
    struct stat st;

    err = stat (AUDIO_IODEV, &st);
    if (err < 0) {
       Error (stderr, "cannot stat\n");
       perror (AUDIO_IODEV);
       exit (1);
    }
    if (!S_ISCHR (st.st_mode)) {
       Error (stderr, "%s is not an audio device\n", AUDIO_IODEV);
       exit (1);
    }

    if ((ACFD = open (AUDIO_CTLDEV, O_RDWR)) < 0) {
       perror (AUDIO_CTLDEV);
       exit (1);
    }

    if (ioctl (ACFD, I_SETSIG, S_MSG) < 0) {
      perror ("I_SETSIG");
      exit (1);
    }

    if (Allocate_buffer () == False) {
      perror ("Play buffer");
      exit (2);
    }

#ifdef  AUDIO_DEV_AMD
    device = AUDIO_DEV_AMD;
#else
    device = 1;
#endif
#ifdef  AUDIO_GETDEV
    if (ioctl (ACFD, AUDIO_GETDEV, &device) < 0) {
      perror ("AUDIO_GETDEV");
    }
#ifdef  AUDIO_DEV_AMD
    CHSTATE(PROCESS, (device == AUDIO_DEV_AMD ? AMD : DBRI));
#else
    CHSTATE(PROCESS, (device == 1 ? AMD : DBRI));
#endif
#else
    CHSTATE(PROCESS, 1);
#endif
    Push_Configuration ();

    CHSTATE(DEVENDIAN, BIGENDIAN);  /** Set Device Endian **/
}

/*****  audio_prinfo_t -> Audio_Device  *****/
Audio_Device To_Audio_Device (AC)
audio_prinfo_t AC;
{
    Audio_Device AD;

    AD.sample_rate = AC.sample_rate;
    AD.channel     = AC.channels;
    AD.bits        = AC.precision;
    AD.encoding    = AC.encoding;
    switch (AC.encoding) {
      case AUDIO_ENCODING_ULAW  : AD.encoding = MULAW8;      break;
      case AUDIO_ENCODING_ALAW  : AD.encoding = ALAW8;       break;
      case AUDIO_ENCODING_LINEAR: AD.encoding = PCM16;       break;
      case AUDIO_ENCODING_NONE  : 
      default                   : AD.encoding = UNKNOWNENC;  break;
    };
    AD.port        = AC.port;
    AD.port_num    = AC.avail_ports;
    AD.pause       = AC.pause;
    AD.samples     = AC.samples;
    AD.error       = AC.error;
    AD.waiting     = AC.waiting;
    return AD;
}

/*****  Audio_Device -> audio_prinfo_t  *****/
audio_prinfo_t To_audio_prinfo_t (AD)
Audio_Device AD;
{
    audio_prinfo_t AC;

    memset (&AC, 0, sizeof (audio_prinfo_t));
    AC.sample_rate   = AD.sample_rate;
    AC.encoding      = AD.encoding;
    switch (AD.encoding) {
      case MULAW8: AC.encoding = AUDIO_ENCODING_ULAW;   break;
      case ALAW8 : AC.encoding = AUDIO_ENCODING_ALAW;   break;
      case PCM8  :
      case PCM16 :
      case PCM32 : AC.encoding = AUDIO_ENCODING_LINEAR; break;
      default    : AC.encoding = AUDIO_ENCODING_NONE;   break;
    };
    AC.precision     = AD.bits;
    AC.channels      = AD.channel;
    AC.port          = AD.port;
    AC.avail_ports   = AD.port_num;
    AC.pause         = AD.pause;
    AC.samples       = AD.samples;
    AC.error         = AD.error;
    AC.waiting       = AD.waiting;
    return AC;
}

/*****  convert audio_prinfo to File_obj  *****/
File_obj To_File_obj (arp)
audio_prinfo_t arp;
{
    File_obj Dev_hdr;

    Dev_hdr.unit_size = arp.precision / 8;
    Dev_hdr.samples_per_unit = 1;
    switch (arp.encoding) {
      case AUDIO_ENCODING_ULAW  : Dev_hdr.encoding = MULAW8;      break;
      case AUDIO_ENCODING_ALAW  : Dev_hdr.encoding = ALAW8;       break;
      case AUDIO_ENCODING_LINEAR: Dev_hdr.encoding = PCM16;       break;
      case AUDIO_ENCODING_NONE  : 
      default                   : Dev_hdr.encoding = UNKNOWNENC;  break;
    };
    Dev_hdr.channels = arp.channels;
    Dev_hdr.sample_rate = arp.sample_rate;
    return Dev_hdr;
}

/*****  Get Device configuration for play   ****/
int Get_Play_Configuration (fd, Dev_hdr)
int fd;
Audio_Device *Dev_hdr;
{
    audio_info_t ap;

    if (ioctl (fd, AUDIO_GETINFO, &ap) < 0) {
      Put_Message ("ioctl : ", "AUDIO_SETINFO");
      return ERROR;
    }

    *Dev_hdr = To_Audio_Device (ap.play);
    return SUCCESS;
}

/*****  Get Device configuration for record   ****/
int Get_Record_Configuration (fd, Dev_hdr)
int fd;
Audio_Device *Dev_hdr;
{
    audio_info_t ap;

    if (ioctl (fd, AUDIO_GETINFO, &ap) < 0) {
      Put_Message ("ioctl : ", "AUDIO_SETINFO");
      return ERROR;
    }

    *Dev_hdr = To_Audio_Device (ap.record);
    return SUCCESS;
}

/*****  Get current audio state  *****/
void GetAudioConfiguration (Play_Dev, Record_Dev, Gain_Dev)
Audio_Device *Play_Dev;
Audio_Device *Record_Dev;
Audio_Gain   *Gain_Dev;
{
    audio_info_t ap;

    if (ioctl (ACFD, AUDIO_GETINFO, &ap) < 0) {
      Put_Message ("ioctl : ", "AUDIO_SETINFO");
    }

    *Play_Dev   = To_Audio_Device (ap.play);
    *Record_Dev = To_Audio_Device (ap.record);

    Gain_Dev -> play_gain      = unscale_gain (ap.play.gain);
    Gain_Dev -> record_gain    = unscale_gain (ap.record.gain);
    Gain_Dev -> monitor_gain   = unscale_gain (ap.monitor_gain);
#ifdef AUDIO_MID_BALANCE
    Gain_Dev -> play_balance   = unscale_balance (ap.play.balance);
    Gain_Dev -> record_balance = unscale_balance (ap.record.balance);
    Gain_Dev -> mute           = ap.output_muted;
#else
    Gain_Dev -> play_balance   = 
            unscale_balance (*(&ap.play.waiting + sizeof (char)));
    Gain_Dev -> record_balance = 
            unscale_balance (*(&ap.record.waiting + sizeof (char)));
    Gain_Dev -> mute           = *(&ap.monitor_gain + sizeof(int));
#endif

}

/******  Set Audio Configuration  ******/
Boolean SetAudioConfiguration (fd, newPlay, newRecord, newGain)
int fd;
Audio_Device newPlay;
Audio_Device newRecord;
Audio_Gain   newGain;
{
    audio_info_t AI;

    AUDIO_INITINFO (&AI);

    AI.play   = To_audio_prinfo_t (newPlay);
    AI.record = To_audio_prinfo_t (newRecord);

    AI.play.gain      = scale_gain ((Cardinal) newGain.play_gain);
                                          /***  adjust discrete value ! ***/
    AI.record.gain    = scale_gain ((Cardinal) newGain.record_gain) - 1;
    AI.monitor_gain   = scale_gain ((Cardinal) newGain.monitor_gain);
#ifdef AUDIO_MID_BALANCE
    AI.play.balance   = scale_balance ((Cardinal) newGain.play_balance);
    AI.record.balance = scale_balance ((Cardinal) newGain.record_balance);
    AI.output_muted   = newGain.mute;
#else
    *(&AI.play.waiting + sizeof (char)) = 
                           scale_balance ((Cardinal) newGain.play_balance);
    *(&AI.record.waiting + sizeof (char)) =
                           scale_balance ((Cardinal) newGain.record_balance);
    *(&AI.monitor_gain + sizeof(int));   = newGain.mute;
#endif
    if (ioctl (fd, AUDIO_SETINFO, &AI) < 0) {
      Put_Message ("ioctl : ", "AUDIO_SETINFO");
      return False;
    }
    return True;
}

/******  Store Audio Configuration  ******/
void Push_Configuration ()
{
    AUDIO_INITINFO (&CONFIGSAVE);
    if (ioctl (ACFD, AUDIO_GETINFO, &CONFIGSAVE) < 0) {
      perror ("AUDIO_GETINFO");
    }
}

/******  Restore Audio Configuration  ******/
void Pop_Configuration ()
{
    int fd;

    fd = open (AUDIO_IODEV, O_WRONLY | O_NDELAY);
    if (fd > 0){
      if (ioctl (fd, AUDIO_SETINFO, &CONFIGSAVE) < 0) {
        perror ("AUDIO_SETINFO");
      }
      Close (fd);
    }
}    

/*****  Close Play or Record FD  *****/
void Close_Pause_FD (TYPE)
int TYPE;
{
    if (TYPE == PLAY)
      {Close (PPFD);}
    else
      {Close (RPFD);}
}

/*****  Adjust play Volume  *****/
void Adjust_Volume (scale)
int scale;
{
    Audio_Device newPlay;
    Audio_Device newRecord;
    Audio_Gain   newGain;
    unsigned gain;
    int step = (STATE(PROCESS) == DBRI ? 1 : 1);

    GetAudioConfiguration (&newPlay, &newRecord, &newGain);

    gain_obj.save_gain = newGain.play_gain;
    gain = (int)(newGain.play_gain * scale / 100.0 / step) * step;
    gain_obj.scale = (newGain.play_gain != 0 ? 
                     (gain * 100) / newGain.play_gain : scale);
    gain = (gain > MAX_GAIN ? MAX_GAIN : gain);

    newGain.play_gain = gain;
    set_playgain (gain);
    gain_obj.adjust_gain = newGain.play_gain;
}

/*****  Reset play Volume  *****/
void Reset_Volume ()
{
    Audio_Device newPlay;
    Audio_Device newRecord;
    Audio_Gain   newGain;
    int gain;

    if (gain_obj.scale < 0) return;

    GetAudioConfiguration (&newPlay, &newRecord, &newGain);

    if (gain_obj.scale == 0)
      if (newGain.play_gain == 0)
        gain = gain_obj.save_gain;
      else
        gain = newGain.play_gain;
    else 
      if (newGain.play_gain == gain_obj.adjust_gain)
        gain = gain_obj.save_gain;
      else
        gain = newGain.play_gain * 100.0 / gain_obj.scale;
    gain = (gain > MAX_GAIN ? MAX_GAIN : gain);

    newGain.play_gain = gain;
    set_playgain (gain);
    gain_obj.scale = -1;
}

/*****  Change Configuration  *****/
Boolean ChangeConfiguration (play_obj, AD)
Audio_obj *play_obj;
Audio_Device  AD;
{
    Audio_Device  newPlay;
    Audio_Device  newRecord;
    Audio_Gain    newGain;
    Audio_Device  CHAD;
    int           encoding, abs, fd = play_obj -> ofd;

/***  Initialize  ***/
    GetAudioConfiguration (&newPlay, &newRecord, &newGain);
    newPlay   = AD;
    abs       = AD.sample_rate;
    newRecord = newPlay;

/***  Set Configuration  ***/
    if (SetAudioConfiguration (fd, newPlay, newRecord, newGain) == False) {
    /***  Find the nearest sample rate  ex. 8012 -> 8000  ***/
      newPlay.sample_rate = abs = about_sample (AD.sample_rate);
      newRecord           = newPlay;
      SetAudioConfiguration (fd, newPlay, newRecord, newGain);
    }

/***  Check device configuration Again  ***/
    if (Get_Play_Configuration (play_obj -> ofd, &CHAD) != AUDIO_SUCCESS) {
      Put_Message (AUDIO_IODEV, " is not an audio device. ");
      return False;
    }

    if ((CHAD.sample_rate != abs) || (CHAD.encoding != AD.encoding)) {
      Put_Message (" Convert ulaw encoding. ", Non);
/***  Linear -> ulaw convert playing  ***/
    }

    play_obj -> Dev_hdr = Audio_Device_to_File_obj (CHAD);

    if (device_drain (play_obj -> ofd, False) != SUCCESS) {
      Put_Message ("Error:", " AUDIO_DRAIN ");
      return False;
    }
    return True;
}

/***** Convert my file header to Sun Device Header  *****/
Audio_hdr Convert_Sun_Header (myheader)
File_obj myheader;
{
    Audio_hdr Test_hdr;

    Test_hdr.sample_rate      = myheader.sample_rate;
    Test_hdr.samples_per_unit = myheader.samples_per_unit;
    Test_hdr.bytes_per_unit   = myheader.unit_size;
    Test_hdr.channels         = myheader.channels;
    Test_hdr.encoding         = myheader.encoding;
    Test_hdr.data_size        = myheader.data_size;
    return Test_hdr;
}

/***** Convert Sun Device Header to my File header  *****/
File_obj Convert_File_Header (sunheader)
Audio_hdr sunheader;
{
    File_obj My_hdr;

    My_hdr.sample_rate      = sunheader.sample_rate;
    My_hdr.samples_per_unit = sunheader.samples_per_unit;
    My_hdr.unit_size        = sunheader.bytes_per_unit;
    My_hdr.channels         = sunheader.channels;
    My_hdr.encoding         = sunheader.encoding;
    My_hdr.data_size        = sunheader.data_size;
    return My_hdr;
}

/***** Compare Device Configuration  ******/
int Compare_Device_Configuration (AD, Test_AD)
Audio_Device AD, Test_AD;
{
    int flag = 0;

    if (AD.sample_rate != Test_AD.sample_rate) flag  = 1;
    if (AD.channel     != Test_AD.channel)     flag |= 2;
    if (AD.bits        != Test_AD.bits)        flag |= 4;
    if (AD.encoding    != Test_AD.encoding)    flag |= 8;
    return flag;
}

/*****  Convert File_obj type to Audio_Device type  *****/
Audio_Device File_obj_to_Audio_Device (FO)
File_obj FO;
{
    Audio_Device AD;

    memset (&AD, ~0, sizeof(Audio_Device));  /* clear others */
    AD.sample_rate = FO.sample_rate;
    AD.channel     = FO.channels;
    AD.bits        = FO.unit_size * 8;
    AD.encoding    = FO.encoding;
/*
    switch (FO.encoding) {
      case MULAW8    : AD.encoding = AUDIO_ENCODING_ULAW;   break;
      case ALAW8     : AD.encoding = AUDIO_ENCODING_ALAW;   break;
      case PCM8      :
      case PCM16     :
      case PCM24     :
      case PCM32     : AD.encoding = AUDIO_ENCODING_LINEAR; break;
      case UNKNOWNENC:
      default        : AD.encoding = AUDIO_ENCODING_NONE;   break;
    }
*/
    return AD;
}

/*****  Convert Audio_Device type to File_obj type  *****/
File_obj Audio_Device_to_File_obj (AD)
Audio_Device AD;
{
    File_obj FO;

    FO.sample_rate = AD.sample_rate;
    FO.channels    = AD.channel;
    FO.unit_size   = AD.bits / 8;
    FO.encoding    = AD.encoding;
    return FO;
}

/***** Set Device Configuration *****/
Boolean SetDeviceConfiguration (play_obj, AD)
Audio_obj    *play_obj;
Audio_Device AD;
{
    Audio_Device Test_AD;

    Test_AD = File_obj_to_Audio_Device (play_obj -> File_hdr);

    if (STATE(STEREODEL) > 0 && STATE(PROCESS) == DBRI &&
        STATE(EFFECT) & CHANNEL){
/*** Try to set semi stereo mode ***/
      Test_AD.channel      = STEREO;
      if (Test_AD.encoding == MULAW8 || True) {
        Test_AD.encoding   = PCM16;
        Test_AD.bits       = 16;
      }
    } 

    play_obj -> Dev_hdr = Audio_Device_to_File_obj (AD);

    if (Compare_Device_Configuration (AD, Test_AD) != 0) {
/*
      play_obj -> Dev_hdr.sample_rate = Dummy_AD.sample_rate;
      play_obj -> Dev_hdr.sample_rate = Dummy_AD.channels;
*/
      if (ChangeConfiguration (play_obj, Test_AD) == False)
        return False;
    }

    play_obj -> bytes_per_units = play_obj -> File_hdr.unit_size * 
                                  play_obj -> File_hdr.channels;
    return True;
}

/*****  Flush output  *****/
void flush_play (fd)
int fd;
{
    if (ioctl (fd, I_FLUSH, FLUSHW) < 0) {
      Put_Message ("ioctl : ", "Flush play");
    }
}

/*****  Flush input  *****/
void flush_record (fd)
int fd;
{
    if (ioctl (fd, I_FLUSH, FLUSHR) < 0) {
      Put_Message ("ioctl : ", "Flush record");
    }
}

/*****  Get Record Error  *****/
int get_record_error (fd, err)
int fd, *err;
{
    audio_info_t AI;

    if (ioctl (fd, AUDIO_GETINFO, &AI) < 0) {
      Put_Message ("ioctl : ", "AUDIO_SETINFO");
      return False;
    }
    *err = AI.record.error;
    return (AI.record.error == 0 ? SUCCESS : *err);
}

/**** Audio Device Drain  *****/
int device_drain (fd, flag)
{
    if (fd < 0) return ERROR;
    if (ioctl (fd, AUDIO_DRAIN, 0) < 0) {
      Put_Message ("Error:  ioctl ", "AUDIO_DRAIN");
      return ERROR;
    }
    return SUCCESS;
}
