/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    WAVfile.c (WAV audio file read/write)
 *
 * Date:    Jan, 4, 1994
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#define NOX
#include "xplay.h"

#define MAGIC1  ((unsigned long)0x52494646)
#define MAGIC2  ((unsigned long)0x57415645)
#define MAGIC3  ((unsigned long)0x666d7420)
#define MAGIC4  ((unsigned long)0x64617461)

int WAV_write_filehdr ();
int WAV_rewrite_filesize ();
Boolean  WAV_read_filehdr ();

/*****  Write File Header for SUN and My header  *****/
int WAV_write_filehdr (rec_obj)
Audio_obj *rec_obj;
{
    int       err = False, fd = rec_obj -> ofd;
    char      buff[BUFSIZ];

    write (fd, "RIFF", 4);
    write_longlittle (rec_obj -> rw_size + 36, fd);
    write (fd, "WAVEfmt ", 8);
    write_longlittle ((long) 16, fd);
    write_shortlittle ((short)1, fd);
    write_shortlittle ((short)rec_obj -> Dev_hdr.channels, fd);
    write_longlittle ((long)rec_obj -> Dev_hdr.sample_rate, fd);
    write_longlittle ((long)(rec_obj -> Dev_hdr.sample_rate *
                              rec_obj -> Dev_hdr.channels *
                              rec_obj -> Dev_hdr.unit_size * 8 + 7) / 8, fd);
    write_shortlittle ((short)(rec_obj -> Dev_hdr.unit_size * 8 + 7) / 8, fd);
    write_shortlittle ((short)(rec_obj -> Dev_hdr.unit_size * 8), fd);
    write (fd, "data", 4);
    write_longlittle ((long)rec_obj -> rw_size, fd);

    rec_obj -> suffix = WAV;
    rec_obj -> File_hdr.endian = SMALLENDIAN;
    if (rec_obj -> Dev_hdr.encoding == MULAW8) {
      rec_obj -> File_hdr.encoding =  PCM8;
      rec_obj -> File_hdr.unit_size = 1;
    } else {
      rec_obj -> File_hdr.encoding =  rec_obj -> Dev_hdr.encoding;
      rec_obj -> File_hdr.unit_size = rec_obj -> Dev_hdr.unit_size;
    }

    memset (buff, 0, sizeof (buff));
    sprintf (buff, "xplay %c", (unsigned char)rec_obj -> volume_scale);
    return write (fd, buff, EXTRAHEADERSIZE);
/*
 *  return Extra header size, not true header size.
 */
}

/*****  Rewrite filesize for WAV  *****/
int WAV_rewrite_filesize (fd, rw_size)
int fd;
unsigned long rw_size;
{
    lseek (fd, sizeof(long), SEEK_SET);
    write_longlittle (rw_size + 36, fd);
    lseek (fd, sizeof(long)*10L, 0);
    return write_longlittle (rw_size, fd);
}

/*** Read File Header for WAV ***/
Boolean WAV_read_filehdr (play_obj)
Audio_obj *play_obj;
{
    int       err = False, len, bits_per_unit;
    int       bytes_per_unit, encoding;
    char      buff[BUFSIZ];
    unsigned  long HEADER[6], samples;
    short     shorttype;
    int       fd = play_obj -> ifd;

    if (fd < 3)
      err = True;
    else {
      if (read (fd, HEADER, sizeof(long) * 4) != 4 * sizeof(long)) err = True;
      if (MAGIC1 != HEADER[0])                      err = True;
      if (MAGIC2 != HEADER[2])                      err = True;
      if (MAGIC3 != HEADER[3])                      err = True;
    }

    if (err == True) {
      Put_Message (CFILENAME(), " is not a valid WAV audio file.");
      return False;
    }

    len = read_longlittle (fd);
    shorttype = read_shortlittle (fd);
    if (shorttype != 1)        err = True;

    play_obj -> File_hdr.channels    = read_shortlittle (fd);
    play_obj -> File_hdr.sample_rate = read_longlittle (fd);
    read_longlittle (fd);
    read_shortlittle (fd);
    bits_per_unit   = read_shortlittle (fd);
    len -= 16;
    if (len) read (fd, buff, len);
    read (fd, HEADER, sizeof(long));
  
    if (HEADER[0] != MAGIC4){
      Put_Message (CFILENAME(), " is not a valid WAV audio file.");
      return False;
    }

    samples = read_longlittle (fd);

    play_obj -> header_size = tell (fd);
    play_obj -> File_hdr.header_size = play_obj -> header_size;

/*** read volume scale for adjust volume  ***/
    play_obj -> volume_scale = 100;
    if (STATE(ADJUST)) {
       int volume_scale = 100;

       memset (buff, 0, sizeof (buff));
       if (read (fd, buff, EXTRAHEADERSIZE) != EXTRAHEADERSIZE) {
         /***  Is this audio file ?  ***/
         Put_Message (CFILENAME()," has not enough data.");
         return False;
       }
       if (memcmp (buff, XPLAYID, strlen (XPLAYID)) == 0) {
         /***  Found XPlay MAGIC  ***/
         volume_scale = (unsigned char) buff[strlen (XPLAYID)];
       }
       play_obj -> volume_scale = volume_scale;
    }

/*
 *  Change WAV file encoding define to 
 *  my encoding define (See xplay.h)  
 */
    switch (bits_per_unit) {
      case 8 : encoding = PCM8;  bytes_per_unit = 1; break;
      case 16: encoding = PCM16; bytes_per_unit = 2; break;
      case 32: encoding = PCM32; bytes_per_unit = 4; break;
      default: encoding = UNKNOWNENC; bytes_per_unit = 0;
               break;
    };
    play_obj -> File_hdr.encoding    = encoding;
    play_obj -> File_hdr.unit_size   = bytes_per_unit;
    play_obj -> File_hdr.samples_per_unit = 1;
    play_obj -> File_hdr.magicid     = WAV;
    play_obj -> File_hdr.data_size   = samples ;
    play_obj -> bytes_per_units = play_obj -> File_hdr.channels *
                                  play_obj -> File_hdr.unit_size;

    play_obj -> File_hdr.endian = SMALLENDIAN;
    play_obj -> suffix          = WAV;
    return True;
}

