/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    audio.c   (play, record)
 *
 * Date:    Dec, 12, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#define NOX
#include "xplay.h"
#include "menu.h"

char   *fbuf  = NULL, *fbuf1 = NULL;          /*  buffer at playing      */
char   *fbuf2 = NULL, *fbuf3 = NULL;          /*  buffer at playing      */
char   *rbuf  = NULL;                         /*  buffer at recording    */
static Audio_obj  play_obj, record_obj;       /*  play and record object */

Audio_obj *export_play_obj = &play_obj;        /*  Export play_obj        */

/*
 *  Prototypes
 */

unsigned scale_gain ();
unsigned unscale_gain ();
unsigned scale_balance ();
unsigned unscale_balance ();
int      audio_rewrite_filehdr ();
int      audio_reload_filehdr ();
int      LoadOpen ();
void     PlayMain ();
void     Playing ();
int      SaveOpen ();
void     RecordMain ();
void     Recording ();
void     FlushInPut ();
void     FlushOutPut ();
void     Flush_Play ();
void     Push_Volume ();
void     Pop_Volume ();
void     Reset_Volume ();
int      Play_Seconds ();
void     Free_buffer ();
Boolean  Allocate_buffer ();
/**********************/
extern   int STATE();
extern   int CHSTATE();
extern   char * STRINGS();
extern   Boolean  SetAudioConfiguration ();
extern   int EnSlow ();
extern   int EnFast ();
extern   int Scale_data ();
extern   File_obj Audio_Device_to_File_obj ();

/*****  Audio Rewrite file header  *****/
int audio_rewrite_filehdr (fd, volume)
int fd;
int volume;
{
    char buff[255];

    memset (buff, 0, sizeof (buff));
    sprintf (buff, "xplay %c", (unsigned char)volume);
    return write (fd, buff, EXTRAHEADERSIZE);
}

/*****  Audio Reload file header  *****/
int audio_reload_filehdr (fd, volume_scale)
int fd;
int *volume_scale;
{
    char buff[255];

    memset (buff, 0, sizeof (buff));
    if (read (fd, buff, EXTRAHEADERSIZE) != EXTRAHEADERSIZE) return -1;
    if (memcmp (buff, XPLAYID, strlen (XPLAYID)) == 0) {
      *volume_scale = (unsigned char) buff[strlen (XPLAYID)];
    }
    return 1;
}

/*****  Convert local gain into device parameters  *****/
unsigned scale_gain (g) 
unsigned g;
{
    return (AUDIO_MIN_GAIN + (unsigned)
        irint(((double) (AUDIO_MAX_GAIN - AUDIO_MIN_GAIN)) *
        ((double)g / (double)MAX_GAIN)));
}

/*****  Convert device gain into the local scaling factor  *****/
unsigned unscale_gain (g)
unsigned g;
{
    return ((unsigned) irint((double)MAX_GAIN *
           (((double)(g - AUDIO_MIN_GAIN)) /
           (double)(AUDIO_MAX_GAIN - AUDIO_MIN_GAIN))));
}

/*****  Convert local balance into device parameters  *****/
unsigned scale_balance (g) 
unsigned g;
{
    return (AUDIO_LEFT_BALANCE + (unsigned)
        irint(((double) (AUDIO_RIGHT_BALANCE - AUDIO_LEFT_BALANCE)) *
        ((double)g / (double)MAX_GAIN)));
}

/*****  Convert device balance into the local scaling factor  *****/
unsigned unscale_balance (g)
unsigned g;
{
    return ((unsigned) irint((double)MAX_GAIN *
           (((double)(g - AUDIO_LEFT_BALANCE)) /
           (double)(AUDIO_RIGHT_BALANCE - AUDIO_LEFT_BALANCE))));
}

/*****  Set Direction  *****/
int SetDirection (direction)
int direction;
{
    Flush_Play ();
    return CHSTATE(DIRECTI, (play_obj.direction = direction));
}

/*****************************/
/****  Audio file   open  ****/
/****  Audio device open  ****/
/*****************************/
int LoadOpen ()
{
    char          buff [BUFSIZ];
    int           err, AOFD, FIFD, volume;
    Audio_Device  Dev_hdr;                /*  audio header for device */

/***  Initialize  ***/

    play_obj.samples = play_obj.file_position = 0;
    CHSTATE(PLAYP, 0);
    sprintf (buff,"%s", CPATHNAME());

    strcpy (play_obj.filename, buff);

/***  AUDIO device open  ***/
    AOFD = play_obj.ofd = open (AUDIO_IODEV, O_WRONLY | O_NDELAY);

    if (AOFD < 0)
      if (errno == EBUSY) {
        Put_Message (" Audio Device Busy. ", Non);
        return False;
      } else {
        Put_Message (" Error: Audio device Opening. ?" , Non);
        return False;
      }

    if (Get_Play_Configuration (AOFD, &Dev_hdr) != SUCCESS) {
      Put_Message (AUDIO_IODEV, " is not an audio device.");
      return False;
    }

/***  AUDIO file open  ***/
    if ((FIFD = open (buff, O_RDONLY)) == -1) {
      Put_Message (" Can't Open File : ", CFILENAME());
      return False;
    }

    play_obj.ifd = FIFD;

    if (GetFileConfiguration (&play_obj) == False) {
      return False;
    }

    if (SetDeviceConfiguration (&play_obj, Dev_hdr) == False) {
      return False;
    }

    if (play_obj.volume_scale != 100)
      Adjust_Volume (play_obj.volume_scale);

/***  For backward playing  ***/
    play_obj.direction = STATE(DIRECTI);

    if (play_obj.direction == BACKWARD) {
      unsigned long test = lseek (FIFD, 0, SEEK_END);

      if (test < 0) {
        Put_Message (" Error: seek error", "Can't play backward");
        return False;
      }
      test -= test % 4;
      play_obj.file_position = test;
    } else {
      unsigned long test;
      test = lseek (FIFD, 0, SEEK_CUR);
      if (test < 0) {
        Put_Message (" Error: seek error", "Can't play forward");
        return False;
      }
      test -=test % 4;
      play_obj.file_position = test;
    }

/***  Set buffer size relation  ***/
    {
      int test;
      test = play_obj.File_hdr.sample_rate * play_obj.bytes_per_units * 2;
      test = (STEPBAR / 4.0 * test) / 20;
      test *= 8 * play_obj.File_hdr.samples_per_unit;
      play_obj.bufsize = test;
    }
/*
    if (Allocate_buffer () == False) {  
      Put_Message (" Error: alloc", "Can't allocate memory.");
      return False;
    }
*/
    return True;
}

/*****  return true if audio buffer is empty  *****/
Boolean CheckAudioBuffer (AD)
Audio_Device AD;
{
    int diff;
    static int counter = 0;

    diff = AD.samples - play_obj.samples;
    diff = (diff < 0 ? -diff : diff);
    counter ++;
    if (diff > AD.sample_rate && counter < 50) return False;
    counter = 0;
    return True;
}

/*****  Play Main  *****/
void PlayMain (AD)
Audio_Device AD;
{
    if (( STATE(PAUSEF) & PLAY ) && STATE(PSTATE) <= DOPLAY) return;

    switch (STATE(PSTATE)) {
      case NOPLAY   :
           InitAudioObj (&play_obj);
           if (LoadOpen () == True) {

             CHSTATE(PSTATE, DOPLAY);
             Put_Message (" Playing  ", CFILENAME());
             Put_Info (&play_obj);
             Put_File_Config (&play_obj);
             SetConfigMessage (&play_obj);
           } else {
            CHSTATE(PSTATE, ENDPLAY);
           }
           Reset_List ();
           Set_View_Location ();
           break;
      case DOPLAY   :
           if (CheckAudioBuffer (AD)) {
             Playing (AD);
           }
           break;
      case ENDPLAY  :{
           int indexes;
           device_drain (play_obj.ofd, False);
           FlushOutPut ();
           if (STATE(ADJUST))
             Reset_Volume ();
           indexes = Forward_List ();
           if (indexes >= 0) {
             CHSTATE(PSTATE, NOPLAY);
             break;
           } else
             if (STATE(LOOPING)) {
               if (Rewind_List() >= 0){
                 CHSTATE(PSTATE, NOPLAY);
                 break;
               }
           } else {
             Rewind_List();
             Reset_List ();
             Set_View_Location ();
           }
           }
      case PLAYERROR:
           stop_proc (0, 0, 0);
           CHSTATE(PSTATE, NOPLAY);
           break;
    }
}

/*****  move Play file pointer  *****/
void move_file_pointer (lab)
int lab;
{
    play_obj.file_position  = lab * play_obj.File_hdr.data_size / MAX_GAIN;
    play_obj.file_position -= play_obj.file_position % 4;
    play_obj.file_position += play_obj.header_size;

    lseek (play_obj.ifd, play_obj.file_position , SEEK_SET);
    Flush_Play ();
}

/*****  Set Read Data Size  *****/
int SetReadDataSize (scale)
int scale;
{
    int fs = 0;

    if (scale == 0) 
      fs = (play_obj.bufsize / STEPBAR); 
    else
      if (scale > 0)  
        fs = (float)(scale + 2) / 2.0 * play_obj.bufsize / STEPBAR;
      else
        fs = 2.0 / (float)((-scale) + 2) * play_obj.bufsize / STEPBAR;
    fs += (fs % play_obj.bytes_per_units);
    return fs;
}

/*****  Read Data From File  *****/
int ReadData (temp, fs)  /* fs = read data size */
char *temp;
int fs;
{
    int cnt = 1;

    if (play_obj.direction == BACKWARD)
      cnt = lseek (play_obj.ifd, play_obj.file_position - fs, SEEK_SET);

    if (cnt > 0)
      cnt = read (play_obj.ifd, (char *)temp, fs);

    if (cnt <= 0) {
      CHSTATE(PSTATE, ENDPLAY);
      return cnt;
    }

    play_obj.file_position = tell (play_obj.ifd) - 
                            (play_obj.direction == BACKWARD ? fs : 0);

    play_obj.file_position -= (play_obj.file_position - play_obj.header_size) %
                               play_obj.bytes_per_units;

    CHSTATE(PLAYP, (int)(play_obj.file_position * MAX_GAIN /
                    play_obj.File_hdr.data_size));

    return cnt;
}

/*****  Play speed scale  *****/
int GetScale(le)
int le;
{
    if (le < 12) return -4;
    if (le < 23) return -3;
    if (le < 34) return -2;
    if (le < 45) return -1;
    if (le < 57) return  0;
    if (le < 68) return  1;
    if (le < 79) return  2;
    if (le < 90) return  3;
    return  4;
}

/*****  Read Audio File and Write Audio Device  *****/
void Playing (AD)
Audio_Device  AD;
{
    int  cnt, err, i, scale, fs;
    char *temp;

    scale = GetScale (STATE(PLAYS));
    fs    = SetReadDataSize (scale);

    if ((cnt = ReadData (fbuf, fs)) <= 0)
      return;

    if (STATE(DEVENDIAN) != play_obj.File_hdr.endian) {
      cnt = Endian (&temp, fbuf, cnt, play_obj.File_hdr.unit_size);
      memcpy (fbuf, temp, cnt);
    }

    if (play_obj.direction == BACKWARD){
      cnt = err = Backward_Data (&temp, fbuf, cnt, play_obj.bytes_per_units);
      memcpy (fbuf, temp, err);
    }

    cnt = Scale_data (&temp, fbuf, cnt, scale, play_obj.bytes_per_units);

    if (STATE(PROCESS) == DBRI) {
      cnt = LinearPCM_Converter (&temp, temp, cnt, play_obj, AD);
    } else
    if (STATE(PROCESS) == AMD){
      cnt = Ulaw_Converter (&temp, temp, cnt, play_obj);
    }
    play_obj.samples += cnt / (AD.bits / 8 * AD.channel);
    err = write (play_obj.ofd, (char *)temp, cnt);

    if (err != cnt) {
      Put_Message ("Output error.", Non);
      CHSTATE(PSTATE, PLAYERROR);
    }

    return;
}

/************************************
 *****  Recording Procedures    *****
 ************************************/

/*****  Open Audio file for save and Audio Device for input  *****/
int SaveOpen ()
{
    Audio_Device  newPlay;
    Audio_Device  newRecord;
    Audio_Gain    newGain;
    char          buf[BUFSIZ], buff[BUFSIZ], infosize[20];
    String        Info;
    struct stat   st;
    int           err, volume_scale = 100, AIFD, FOFD;
    Audio_Device  Dev_rhdr;                /*  audio header for device */
    Audio_obj     Audio_Dev;

    sprintf (buff, "%s/%s", STRINGS(RECDIR), STRINGS(RECFILE));
    OptimizeDirname (buf, buff);
    strcpy (record_obj.filename, buf);
    record_obj.rw_size = 0;
    GetAudioConfiguration (&newPlay, &newRecord, &newGain);

    err = stat (AUDIO_IODEV, &st);
    if (err < 0) {
      Put_Message (" Cannot open ", AUDIO_IODEV);
      return False;
    }
    if (!S_ISCHR (st.st_mode)) {
      Put_Message (AUDIO_IODEV, " is not an audio device. ");
      return False;
    }

    record_obj.bufsize = newRecord.sample_rate * newRecord.bits *
                         newRecord.channel;
    record_obj.bufsize /= 8;

    if ((rbuf = (char *)calloc(record_obj.bufsize, sizeof (char) * 2))==NULL) {
      Put_Message ("Can't allocate memory...", Non);
      return False;
    }
    AIFD = record_obj.ifd = open (AUDIO_IODEV, O_RDONLY | O_NDELAY);
    if (AIFD < 0) {
        if (errno == EBUSY) {
          Put_Message (AUDIO_IODEV, " is busy. ");
        } else {
          Put_Message ("Error :", " Opening .");
        }
        return False;
    }

    /**  Set Current Audio Configuratin **/
    SetAudioConfiguration (AIFD, newPlay, newRecord, newGain);

    if (Get_Record_Configuration (AIFD, &Dev_rhdr) != SUCCESS) {
       Put_Message (AUDIO_IODEV, " is not an audio device. ");
       return False;
    }

    FOFD = record_obj.ofd = open (buf, (O_RDWR | O_CREAT | O_TRUNC), 0644);

    if (FOFD < 0) {
      Put_Message (" Cannot open ", STRINGS(RECFILE));
      return False;
    }

    sprintf (infosize, "%%.%ds", MAX_INFO);
    sprintf (record_obj.Infostr, infosize, (char *)STRINGS(INFOSTR));

    record_obj.Dev_hdr  = 
    record_obj.File_hdr = Audio_Device_to_File_obj (Dev_rhdr);
    record_obj.volume_scale = volume_scale;

    if ((record_obj.rw_size = Write_File_Header (&record_obj)) < 0) {
       Put_Message (" Error in writing header for ", STRINGS(RECFILE));
       return False;
    }

    (void) fcntl (AIFD, F_SETFL, fcntl (AIFD, F_GETFL, 0) | O_NDELAY);

    if (STATE(LIMTTIM) > 0)
      CHSTATE(LIMTSIZ, Millsecs_to_bytes (Dev_rhdr.sample_rate * 
                      Dev_rhdr.channel * Dev_rhdr.bits / 8, STATE(LIMTTIM)));

    else
      CHSTATE(LIMTSIZ, 0);
    if (STATE(LIMTSIZ) < 0) CHSTATE(LIMTSIZ, 0);
    return True;
}

/*****  Recording Trigger  *****/
int RecTrigger (buff, cnt)
char *buff;
int cnt;
{
   int i, units, bytes, trigger = 0, level, miss = 0;
   union _u_data {
     short datum;
     char linear[4];
   } u_data;

   level = STATE(TRIGL) * STATE(TRIGL) * 32767 / 10000;
   bytes = record_obj.Dev_hdr.unit_size;
   units = cnt / bytes;

   for (i = 0;i < units;i++) {
      if (record_obj.Dev_hdr.encoding == MULAW8) {
        u_data.datum = st_ulaw_to_linear_slow (buff[i * bytes]);
      } else {
        u_data.linear[0] = buff[i * bytes];
        u_data.linear[1] = buff[i * bytes + 1];
      }

      if (abs(u_data.datum) > level) {
        trigger ++;
      } else {
        miss ++;
      }
   }
   if (trigger > cnt / 10)
     return START;
   else
     return WAITING;
}

/*****  Record Main  *****/
void RecordMain (newRecord)
Audio_Device newRecord;
{
    int  err, dummy;
    char buff[100];
    String str;
    static int timec = 0;

    if (( STATE(PAUSEF) & RECORD ) && STATE(RSTATE) <= DOREC) return;
    switch (STATE(RSTATE)) {
      case NOREC    :
           InitAudioObj (&record_obj);
           if (SaveOpen () == True) {
             CHSTATE(RSTATE, DOREC);
             CHSTATE(OVERFLW, False);
             flush_record (record_obj.ifd);
             timec  = newRecord.sample_rate *
                      newRecord.bits *
                      newRecord.channel;
             timec /= 8;
             if (STATE(RECTRIGGER) == WAITING){
               Put_Trigger_Waiting ();
             }
           } else {
             CHSTATE(RSTATE, RECERROR);
           }
           break;
      case DOREC    :
           Recording (newRecord);
           if (STATE(RECTRIGGER) != WAITING){
             Put_File_Size(record_obj.rw_size, timec, newRecord.error);
           }
           break;
      case ENDREC   :
           Rewrite_File_Size (record_obj);
           if (get_record_error (record_obj.ifd, &err) != SUCCESS) {
             record_errbox ("  Error reading device status. ");
           }
            else if (err) {
                record_errbox ("  Data overflow occurred.  ");
           }
           if (STATE(OVERFLW)) {
             sprintf (buff, " Over maximum file size %d KB.", STATE(RLIMIT));
             record_errbox (buff);
           }
      case RECERROR :
           Free (rbuf);
           CHSTATE(RSTATE, NOREC);
           rec_stop_proc (NULL, 0, 0);
           break;
      case RECOVERMAX:
           CHSTATE(RSTATE, ENDREC);
           break;
    }
}

/*****  Read Audio Device and Write Audio File  *****/
void Recording (AD)
Audio_Device AD;
{
    int  cnt, err, i, bsize, silent_byte;
    char buff[80], *temp;

    bsize = record_obj.bufsize;
    if (STATE(LIMTSIZ) > 0)
      bsize = ((STATE(LIMTSIZ) - record_obj.rw_size > bsize) ? 
                             bsize : (STATE(LIMTSIZ) - record_obj.rw_size));

    for (i = 0; i < 20; i++) {
      cnt = read (record_obj.ifd, (char *)rbuf, bsize);
      if (cnt != 0) break;
    }
    if (cnt <= 0) {
      if (cnt == 0) return;
      switch (errno) {
#if !defined(SVR4)
        case EAGAIN:
#endif
        case EWOULDBLOCK: return;
        default:
                 Put_Message (" Error Reading Audio Device. ", Non);
                 CHSTATE(RSTATE, RECERROR);
                 return;
      }
    }
    if (STATE(RECTRIGGER) == WAITING)
      if (RecTrigger (rbuf, cnt) == WAITING)
        return;
      else {
        CHSTATE(RECTRIGGER, START);
        SetRecTriggerButton ();
        silent_byte = cnt * 2;
        silent_byte = (silent_byte + cnt > record_obj.bufsize ? 
                       record_obj.bufsize - cnt : silent_byte);
        memcpy (rbuf + silent_byte, rbuf, cnt);
        memset (rbuf, 0, silent_byte);
        cnt += silent_byte;
      }
    temp = rbuf;

    if (AD.encoding == MULAW8 && AD.encoding != record_obj.File_hdr.encoding) {
      cnt = Ulaw_to_Linear (&temp, rbuf, cnt);
      memcpy (rbuf, temp, cnt);
      cnt = Linear16_to_8 (&temp, rbuf, cnt);
      memcpy (rbuf, temp, cnt);
    }

    if (STATE(DEVENDIAN) != record_obj.File_hdr.endian)
      cnt = Endian (&temp ,rbuf, cnt, record_obj.File_hdr.unit_size);

    err = write (record_obj.ofd, (char *)temp, cnt);
    if (err != cnt) {
      Put_Message (" Error writing. ", Non);
      CHSTATE(RSTATE, RECERROR);
      return;
    }
    record_obj.rw_size += err;
    if (record_obj.rw_size > STATE(RLIMIT) * 1024) {
      sprintf (buff, "  %d Kbytes", STATE(RLIMIT));
      Put_Message ("Over Max Recording Size", buff);
      CHSTATE(OVERFLW, True);
      CHSTATE(RSTATE, RECOVERMAX);
    } else if (record_obj.rw_size >= STATE(LIMTSIZ) && STATE(LIMTSIZ) != 0) {
      CHSTATE(RSTATE, ENDREC);
      Put_Message ("  Recording Time Out.  ", Non);
    }
    return;
}

/*****  Seek File Pointer for search Proc  *****/
void Seek_File (type)
int type;
{
    if (type == FORWARD) {
      if (play_obj.direction == BACKWARD)
        lseek (play_obj.ifd, play_obj.bufsize + 0, SEEK_CUR);
      else
        lseek (play_obj.ifd, play_obj.bufsize, SEEK_CUR);
    } else {
      long l = tell (play_obj.ifd) - play_obj.header_size;
      if (l > play_obj.bufsize) l = play_obj.bufsize;
      if (play_obj.direction == BACKWARD)
        lseek (play_obj.ifd, -(l) + 0, SEEK_CUR);
      else
        lseek (play_obj.ifd, -(l), SEEK_CUR);
    }
    play_obj.file_position = tell (play_obj.ifd);
    Flush_Play ();
}

/*****  Flush Audio Output FD  ******/
void Flush_Play ()
{
    Audio_Device  newPlay;
    Audio_Device  newRecord;
    Audio_Gain    newGain;

    if (play_obj.ofd < 0) return;
    GetAudioConfiguration (&newPlay, &newRecord, &newGain);

    (void) flush_play (play_obj.ofd);
    if (play_obj.ifd > 0 && (STATE(PRFLAG) & PLAY)) {
      int diff  = (play_obj.samples - newPlay.samples);
      int seeks =  diff * (play_obj.File_hdr.unit_size * 
                           play_obj.File_hdr.channels);
      play_obj.file_position -= (STATE(DIRECTI) == FORWARD ? seeks : -seeks);
      lseek (play_obj.ifd, play_obj.file_position , SEEK_SET);
    }
    play_obj.Dev_hdr = Audio_Device_to_File_obj (newPlay);
    play_obj.samples = newPlay.samples;
}

/*****  Flush and Close Audio Output Descriptor  *****/
void FlushOutPut ()
{
    /* flush output queues  */
    if (play_obj.ofd > 0) {
      Flush_Play ();
      Close (play_obj.ofd);
    }
    Close (play_obj.ifd);
    CHSTATE(PLAYP, 0);
    CHSTATE(PSTATE, NOPLAY);
/*
    Free_buffer ();
*/
    if (STATE(ADJUST))
      Reset_Volume ();
}

/*****  Flush and Close Audio Input Descriptor  *****/
void FlushInPut ()
{
    /* flush input queues */
    if (record_obj.ifd > 0) {
      (void) flush_record (record_obj.ifd);
      Close (record_obj.ifd);
    }
    Close (record_obj.ofd);
    Free (rbuf);
    STATE(RSTATE, NOREC);
}

/*****  Allocate buffer for play  *****/
Boolean Allocate_buffer ()
{
    if ((fbuf  = (char *) calloc (AUDIOBUFSIZ, sizeof (char))) == NULL) {
      return False;
    }
    if ((fbuf1 = (char *) calloc (AUDIOBUFSIZ, sizeof (char))) == NULL) {
      Free (fbuf);
      return False;
    }
    if ((fbuf2 = (char *) calloc (AUDIOBUFSIZ, sizeof (char))) == NULL) {
      Free (fbuf1);
      Free (fbuf);
      return False;
    }
    if ((fbuf3 = (char *) calloc (AUDIOBUFSIZ, sizeof (char))) == NULL) {
      Free (fbuf2);
      Free (fbuf1);
      Free (fbuf);
      return False;
    }
    memset (fbuf,  0, AUDIOBUFSIZ);
    memset (fbuf1, 0, AUDIOBUFSIZ);
    memset (fbuf2, 0, AUDIOBUFSIZ);
    memset (fbuf3, 0, AUDIOBUFSIZ);
    return True;
}

/*****  Free  befor  Quits  *****/
void Free_buffer ()
{
    Free (fbuf3);
    Free (fbuf2);
    Free (fbuf1);
    Free (fbuf);
}

/*****  Return play seconds   *****/
int Play_Seconds ()
{
    int timec;

    if ((STATE(PRFLAG) & PLAY) && (STATE(PSTATE) == DOPLAY)) {
      timec  = play_obj.File_hdr.sample_rate *
               play_obj.File_hdr.unit_size *
               play_obj.File_hdr.channels;
      timec /= play_obj.File_hdr.samples_per_unit;
      return (timec == 0 ? 0 : play_obj.file_position / timec);
    }
    return -1;
}

