/***************************************
 *  Sun Audio Configuration Displayer  *
 *  Aug, 21, 1993  A. Hirahara         *
 ***************************************/

#include <errno.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <strings.h>
#include <stropts.h>
#include <dirent.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/file.h>

#include <sun/audioio.h>
#include <multimedia/audio_device.h>
#include <multimedia/ulaw2linear.h>
#include <multimedia/libaudio.h>

enum {NON, PLAY, RECORD};
int ACFD, paport = 0, raport = 0;

/******  Get Value  ******/
int get_value (min, max)
unsigned min, max;
{
     unsigned int value;
     char buff[255];

     printf ("\nInput value (%d - %d) : ", min, max);
     gets (buff);
     value = atoi (buff);
     if (value == 0)
       if (buff[0] != '0') return -1;
     if (value >= min && value <= max) return value; else return -1;
}

/******  Set Gain  ******/
void set_gain (type, value)
int  type;
char *value;
{
    int va;
    audio_info_t  AI;

    va = atoi (value);
    if (va == 0 && value[0] != '0') va = get_value (0, 255);
    if (va >= 0 && va <= 255) {
      AUDIO_INITINFO (&AI);
      if (type == PLAY)
        AI.play.gain = va;
      else
        AI.record.gain = va;
      if (ioctl (ACFD, AUDIO_SETINFO, &AI) < 0) {
        perror ("AUDIO_SETINFO");
      }
    }
}

/******  Set Stereo Balance  ******/
void set_balance (type, val)
int  type;
char *val;
{
    int value;
    audio_info_t  AI;

    value = atoi (val);
    if (value == 0 && val[0] != '0') value = get_value (0, 64);
    if (value >= 0 && value <= 64) {
      AUDIO_INITINFO (&AI);
      if (type == PLAY)
        (char)*(&AI.play.waiting + 1) = (char) value;   /** stereo balance **/
      else
        (char)*(&AI.record.waiting + 1) = (char) value; /** stereo balance **/
      if (ioctl (ACFD, AUDIO_SETINFO, &AI) < 0) {
        perror ("AUDIO_SETINFO");
      }
    }
}

/******  Set Monitor Gain  ******/
void set_monitor (val)
char *val;
{
    int value;
    audio_info_t  AI;

    value = atoi (val);
    if (value == 0 && val[0] != '0') value = get_value (0, 255);
    if (value >= 0 && value <= 255) {
      AUDIO_INITINFO (&AI);
      AI.monitor_gain = value;
      if (ioctl (ACFD, AUDIO_SETINFO, &AI) < 0) {
        perror ("AUDIO_SETINFO");
      }
    }
}

/******  Set Port  ******/
void set_port (type, val)
int  type;
char *val;
{
    int value, aport;
    audio_info_t  AI;

    if (type == PLAY) aport = paport; else aport = raport;
    value = atoi (val);
    if (value == 0 && val[0] != '0') value = get_value (0, aport);
    if (value >= 0 && value <= aport) {
      AUDIO_INITINFO (&AI);
      if (type == PLAY)
        AI.play.port = value;
      else
        AI.record.port = value;
      if (ioctl (ACFD, AUDIO_SETINFO, &AI) < 0) {
        perror ("AUDIO_SETINFO");
      }
    }
}

/******  Set Pause  ******/
void set_pause (type, value)
int type;
char *value;
{
    audio_info_t  AI;

    if (value[0] == NULL) {
      printf ("\nInput [T] ... Set Pause; Any Clear Pause > ");
      gets (value);
    }
    AUDIO_INITINFO (&AI);
    if (toupper (value[0]) == 'T')
      if (type == PLAY)
        AI.play.pause = TRUE;
      else
        AI.record.pause = TRUE;
    else
      if (type == PLAY)
        AI.play.pause = FALSE;
      else
        AI.record.pause = TRUE;
    if (ioctl (ACFD, AUDIO_SETINFO, &AI) < 0) {
      perror ("AUDIO_SETINFO");
    }
}

/******  Set Mute  ******/
void set_mute (value)
char *value;
{
    audio_info_t  AI;
    char *p;

    p = (char*)&AI.monitor_gain;
    if (value[0] == NULL) {
      printf ("\n[T] ... Mute;   Any ... Clear Mute > ");
      gets (value);
    }
    AUDIO_INITINFO (&AI);
    if (toupper (value[0]) == 'T')
      *(p + sizeof (int)) = (char) TRUE;
    else
      *(p + sizeof (int)) = (char) FALSE;
    if (ioctl (ACFD, AUDIO_SETINFO, &AI) < 0) {
      perror ("AUDIO_SETINFO");
    }
}

/******  Load Audio Configuration  ******/
void Get_Configuration (data)
audio_info_t *data;
{
    AUDIO_INITINFO (data);
    if (ioctl (ACFD, AUDIO_GETINFO, data) < 0) {
      perror ("AUDIO_GETINFO");
    }
    if ((paport = *(&(data -> play.port) + 1)) < 1) paport = 2;
    if ((raport = *(&(data -> record.port) + 1)) < 1) raport = 1;
}

/******  Flag to String  ******/
char * Flag (flag)
unsigned flag;
{
      if (flag) return ("True"); else return ("False");
}

/******  Print Audio Configuration  ******/
Print_Configuration (data)
audio_prinfo_t data;
{
    printf ("\nSample Rate    :  %5d , ", data.sample_rate);
    printf ("\tChannels       :  %5d", data.channels);
    printf ("\nPrecision      :  %5d , ", data.precision);
    printf ("\tEncoding       :  %5d", data.encoding);
    printf ("\nGain          *:  %5d , ", data.gain);
    printf ("\tPort          *:  %5d", data.port);
    printf ("\nAvail Ports    #  %5d , ", *(&data.port + 1));
    printf ("\tSamples        :  %5d", data.samples);
    printf ("\nEof            :  %5d , ", data.eof);
    printf ("\tPause         *:  %5s", Flag (data.pause));
    printf ("\nError          :  %5s , ", Flag (data.error));
    printf ("\tWaiting        :  %5s", Flag (data.waiting));
    printf ("\nBalance       *#  %5d , ", (char)*(&data.waiting + 1));
    printf ("\tMinordev       #  %5d", *(&data.waiting + 3));
    printf ("\nOpen           :  %5s , ", Flag (data.open));
    printf ("\tActive         :  %5s", Flag (data.active));
    printf ("\n");
}

/******  Change Valuse  ******/
void Change_Values (value)
char *value;
{
    char buff[255];

    memset (buff, 0, sizeof (buff));
    if (value[1] == NULL) {
      printf ("\n");
      printf ("P ... Change Play   Configuration\n");
      printf ("R ... Change Record Configuration\n");
      printf ("O ... Change Other  Configuration\n\n?> ");
      gets (buff);
      strcpy (&value[1], buff);
    }
    memset (buff, 0, sizeof (buff));
    if (value[1] == NULL) return;
    if (value[2] == NULL)
      switch (toupper (value[1])) {
        case 'P' :
                   printf ("\n");
                   printf ("G ... Change Gain\n");
                   printf ("P ... Change Port\n");
                   printf ("W ... Change Pause\n");
                   printf ("B ... Change Balance\n\n?> ");
                   gets (buff);
                   strcpy (&value[2], buff);
                   break;
        case 'R' :
                   printf ("\n");
                   printf ("G ... Change Gain\n");
                   printf ("P ... Change Port\n");
                   printf ("W ... Change Pause\n");
                   printf ("B ... Change Balance\n\n?> ");
                   gets (buff);
                   strcpy (&value[2], buff);
                   break;
        case 'O' :
                   printf ("\n");
                   printf ("M ... Change Monitor Gain\n");
                   printf ("O ... Change Output Mute\n\n?> ");
                   gets (buff);
                   strcpy (&value[2], buff);
                   break;
        default  : return;
      };

    switch (toupper (value[1])) {
      case 'P':
               switch (toupper (value[2])) {
                 case 'G': set_gain (PLAY, &value[3]); break;
                 case 'P': set_port (PLAY, &value[3]); break;
                 case 'W': set_pause (PLAY, &value[3]); break;
                 case 'B': set_balance (PLAY, &value[3]); break;
                 default : return;
               }
               break;
      case 'R':
               switch (toupper (value[2])) {
                 case 'G': set_gain (RECORD, &value[3]); break;
                 case 'P': set_port (RECORD, &value[3]); break;
                 case 'W': set_pause (RECORD, &value[3]); break;
                 case 'B': set_balance (RECORD, &value[3]); break;
                 default : return;
               }
               break;
      case 'O':
               switch (toupper (value[2])) {
	         case 'M': set_monitor (&value[3]); break;
	         case 'O': set_mute (&value[3]);    break;
                 default : return;
               }
               break;
      default : return;
    }
}

/*****************/
main ()
{
    char buff[255];
    audio_info_t data;

    if ((ACFD = open ("/dev/audioctl", O_RDWR)) < 0) {
       perror ("/dev/audioctl");
       exit (1);
    }
    memset (buff, 0, sizeof (buff));
    while (1) {
       Get_Configuration (&data);
       switch (toupper (buff[0])) {
         case 'Q' :  close (ACFD); exit (0);
         case 'C' :  Change_Values (buff); break;
         default  :
                  printf ("\n--- Play Configuration ---");
                  Print_Configuration (data.play);
                  printf ("\n--- Record Configuration ---");
                  Print_Configuration (data.record);
                  printf ("\n--- Other Values ---");
                  printf ("\nMonitor Gain  *:  %5d , ", data.monitor_gain);
                  printf ("\tOutput Muted  *#  %5s\n",
                   Flag (*((char*)&data.monitor_gain + sizeof(int))));
                  break;
       };
       printf ("\nQ: Quit , C: Change , Ret: Display :?> ");
       memset (buff, 0, sizeof (buff));
       gets (buff);
    };
}
