/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    audiomisc.c  (Audio misc)
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#define NOX
#include "xplay.h"
#include "menu.h"

extern char * Basename ();
int FileSuffix ();
int about_sample ();
char * Encoding_to_String ();
Boolean  GetFileConfiguration ();

/*****  About  *****/
int about_sample (sampler)
int sampler;
{
    int i, srb, srs;

    for (i = 0;srate[i].value > 0;i++){
      srb = srate[i].sample * (1.0 + SAMPLE_WIDTH);
      srs = srate[i].sample * (1.0 - SAMPLE_WIDTH);
      if (sampler > srs && sampler < srb)
        return srate[i].sample;
    }
    return sampler;
}

/*****  Get File Suffix  *****/
int FileSuffix (filename)
char *filename;
{
     char *cp;

     if ((cp = (char *) strrchr ((char*) Basename(filename), '.')) == NULL) {
       return ULRAW;
     }
     if (strcmp (cp, ".au") == 0)
       return SUN;
     else
     if (strcmp (cp, ".ul") == 0)
       return ULRAW;
     else
     if (strcmp (cp, ".wav") == 0)
       return WAV;
 
     return UNKNOWNSUFFIX;
}

/*****  Convert encoding number to string  *****/
char * Encoding_to_String (encoding)
Cardinal encoding;
{
    return (encoding == PCM8     ? "Linear  8bit" :
           (encoding == PCM16    ? "Linear 16bit" :
           (encoding == PCM32    ? "Linear 32bit" :
           (encoding == MULAW8   ? "U-law   8bit" :
           (encoding == ALAW8    ? "A-law   8bit" :
           (encoding == FLOAT32  ? "Float  32bit" : 
           (encoding == DOUBLE64 ? "Double 64bit" :
                                   "Unknown type")))))));
}

/*****  Get Audio File Configuration  *****/
Boolean GetFileConfiguration (play_obj)
Audio_obj *play_obj;
{
    switch (FileSuffix (play_obj -> filename)) {
      case SUN:
                return SUN_read_filehdr (play_obj);
      case ULRAW:
                return ULRAW_read_filehdr (play_obj);
      case WAV:
                return WAV_read_filehdr (play_obj);
      default : 
                play_obj -> ifd = NULL;
                break;
    }
    Put_Message ("Unknown suffix  ", Basename (play_obj -> filename));
    return False;
}

/*****  Write File Header  *****/
int Write_File_Header (rec_obj)
Audio_obj * rec_obj;
{
    switch (FileSuffix (rec_obj -> filename)) {
      case ULRAW:
      case SUN  :
                  return SUN_write_filehdr (rec_obj);
      case WAV  :
                  return WAV_write_filehdr (rec_obj);
      default   : break;
    }

    Put_Message ("Unknown suffix  ", Basename (rec_obj -> filename));
    return False;
}

/*****  Re-write File Size  *****/
int Rewrite_File_Size (AO)
Audio_obj AO;
{
    switch (AO.suffix) {
      case ULRAW:
      case SUN  :
                  return SUN_rewrite_filesize (AO.ofd, AO.rw_size);
      case WAV  :
                  return WAV_rewrite_filesize (AO.ofd, AO.rw_size);
      default   : return -1;
    }
}

/***** Read long data of little endian *****/
unsigned long read_longlittle (fd)
int fd;
{
    unsigned char buff[10];

    read (fd, buff, sizeof (long));
    return  (long) buff[0] | ((long) buff[1] << 8) |
           ((long) buff[2] << 16) | ((long) buff[3] << 24);
}

/***** Read short data of little endian *****/
unsigned short read_shortlittle (fd)
int fd;
{
    unsigned char buff[10];

    read (fd, buff, sizeof (short));
    return  (long) buff[0] | ((long) buff[1] << 8) ;
}

/***** Write long data of little endian *****/
int write_longlittle (value, fd)
long value;
int  fd;
{
    unsigned char buff[10];
    unsigned long data = value;

    buff[3] = (data >> 24) & 0x000000ff;
    buff[2] = (data >> 16) & 0x000000ff;
    buff[1] = (data >>  8) & 0x000000ff;
    buff[0] = (data      ) & 0x000000ff;
    return write (fd, buff, sizeof (long));
}

/***** Write short data of little endian *****/
unsigned short write_shortlittle (value, fd)
short value;
int   fd;
{
    unsigned char buff[10];
    unsigned short data = value;

    buff[1] = (data >>  8) & 0x00ff;
    buff[0] = (data      ) & 0x00ff;
    return write (fd, buff, sizeof (short));
}

