/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    configp.c    (Configuration panel)
 *
 * Date:    Dec, 15, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#include "xplay.h"
#include "bitmaps/dismiss.xbm"

Widget ConfigBox, ConfigPanel, ConfigArea1, ConfigFile, ConfigDevice;
Widget ConfigArea2, ConfigEnc_l, ConfigEnc_s, ConfigSmp_l, ConfigSmp_s;
Widget ConfigCha_l, ConfigCha_s, stedel_bar, stedel_l, cndismiss;

extern Audio_obj *export_play_obj;        /*  Export play_obj        */

extern void Popdown_Panel ();
extern void Thumb ();
extern void Scroll ();

void Effect_proc ();
void SetConfigMessage ();

/********  Configuration Panel Widgets  ********/
Widget Create_Config_Panel (parent)
Widget parent;
{
    Dimension h;

    ConfigBox   = XtVaCMW ("ConfigBox",   BoxW,    parent  ,  NULL);
    ConfigPanel = XtVaCMW ("ConfigPanel", PanedW,  ConfigBox, NULL);

    ConfigArea1 = XtVaCMW ("ConfigArea1", FormW,   ConfigPanel, NULL);
    ConfigFile  = XtVaCMW ("ConfigFile",  LabelW,  ConfigArea1, NULL);
    ConfigDevice= XtVaCMW ("ConfigDevice",LabelW,  ConfigArea1, NULL);

    ConfigArea2 = XtVaCMW ("ConfigArea2", FormW,   ConfigPanel, NULL);
    ConfigEnc_l = XtVaCMW ("ConfigEnc_l", ToggleW, ConfigArea2, NULL);
    ConfigEnc_s = XtVaCMW ("ConfigEnc_s", LabelW,  ConfigArea2, NULL);
    ConfigSmp_l = XtVaCMW ("ConfigSmp_l", ToggleW, ConfigArea2, NULL);
    ConfigSmp_s = XtVaCMW ("ConfigSmp_s", LabelW,  ConfigArea2, NULL);
    ConfigCha_l = XtVaCMW ("ConfigCha_l", ToggleW, ConfigArea2, NULL);
    ConfigCha_s = XtVaCMW ("ConfigCha_s", LabelW,  ConfigArea2, NULL);

    stedel_l    = XtVaCMW ("stedel_l",    LabelW,  ConfigArea2, larg, NULL);
    stedel_bar  = XtVaCMW ("stedel_bar",  ScrollW, ConfigArea2, barg, NULL);
    cndismiss   = XtVaCMW ("cndismiss",   CommandW,ConfigArea2, barg, NULL);

    XtAddCallback (cndismiss,  XtNcallback,  Popdown_Panel,(caddr_t)PCONF);
    XtAddCallback (stedel_bar, XtNjumpProc,  Thumb,        (caddr_t)STEREODEL);
    XtAddCallback (stedel_bar, XtNscrollProc, Scroll,      (caddr_t)STEREODEL);
    XtAddCallback (ConfigEnc_l,  XtNcallback,   Effect_proc,(caddr_t)ENCODING);
    XtAddCallback (ConfigSmp_l,  XtNcallback,   Effect_proc,(caddr_t)SAMPLE);
    XtAddCallback (ConfigCha_l,  XtNcallback,   Effect_proc,(caddr_t)CHANNEL);

    Bitmap (cndismiss, dismiss_bits, dismiss_width, dismiss_height);

    return ConfigBox;
}

/*****  Set stereo delay (Semi stereo effect)  *****/
void stereo_delay_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    CHSTATE(STEREODEL, lab);
}

/*****  Enable/Disable effects  *****/
void Effect_proc (w, types, num_params)
Widget  w;
int     types;
int     num_params;
{
    int effect = STATE(EFFECT);

    if (effect & types) 
      effect &= ~(effect & types);
    else
      effect |= types;
    CHSTATE(EFFECT, effect);
    switch (types) {
      case ENCODING:
           XtVaSetValues (ConfigEnc_l, XtNstate,
                         (effect & types ? True : False), NULL);
           Flush_Play ();
           break;
      case SAMPLE  :
           XtVaSetValues (ConfigSmp_l, XtNstate, 
                         (effect & types ? True : False), NULL);
           Flush_Play ();
           break;
      case CHANNEL :
           XtVaSetValues (ConfigCha_l, XtNstate,
                         (effect & types ? True : False), NULL);
           Flush_Play ();
           break;
    }
    SetConfigMessage (export_play_obj);
}

/*****  Put file configuration for config panel  *****/
void Put_File_Config (pobj)
Audio_obj *pobj;
{
    char buff[255];

    if (pobj != NULL) {
      sprintf (buff, "File  : %5d Hz %s %s %s",
        pobj -> File_hdr.sample_rate, 
        Encoding_to_String (pobj -> File_hdr.encoding),
         pobj -> File_hdr.channels == STEREO ? "Stereo  " : "Monaural",
        (pobj -> File_hdr.magicid  ==    SUN ? "Sun/Next" :
        (pobj -> File_hdr.magicid  ==    WAV ? "Wav     " : "Unknown ")));
    } else {
      strcpy (buff, "File  :                      ");
    }
    XtVaSetValues(ConfigFile, XtNlabel, buff, NULL);
}

/***** Put Device Configuration for config panel  *****/
void Put_Device_Config (newPlay)
Audio_Device newPlay;
{
    char buff[256];

    sprintf (buff, "Device: %5d Hz %s %s %s",
        newPlay.sample_rate, 
        Encoding_to_String (newPlay.encoding),
        newPlay.channel == STEREO ? "Stereo  " : "Monaural",
        STATE(PROCESS)  == DBRI   ? "DBRI   " : 
        STATE(PROCESS)  == AMD    ? "AMD    " : "Unknown");

    XtVaSetValues (ConfigDevice, XtNlabel, buff, NULL);
}

/*****  Set Configuration message  *****/
void SetConfigMessage (play_obj)
Audio_obj *play_obj;
{
    char buff[256];

    if (STATE(PRFLAG) & PLAY && play_obj != NULL) {
      if (STATE(EFFECT) & ENCODING) {
          if (play_obj -> File_hdr.encoding != play_obj -> Dev_hdr.encoding) {
            sprintf (buff, "%s -> %s", 
                        Encoding_to_String (play_obj -> File_hdr.encoding),
                        Encoding_to_String (play_obj -> Dev_hdr.encoding));
            XtVaSetValues (ConfigEnc_s, XtNlabel, buff, NULL);
          } else {
            XtVaSetValues (ConfigEnc_s, XtNlabel, "Enable ", NULL);
          }
      } else {
         XtVaSetValues (ConfigEnc_s, XtNlabel, "Disable", NULL);
      }
      if (STATE(EFFECT) & SAMPLE) {
         if (play_obj->File_hdr.sample_rate != play_obj->Dev_hdr.sample_rate){
            sprintf (buff, "%d Hz -> %d Hz", 
                              play_obj->File_hdr.sample_rate,
                              play_obj->Dev_hdr.sample_rate);
            XtVaSetValues (ConfigSmp_s, XtNlabel, buff, NULL);
         } else {
            XtVaSetValues (ConfigSmp_s, XtNlabel, "Enable ", NULL);
	 }
      } else {
         XtVaSetValues (ConfigSmp_s, XtNlabel, "Disable", NULL);
      }
      if (STATE(EFFECT) & CHANNEL) {
        if (play_obj->File_hdr.channels != play_obj->Dev_hdr.channels) {
          sprintf (buff, "%s -> %s", 
           (play_obj->File_hdr.channels == MONO ? "Monaural" : "Stereo"),
           (play_obj-> Dev_hdr.channels == MONO ? "Monaural" : "Stereo"));
          XtVaSetValues (ConfigCha_s, XtNlabel, buff, NULL);
	} else {
          XtVaSetValues (ConfigCha_s, XtNlabel, "Enable ", NULL);
	}
      } else {
           XtVaSetValues (ConfigCha_s, XtNlabel, "Disable ", NULL);
      }
    } else {
      if (STATE(EFFECT) & ENCODING) {
        XtVaSetValues (ConfigEnc_s, XtNlabel, "Enable ", NULL);
        XtVaSetValues (ConfigEnc_l, XtNstate, True     , NULL);
      } else {
        XtVaSetValues (ConfigEnc_s, XtNlabel, "Disable", NULL);
        XtVaSetValues (ConfigEnc_l, XtNstate, False    , NULL);
      }
      if (STATE(EFFECT) & SAMPLE) {
        XtVaSetValues (ConfigSmp_s, XtNlabel, "Enable ", NULL);
        XtVaSetValues (ConfigSmp_l, XtNstate, True, NULL);
      } else {
        XtVaSetValues (ConfigSmp_s, XtNlabel, "Disable", NULL);
        XtVaSetValues (ConfigSmp_l, XtNstate, False, NULL);
      }
      if (STATE(EFFECT) & CHANNEL) {
        XtVaSetValues (ConfigCha_s, XtNlabel, "Enable ", NULL);
        XtVaSetValues (ConfigCha_l, XtNstate, True, NULL);
      } else {
        XtVaSetValues (ConfigCha_s, XtNlabel, "Disable", NULL);
        XtVaSetValues (ConfigCha_l, XtNstate, False, NULL);
      }
    }
}
