/*************************************************************
 *
 * XPlay: Audio Control Panel            version 2.1
 *
 * File:  gainp.c (Gain Panel: Gain, Balance, Speed, etc)
 *
 * Date:  Sep, 1, 1993
 *
 * By:    Atsushi Hirahara
 *        School of Science and EnGineering, Waseda University
 *        hirahara@fuka.info.waseda.ac.jp
 *
 *************************************************************/

#include "xplay.h"
#include "menu.h"
#include "bitmaps/record.xbm"
#include "bitmaps/play.xbm"
#include "bitmaps/stop.xbm"
#include "bitmaps/pause.xbm"
#include "bitmaps/skip_f.xbm"
#include "bitmaps/skip_b.xbm"
#include "bitmaps/search_f.xbm"
#include "bitmaps/search_b.xbm"

Widget ControlBox, input_l, input, sample_l, sample;
Widget cpaned, area1, area2, area3;
Widget output_l, output, channel_l, channel;
Widget looping_l, looping, direc_l, direc;
Widget play, stop, Pause, record;
Widget searchB, skipB, skipF, searchF;
Widget listtype, cview, clist, cviewbox, delete, del_all;
Widget info, load, file_l, file_str, info_l, info_str, config_l, config_s;

extern Widget    Control, PopInfo, PopConfig;
extern Widget    toplevel;
extern Widget    rec_in_s, rec_samp_s, rec_chnl_s, rec_pause;

extern Audio_Device Play_Dev;
extern Audio_Device Record_Dev;
/*
 *  Prototypes
 */
extern  void  Mark_File_in_List ();
extern  void  Popup_Panel ();
extern  char * Encoding_to_String ();

void    Set_Sample_Rate ();
void    Put_Info ();
void    channel_proc ();
void    direc_proc ();
void    input_proc ();
void    pause_proc ();
void    looping_proc ();
void    play_proc ();
void    record_proc ();
void    rec_stop_proc ();
void    stop_proc ();
void    skip_proc ();
void    search_proc ();
void    delete_proc ();
void    set_output_label ();
void    set_input_label ();
void    set_sample_rate_label ();
void    set_channel_label ();
void    set_pause_label ();
void    Reset_List ();
void    Set_View_Location ();
void    Set_Output ();
void    Listtype_proc ();
void    Set_Listtype ();

Widget  Create_Control_Panel ();

/*****  Translations for List Widge *****/
static char clistTranslations [] =
       "<Btn2Up> : Play_in_List() \n\
        <Btn3Up> : Delete_Mark_File()";

/********  Control Panel Widgets  ********/
Widget Create_Control_Panel (parent)
Widget parent;
{
    Arg wargs[2];

    ControlBox = XtVaCMW ("ControlBox", BoxW,   parent,  NULL);
    Control  = XtVaCMW ("cpaned",    PanedW,    ControlBox, NULL);

    area1    = XtVaCMW ("area1",     FormW,     Control, farg, NULL);
    input_l  = XtVaCMW ("input_l",   LabelW,    area1, NULL);
    input    = XtVaCMW ("input",     CommandW,  area1, NULL);
    sample_l = XtVaCMW ("sample_l",  LabelW,    area1, NULL);
    sample   = XtVaCMW ("sample",    MenuW,     area1, NULL);
    output_l = XtVaCMW ("output_l",  LabelW,    area1, NULL);
    output   = XtVaCMW ("output",    MenuW,     area1, NULL);
    channel_l= XtVaCMW ("channel_l", LabelW,    area1, NULL);
    channel  = XtVaCMW ("channel",   CommandW,  area1, NULL);
    looping_l= XtVaCMW ("looping_l", LabelW,    area1, NULL);
    looping  = XtVaCMW ("looping",   ToggleW,   area1, NULL);
    direc_l  = XtVaCMW ("direc_l",   LabelW,    area1, NULL);
    direc    = XtVaCMW ("direc",     ToggleW,   area1, NULL);

    play     = XtVaCMW ("play",      ToggleW,   area1, cntarg, NULL);
    stop     = XtVaCMW ("stop",      CommandW,  area1, cntarg, NULL);
    Pause    = XtVaCMW ("Pause",     ToggleW,   area1, cntarg, NULL);
    record   = XtVaCMW ("record",    ToggleW,   area1, cntarg, NULL);
    skipB    = XtVaCMW ("skipB",     CommandW,  area1, cntarg, NULL);
    searchB  = XtVaCMW ("searchB",   RepeaterW, area1, cntarg, NULL);
    searchF  = XtVaCMW ("searchF",   RepeaterW, area1, cntarg, NULL);
    skipF    = XtVaCMW ("skipF",     CommandW,  area1, cntarg, NULL);

    area2    = XtVaCMW ("area2",     FormW,     Control, farg, NULL);
    listtype = XtVaCMW ("listtype",  CommandW,  area2,         NULL);
    cview    = XtVaCMW ("cview",     ViewPW,    area2,         NULL);
    cviewbox = XtVaCMW ("cviewbox",  BoxW,      cview,         NULL);
    clist    = XtVaCMW ("clist",     ListW,     cviewbox,      NULL);

    area3    = XtVaCMW ("area3",     FormW,     Control, farg, NULL);
    load     = XtVaCMW ("load",      ToggleW,   area3,    NULL);
    PopInfo  = XtVaCMW ("info",      ToggleW,   area3,    NULL);
    delete   = XtVaCMW ("delete",    CommandW,  area3,    NULL);
    del_all  = XtVaCMW ("del_all",   CommandW,  area3,    NULL);
    PopConfig= XtVaCMW ("PopConfig", ToggleW,   area3,    NULL);
    file_l   = XtVaCMW ("file_l",    LabelW,    area3,    NULL);
    file_str = XtVaCMW ("file_str",  AsciiW,    area3,    NULL);
    info_l   = XtVaCMW ("info_l",    LabelW,    area3,    NULL);
#ifdef MULTI
    if (STATE(LANGF))
      info_str = XtVaCMW ("info_str",  MultiW,    area3,    NULL);
    else
      info_str = XtVaCMW ("info_str",  AsciiW,    area3,    NULL);
#else
    info_str = XtVaCMW ("info_str",  AsciiW,    area3,    NULL);
#endif 
    config_l = XtVaCMW ("config_l",  LabelW,    area3,    NULL);
    config_s = XtVaCMW ("config_s",  LabelW,    area3,    NULL);

    MyAugmentTranslations (clist, clistTranslations);

    XtAddCallback (input,   XtNcallback,   input_proc,   NULL);
    XtAddCallback (channel, XtNcallback,   channel_proc, NULL);
    XtAddCallback (play,    XtNcallback,   play_proc,    NULL);
    XtAddCallback (Pause,   XtNcallback,   pause_proc,   (caddr_t)PLAY);
    XtAddCallback (record,  XtNcallback,   Popup_Panel,  (caddr_t)PREC);
    XtAddCallback (looping, XtNcallback,   looping_proc, NULL);
    XtAddCallback (direc,   XtNcallback,   direc_proc,   NULL);
    XtAddCallback (stop,    XtNcallback,   stop_proc,    NULL);
    XtAddCallback (skipF,   XtNcallback,   skip_proc,    (caddr_t)FORWARD);
    XtAddCallback (skipB,   XtNcallback,   skip_proc,    (caddr_t)BACKWARD);
    XtAddCallback (searchF, XtNcallback,   search_proc,  (caddr_t)FORWARD);
    XtAddCallback (searchB, XtNcallback,   search_proc,  (caddr_t)BACKWARD);
    XtAddCallback (listtype,XtNcallback,   Listtype_proc,NULL);
    XtAddCallback (clist,   XtNcallback,   Mark_File_in_List, (caddr_t)NULL);
    XtAddCallback (delete,  XtNcallback,   delete_proc,  (caddr_t)THISONE);
    XtAddCallback (del_all, XtNcallback,   delete_proc,  (caddr_t)ALL);
    XtAddCallback (PopInfo, XtNcallback,   Popup_Panel,  (caddr_t)PINFO);
    XtAddCallback (load,    XtNcallback,   Popup_Panel,  (caddr_t)PFILE);
    XtAddCallback (PopConfig, XtNcallback,   Popup_Panel,(caddr_t)PCONF);

    if (STATE(SPLIT))
      Create_Message_Panel (Control);

    Bitmap (play, play_bits, play_width, play_height);
    Bitmap (record, record_bits, record_width, record_height);
    Bitmap (stop, stop_bits, stop_width, stop_height);
    Bitmap (Pause, pause_bits, pause_width, pause_height);
    Bitmap (skipB, skip_b_bits, skip_b_width, skip_b_height);
    Bitmap (skipF, skip_f_bits, skip_f_width, skip_f_height);
    Bitmap (searchB, search_b_bits, search_b_width, search_b_height);
    Bitmap (searchF, search_f_bits, search_f_width, search_f_height);

    if (STATE(PROCESS) == AMD) {
      srate[1].value  = 0;
      soutput[2].value = 0;
    }
    Create_Menu_Pane (sample, "SampleRate", srate,  Set_Sample_Rate);
    Create_Menu_Pane (output, "Output",     soutput, Set_Output);

    XtVaSetValues (looping, XtNlabel, "    Off    ", NULL);
    XtVaSetValues (direc,   XtNlabel, "  Normal   ", NULL);
    if (STATE(PROCESS) == AMD) {
      XtVaSetValues (input_l,   XtNsensitive, False, NULL);
      XtVaSetValues (input,     XtNsensitive, False, NULL);
      XtVaSetValues (sample_l,  XtNsensitive, False, NULL);
      XtVaSetValues (sample,    XtNsensitive, False, NULL);
      XtVaSetValues (channel_l, XtNsensitive, False, NULL);
      XtVaSetValues (channel,   XtNsensitive, False, NULL);
    }

    XawListChange (clist, Null, 0, 0, True);

    return ControlBox;
}

/*******  Input Port Toggle (Microphone | Line in )  *******/
void input_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    invert_inputport (Record_Dev);
}

/********  Channels  (1 is MONO, 2 is STEREO)  ********/
void channel_proc (w, params, num_params)
Widget  w;
String  *params;
int     num_params;
{
    invert_channel (Play_Dev);   /* Play channel equal to record channel */
}

/********  Play Pause Toggle  ********/
void pause_proc (w, flag, num_params)
Widget  w;
int     flag;
int     num_params;
{
   if ( flag == RECORD )
     invert_recpause (Record_Dev);
   else
     invert_playpause (Play_Dev);
}

/********  Play Looping Toggle  ********/
void looping_proc (w, params, num_params)
Widget  w;
String  *params;
int     num_params;
{
    if (STATE(LOOPING)) {
      XtVaSetValues (looping, XtNlabel, "Off", XtNstate, False, NULL);
      CHSTATE(LOOPING, False);
    } else {
      XtVaSetValues (looping, XtNlabel, "On", XtNstate, True,  NULL);
      CHSTATE(LOOPING, True);
    }
}

/********  Play Start  ********/
void play_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    int indexes;

    if ( ! (STATE(PRFLAG) & PLAY) ) {
      indexes = STATE(CURTLST);
      if (indexes < 0) {
        Put_Message (" No file is loaded. " , Non);
        XtVaSetValues (play, XtNstate, False, NULL);
        if ( ! (STATE(POPUP) & PFILE) )
          Popup_Panel (toplevel, PFILE, 1);
        return;
      } else {
        FlushOutPut ();
      }
      if (STATE(PAUSEF) & PLAY ) {
        Put_Message (" Pausing  ", CFILENAME());
      }
      CHSTATE(PRFLAG, STATE(PRFLAG) | PLAY);
      CHSTATE(PSTATE, NOPLAY);
    }
    if ( STATE(PRFLAG) & PLAY ) 
      XtVaSetValues (play, XtNstate, True, NULL);
    else
      XtVaSetValues (play, XtNstate, False, NULL);
}

/********  Play Stop  ********/
void stop_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    XtVaSetValues (play, XtNstate, False, NULL);
    if ( ! STATE(PRFLAG) )
      Put_Message (DEMO, Non);
    CHSTATE(PRFLAG, (STATE(PRFLAG) & ~PLAY));
    FlushOutPut ();
    if ( ! STATE(PRFLAG) ) {
      Put_Info (NULL);
      Put_File_Config (NULL);
      SetConfigMessage (NULL);
    }
}

/*****  Direction proc  *****/
void direc_proc (w, params, num_params)
Widget w;
int params, num_params;
{
    int direction;

    direction = (STATE(DIRECTI) == FORWARD ? BACKWARD : FORWARD);
    SetDirection (direction);
    if (direction == FORWARD) {
      XtVaSetValues (direc, XtNlabel, "Normal", XtNstate, False, NULL);
    } else {
      XtVaSetValues (direc, XtNlabel, "Reverse", XtNstate, True,  NULL);
    }     
    XtVaSetValues (direc, XtNstate, (direction != FORWARD ? True : False), NULL);
}

/********  Skip Forward / Backword Play  ********/
void skip_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    int indexes;

    if ((STATE(PRFLAG) & PLAY) && STATE(PSTATE) == DOPLAY) {
      if (params == FORWARD)
        indexes = Forward_List ();
      else
        indexes = Backward_List ();
      if (indexes >= 0) {
        FlushOutPut ();
        CHSTATE(PSTATE, NOPLAY);
        if (params == FORWARD)
          Put_Message (" Skip Forward. ", Non);
        else
          Put_Message (" Skip Backward. ", Non);
      } else {
        if (params == FORWARD)
          indexes = Rewind_List ();
        else
          indexes = End_of_List ();
        if (indexes >= 0) {
          FlushOutPut ();
          CHSTATE(PSTATE, NOPLAY);
          if (params == FORWARD)
            Put_Message (" Wrapped Forward. ", Non);
          else
            Put_Message (" Wrapped Backward. ", Non);
        }
      }
    }
}

/********  Search Forward / Backward Play  ********/
void search_proc (w, type, num_param)
Widget w;
int type;
int num_param;
{
    if ( ! (STATE(PRFLAG) & PLAY) ) return;
    Seek_File (type);
}

/*******  List type proc   ******/
void Listtype_proc (w, type, nump)
Widget w;
int type;
int nump;
{
     if (STATE(SHORTLIST))
       CHSTATE(SHORTLIST, False);
     else
       CHSTATE(SHORTLIST, True);
     Reset_List ();
     Set_View_Location ();
}

/********  Delete Lists  ********/
void delete_proc (w, type, num_param)
Widget w;
int type;
int num_param;
{
    if (type == THISONE)
      Del_List (STATE(CURTLST));
    else
      Delete_All ();
    FlushOutPut ();
    CHSTATE(PSTATE, NOPLAY);
    if (type == ALL || STATE(CURTLST) < 1)
     stop_proc (w, type, num_param);
}

/******** Internal speaker or external jack or Line out Label ********/
void set_output_label (flag)
int flag;
{
    Dimension wi;

    XtVaGetValues (output, XtNwidth, &wi, NULL);

    if (STATE(MUTED)) {
      XtVaSetValues (output, XtNlabel, soutput[3].label, NULL);
    } else
    switch (flag) {
      case AUDIO_SPEAKER: 
                   XtVaSetValues (output, XtNlabel, soutput[0].label, NULL);
                   break;
      case AUDIO_HEADPHONE:
                   XtVaSetValues (output, XtNlabel, soutput[1].label, NULL);
                   break;
      case AUDIO_LINE_OUT:
                   XtVaSetValues (output, XtNlabel, soutput[2].label, NULL);
                   break;
      case AUDIO_SPEAKER | AUDIO_HEADPHONE:
                   XtVaSetValues (output, XtNlabel, soutput[4].label, NULL);
                   break;
      case AUDIO_SPEAKER | AUDIO_LINE_OUT:
                   XtVaSetValues (output, XtNlabel, soutput[5].label, NULL);
                   break;
      case AUDIO_LINE_OUT | AUDIO_HEADPHONE:
                   XtVaSetValues (output, XtNlabel, soutput[6].label, NULL);
                   break;
      case AUDIO_SPEAKER | AUDIO_HEADPHONE | AUDIO_LINE_OUT:
                   XtVaSetValues (output, XtNlabel, soutput[7].label, NULL);
                   break;
    };
    XtVaSetValues (output, XtNwidth, wi, NULL);
}

/******** Internal Microphone or Line in ********/
void set_input_label (flag)
int flag;
{
    Dimension iw, riw;

    XtVaGetValues (input, XtNwidth, &iw, NULL);
    XtVaGetValues (rec_in_s, XtNwidth, &riw, NULL);

    if (flag) {
      XtVaSetValues (input, XtNlabel, "Microphone", XtNwidth, iw, NULL);
      XtVaSetValues (rec_in_s, XtNlabel, "Microphone", XtNwidth, riw, NULL);
    } else {
      XtVaSetValues (input, XtNlabel, "Line in", XtNwidth, iw, NULL);
      XtVaSetValues (rec_in_s, XtNlabel, "Line in", XtNwidth, riw, NULL);
    }
}

/*****  Sample rate and Encoding Menu Label  ******/
void set_sample_rate_label (rate, encoding)
int rate;
int encoding;
{
    Dimension wi, rwi;
    char buff[20], buff2[50];

    XtVaGetValues (sample, XtNwidth, &wi, NULL);
    XtVaGetValues (rec_samp_s, XtNwidth, &rwi, NULL);
    switch (encoding) {
      case U_LAW : sprintf (buff2, "%d Hz , U-law , 8bit  ", rate);
                   sprintf (buff,  " %d Hz U  ", rate); break;
      case A_LAW : sprintf (buff2, "%d Hz , A-law , 8bit  ", rate);
                   sprintf (buff,  " %d Hz A  ", rate); break;
      case LINEAR: sprintf (buff2, "%d Hz , Linear , 16bit", rate);
                   sprintf (buff,  " %d Hz L  ", rate); break;
      default:     sprintf (buff,  " Unknown  ");    
                   sprintf (buff2, " Unknown  ");        break;
    }
    XtVaSetValues (sample,  XtNlabel,  buff, XtNwidth, wi, NULL);
    XtVaSetValues (rec_samp_s,  XtNlabel, buff2, XtNwidth, rwi, NULL);
}

/*******  Channel Label  *******/
void set_channel_label (chan)
unsigned chan;
{
    Dimension wi, rwi;
    char buff[20];

    XtVaGetValues (channel, XtNwidth, &wi, NULL);
    XtVaGetValues (rec_chnl_s, XtNwidth, &rwi, NULL);
    if (chan == MONO)
      sprintf (buff, "Monaural");
    else 
      sprintf (buff, "Stereo");
    XtVaSetValues (rec_chnl_s,  XtNlabel, buff, XtNwidth, rwi, NULL);
    XtVaSetValues (channel, XtNlabel, buff, XtNwidth, wi, NULL);
}

/********  Play and Record Pause Label  ********/
void set_pause_label (pr, flag)
int pr;
unsigned flag;
{
    if (pr == PLAY) {
      if (flag) {
        XtVaSetValues (Pause, XtNlabel, "Resume", XtNstate, True, NULL);
        if ( STATE(PRFLAG) & PLAY ) {
          Put_Message (" Pausing  ", CFILENAME());
        } else {
          Put_Message (" Audio Device Pausing (Play).", Non);
        }
        CHSTATE(PAUSEF, (STATE(PAUSEF) | PLAY));
      } else {
        XtVaSetValues (Pause, XtNlabel, "Pause", XtNstate, False, NULL);
        if ( STATE(PRFLAG) & PLAY ) 
          Put_Message (" Playing  ", CFILENAME());
        else
          Put_Message (DEMO, Non);
        Close_Pause_FD (PLAY);
        CHSTATE(PAUSEF, (STATE(PAUSEF) & ~PLAY));
      }
    } else {
      if (flag) {
        XtVaSetValues (rec_pause, XtNlabel, "Resume", XtNstate, True, NULL);
        if ( STATE(PRFLAG) & RECORD ) {
          Put_Message (" Record Pausing: ", STRINGS(RECFILE));
        } else {
          Put_Message (" Audio Device Pausing (Record).", Non);
        }
        CHSTATE(PAUSEF, (STATE(PAUSEF) | RECORD));
      } else {
        XtVaSetValues (rec_pause, XtNlabel, "Pause", XtNstate, False, NULL);
        if ( STATE(PRFLAG) & RECORD )
          Put_Message (" Recording ", STRINGS(RECFILE));
        else
          Put_Message (DEMO, Non);
        Close_Pause_FD (RECORD);
        CHSTATE(PAUSEF,(STATE(PAUSEF) & ~RECORD));
      }
    }
}

/*****  Put Information strings  *****/
void Put_Info(pobj)
Audio_obj *pobj;
{
    char buff[255], *info;

    if (pobj != NULL) {
      sprintf (buff, "%d Hz %s %s",
        pobj -> File_hdr.sample_rate, 
        Encoding_to_String (pobj -> File_hdr.encoding),
        pobj -> File_hdr.channels == STEREO ? "Stereo" : "Monaural");
      info = pobj -> Infostr;
    } else {
      info = " ";
      strcpy (buff, "   ");
    }
    XtVaSetValues(info_str, XtNstring, info, NULL);
    XtVaSetValues(config_s, XtNlabel,  buff, NULL);
}

/*****  Set List type label and file number  *****/
void Set_Listtype ()
{
    char buff[BUFSIZ];

    if (STATE(SHORTLIST)) {
      sprintf (buff, "%3d Files  :  Base name     ", STATE(LISTNUM));
    } else {
      sprintf (buff, "%3d Files  :  Full path name", STATE(LISTNUM));
    }
    XtVaSetValues (listtype, XtNlabel, buff, NULL);
}

/*****  Reset List view Window  *****/
void Reset_List ()
{
    int i;
    String * lists;

    lists = (String *)LISTPOINTER();

    lists[STATE(LISTNUM)] = Null;       
    XawListChange (clist, lists, 0, 0, True);
    if (STATE(CURTLST) >= 0 && STATE(LISTNUM) > 0){
      XawListHighlight (clist, STATE(CURTLST));
      XtVaSetValues (file_str, XtNstring, CFILENAME(), NULL);
    } else {
      XtVaSetValues (file_str, XtNstring, "", NULL);
    }
    Set_Listtype ();
}

/*****  Set Viewport Location if playing  *****/
void Set_View_Location ()
{
    float yoff;

    if (STATE(LISTNUM) > 0)
      yoff = (float)(STATE(CURTLST) - 3) / STATE(LISTNUM); else yoff = 0.0;
    if (XtIsRealized (cview))
      XawViewportSetLocation (cview, (STATE(SHORTLIST) ? 0.0: 1.0),
                                     (yoff < 0.0 ? 0.0 : yoff));
}

/********  Sample rate and Encoding  ********/
void Set_Sample_Rate (w, id)
Widget  w;
int     id;
{
    Audio_Device  newPlay;
    Audio_Device  newRecord;
    Audio_Gain    newGain;

    GetAudioConfiguration (&newPlay, &newRecord, &newGain);
    set_audio_config (srate[id].sample, srate[id].encoding, 
           srate[id].precision, min (srate[id].channels, newPlay.channel));
}

/********  Output Port (Speaker | Headphone | Line out)  ********/
void Set_Output (w, id)
Widget  w;
int     id;
{
    set_outputport (soutput[id].value);
}

