/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    convert.c    (Audio data converter)
 *
 * Date:    Dec, 13, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#define  NOX
#include "xplay.h"

extern char   *fbuf2, *fbuf1;
extern char   *fbuf, *fbuf3;
extern char   *rbuf;

int      EnSlow ();
int      EnFast ();
int      Scale_data ();
int      Backward_Data ();
int      Linear_to_Ulaw ();
int      Ulaw_to_Linear ();
int      Linear8_to_16 ();

/**************************/
/**** Backward Data    ****/
/**** Input : All type ****/
/**** Output: All type ****/
/**************************/
int Backward_Data (dst, src, cnt, bytes_per_units)
char **dst, *src;
int  cnt, bytes_per_units;
{
    int i, c, j;

    for (i = 0, c = 0;i < cnt - 1;i += bytes_per_units) {
       for (j = 0;j < bytes_per_units;j++)
          fbuf2[c++] = src[cnt - i + j - bytes_per_units];
    }
    *dst = fbuf2;
    return c;
}

/***************************/
/****   Speed Control   ****/
/**** Input : All type  ****/
/**** Output: All type  ****/
/***************************/

/*****  EnFast or EnSlow Play Data  *****/
int Scale_data (dst, src, cnt, scale, bytes_per_units)
char **dst, *src;
int  cnt, scale, bytes_per_units;
{
    unsigned int i, len, j, col;

    if (scale == 0) {
      *dst = src;
      return cnt; 
    }
    len  = (unsigned int)(scale > 0 ? 
                  (cnt * 2 / (scale + 2)) : cnt * (-scale + 2) / 2);
    len -= len % bytes_per_units;
    for (i = 0; i < len;i += bytes_per_units) {
       col = (i * cnt / len / 2) * 2;
       for (j = 0; j < bytes_per_units; j++)
          fbuf2[i + j] = src[col + j];
    }
    *dst = fbuf2;
    return len;
}

/******************************************/
/**** Linear 8bit to 16bit             ****/
/**** Input : Unsigned Linear  8 bit   ****/
/**** Output: Signed   Linear 16 bit   ****/
/******************************************/
int Linear8_to_16 (dst, src, cnt)
char **dst, *src;
int cnt;
{
    int i, j;
    union _u_data {
      short linear;
      char  data[2];
    } u_data;

    for (i = 0, j = 0; i < cnt; i++) {
       u_data.linear = ((unsigned char)src[i] << 8) - 0x8000;
       fbuf2[j++] = u_data.data[0];
       fbuf2[j++] = u_data.data[1];
    }
    *dst = fbuf2;
    return j;
}

/******************************************/
/**** Linear 16bit to 8bit             ****/
/**** Input : Unsigned Linear  8 bit   ****/
/**** Output: Signed   Linear 16 bit   ****/
/******************************************/
int Linear16_to_8 (dst, src, cnt)
char **dst, *src;
int cnt;
{
    int i, j;
    unsigned char uc;
    union _u_data {
      short linear;
      char  data[2];
    } u_data;

    for (i = 0, j = 0; i < cnt / 2; i++) {
       u_data.data[0] = src[j++];
       u_data.data[1] = src[j++];
       uc = (char)(u_data.linear >> 8) + 0x80;
       fbuf2[i] =  uc;
    }
    *dst = fbuf2;
    return i;
}

/*********************************/
/**** Linear to u-law         ****/
/**** Input : Linear Monaural ****/
/**** Output: U-law  Monaural ****/
/*********************************/
int Linear_to_Ulaw (dst, src, cnt)
char **dst, *src;
int cnt;
{
    int i;
    union _u_data {
      short linear;
      char  data[2];
    } u_data;

    if ( ! (STATE(EFFECT) & ENCODING)) {
      memcpy (fbuf2, src, cnt);
      i = cnt;
    } else {
      for (i = 0; i < cnt / 2;i++) {
        u_data.data[0] = src[2 * i + 0];
        u_data.data[1] = src[2 * i + 1];
        fbuf2[i] = (char) st_linear_to_ulaw ((int) u_data.linear);
      }
    }
    *dst = fbuf2;
    return i;
}

/*********************************/
/**** U-law to Linear         ****/
/**** Input : U-law Monaural  ****/
/**** Output: Linear Monaural ****/
/*********************************/
int Ulaw_to_Linear (dst, src, cnt)
char **dst, *src;
int cnt;
{
    int i, c = cnt;
    union _u_data {
      short datum;
      char  linear[5];
    } u_data;

    if ( ! (STATE(EFFECT) & ENCODING)) {
      memcpy (fbuf2, src, cnt);
    } else {
      for (i = 0, c = 0; i < cnt; i++) {
        u_data.datum = st_ulaw_to_linear_slow(src[i]);
        fbuf2[c++] = u_data.linear[0];
        fbuf2[c++] = u_data.linear[1];
      }
    }
    *dst = fbuf2;
    return c;
}

/*********************************/
/**** Monaural to Stereo      ****/
/**** Input : Linear Monaural ****/
/**** Output: Linear Stereo   ****/
/*********************************/
int Monaural_to_Stereo (dst, src, cnt, sample_rate, stereodelay)
char **dst, *src;
int cnt, sample_rate, stereodelay;
{
    int i, j = 0, k = 0, delay1;
    int del = (stereodelay - MAX_GAIN/2) * MAX_STEREODELAY / 50;

    if ( ! (STATE(EFFECT) & CHANNEL)) {
      memcpy (fbuf2, src, cnt);
      *dst = fbuf2;
      return cnt;
    }
    delay1  = sample_rate * (del > 0 ? del : -del) / 2000;
    delay1 -= delay1 % 8;

    delay1 = (delay1 > cnt ? cnt : delay1);
    memcpy (&fbuf3[delay1], src, cnt - delay1);      
    if (del > 0) {
      while (k < cnt) {
        for (i = 0;i < 2;i++){
          fbuf2[j + 0] = src [k];
          fbuf2[j + 2] = fbuf3[k++];
          j++;
        }
        j += 2;
      }
    } else {
      while (k < cnt) {
        for (i = 0;i < 2;i++){
          fbuf2[j + 0] = fbuf3[k];
          fbuf2[j + 2] = src [k++];
          j++;
        }
        j += 2;
      }
    }
    *dst = fbuf2;
    memcpy (fbuf3, &src[cnt - delay1], delay1);
    cnt *= 2;
    return cnt;
}

/*********************************/
/**** Stereo to Monaural      ****/
/**** Input : Linear Stereo   ****/
/**** Output: Linear Monaural ****/
/*********************************/
int Stereo_to_Monaural (dst, src, cnt)
char **dst, *src;
int  cnt;
{
    int i, len, j, k;
    union _u_data {
      short unit;
      char  data[3];
    } u_dataL, u_dataR, u_dataAVG;

    if ( ! (STATE(EFFECT) & CHANNEL)) {
      memcpy (fbuf2, src, cnt);
      k = cnt;
    } else {
      len = cnt / 4;
      for (i = 0 , j = 0, k = 0; i < len; i++) {
         u_dataL.data[0] = src[j++];
         u_dataL.data[1] = src[j++];
         u_dataR.data[0] = src[j++];
         u_dataR.data[1] = src[j++];
         u_dataAVG.unit = (u_dataL.unit + u_dataR.unit) / 2;
         fbuf2[k++] = u_dataAVG.data[0];
         fbuf2[k++] = u_dataAVG.data[1];
      }
    }
    *dst = fbuf2;
    return k;
}

/*********************************/
/**** Change Sample Rate      ****/
/**** Input : Linear S / M    ****/
/**** Output: Linear S / M    ****/
/*********************************/
/***** src_rate -> dst_rate *****/
int ChangeSampleRate (dst, dst_rate, src, src_rate, cnt, channel)
char **dst, *src;
int cnt, dst_rate, src_rate;
{
    unsigned int i, col, len, avg;
    short  predata = 0;
    union _u_data {
      short unit;
      char  data[3];
    } u_data, u_data2;

    *dst = src;
    if (dst_rate == src_rate || src_rate == 0) return cnt;

    if ( ! (STATE(EFFECT) & SAMPLE)) {
      *dst = src;
      return cnt;
    }
    len  = (unsigned int)(cnt * dst_rate) / src_rate;
    len -= len % 4;
    if (channel == MONO)
      for (i = 0; i < len;i += 2) {
         col = (i * cnt / len / 2) * 2;
         u_data.data[0] = src[col + 0];
         u_data.data[1] = src[col + 1];
         avg = (u_data.unit + predata) / 2;
         predata = u_data.unit = (short) avg;
         fbuf2[i + 0] = u_data.data[0];
         fbuf2[i + 1] = u_data.data[1];
      }
    else
      for (i = 0; i < len;i += 4) {
         col = (i * cnt / len / 4) * 4;
         fbuf2[i + 0] = src[col + 0];
         fbuf2[i + 1] = src[col + 1];
         fbuf2[i + 2] = src[col + 2];
         fbuf2[i + 3] = src[col + 3];
      }
    *dst = fbuf2;
    return len;
}

/*****  Linear PCM Converter  (DBRI)  *****/
int LinearPCM_Converter (dst, src, cnt, play_obj, AD)
char **dst, *src;
int cnt;
Audio_obj play_obj;
Audio_Device AD;
{
    char *temp;

    memcpy (fbuf1, src, cnt);
    temp = fbuf1;
    switch (play_obj.File_hdr.encoding) {
      case MULAW8:                          /*  MU-Law File          */
           if (AD.encoding == PCM16 || AD.encoding == PCM8) {
                                            /*  Device is Linear 8/16  */
               cnt = Ulaw_to_Linear (&temp, fbuf1, cnt);
               memcpy (fbuf1, temp, cnt);
               cnt = ChangeSampleRate (&temp, AD.sample_rate, fbuf1,
                                   play_obj.File_hdr.sample_rate, cnt, MONO);
               if (AD.channel == STEREO) {
                  memcpy (fbuf1, temp, cnt);
                  cnt = Monaural_to_Stereo (&temp, fbuf1, cnt,
                               play_obj.Dev_hdr.sample_rate, STATE(STEREODEL));
                  /* ULAW -> LINEAR Stereo   */
               } else {
                  /* ULAW -> LINEAR_Monaural */
               }
               if (AD.encoding == PCM8) {
                 memcpy (fbuf1, temp, cnt);
                 cnt = Linear16_to_8 (&temp, fbuf1, cnt);
	       }
           } else break;  /* Device is Mu-Law  ULAW -> ULAW (not convert)*/
           break;
      case PCM8 :                             /* Linear PCM  8 bit File */
           cnt = Linear8_to_16 (&temp, fbuf1, cnt);
           memcpy (fbuf1, temp, cnt);
      case PCM16:                             /* Linear PCM 16 bit File */
           cnt = ChangeSampleRate (&temp, AD.sample_rate, fbuf1,
                                      play_obj.File_hdr.sample_rate, cnt,
                                      play_obj.File_hdr.channels);
           memcpy (fbuf1, temp, cnt);
           if (play_obj.File_hdr.channels == MONO) { /* Linear Monaural */
              if (AD.encoding == MULAW8) {
                 cnt = Linear_to_Ulaw (&temp, fbuf1, cnt);
                 /* LINEAR Monaural -> ULAW */
              } else
              if (AD.encoding == PCM16 && AD.channel == STEREO) { 
                 /* Semi-Stereo mode */                 
                 cnt = Monaural_to_Stereo (&temp, fbuf1, cnt,
                          play_obj.File_hdr.sample_rate, STATE(STEREODEL));
                 /* Linear Monaural -> Linear Stereo */
              }
           } else {                                   /* Linear Stereo    */
             if (AD.encoding == MULAW8) {             /* Device is Mu-law */
                cnt = Stereo_to_Monaural (&temp, fbuf1, cnt);
                memcpy (fbuf1, temp, cnt);
                cnt = Linear_to_Ulaw (&temp, fbuf1, cnt);
                /* LINEAR Stereo ->  ULAW */
             } else                                   /* Device is Linear */
             if (AD.encoding == PCM16 && AD.channel == MONO) { 
                cnt = Stereo_to_Monaural (&temp, fbuf1, cnt);
                /* LINEAR Stereo -> Monaural */
	     }
	   }
           break;
      default: break;
    };
    *dst = temp;
    return cnt;
}

/*****  Convert Data for AMD (8bit, u-law, monaural only)  *****/
int Ulaw_Converter (dst, src, cnt, play_obj)
char **dst, *src;
int  cnt;
Audio_obj play_obj;
{
    char *temp;

    memcpy (fbuf1, src, cnt);
    switch (play_obj.File_hdr.encoding) {
      case MULAW8:                        /* Ulaw -> Ulaw */
           temp = fbuf1;                  /*** Do Nothing ***/
           break;
      case PCM16 :                        /* Linear 16 File */
           if (play_obj.File_hdr.channels == MONO) { /* Monaural File */
               cnt = ChangeSampleRate (&temp, 8000, fbuf1, 
                                       play_obj.File_hdr.sample_rate, cnt);
               memcpy (fbuf1, temp, cnt);
               cnt = Linear_to_Ulaw (&temp, fbuf1, cnt);
               /* LINEAR Monaural -> ULAW */
           } else {                       /* Strereo File */
              cnt = Stereo_to_Monaural (&temp, fbuf1, cnt);
              memcpy (fbuf1, temp, cnt);
              cnt = ChangeSampleRate (&temp, 8000, fbuf1, 
                                     play_obj.File_hdr.sample_rate, cnt);
              memcpy (fbuf1, temp, cnt);
              cnt = Linear_to_Ulaw (&temp, fbuf1, cnt);
              /* LINEAR Stereo -> ULAW */
           }
           break;
      default: break;
    };
    *dst = temp;
    return cnt;
}

/*****  Mill Secs to byte converter  *****/
int Millsecs_to_bytes(base, millsecs)
int base, millsecs;
{
    int secs = millsecs / 1000;
    int mill = millsecs % 1000;

    return  (base * secs) + (base * mill) / 1000;
}

/*****  Endian change small <-> big  *****/
int Endian (dst, src, cnt, unit_size)
char **dst;
char *src;
int  cnt, unit_size;
{
    int i, j, count = 0;
    static char buffer[AUDIOBUFSIZ];

    if (unit_size == 1) {
      memcpy (buffer, src, cnt);
      *dst = buffer;
      return cnt;
    }
    for (i = 0; count < cnt; i++) {
       for (j = 0;j < unit_size; j++) {
          buffer[i * unit_size + unit_size - j - 1] = src[count++];
       }
    }
    *dst = buffer;
    return cnt;
}

