/*****************************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    filemgr.c  (File Selector)
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 *****************************************************************/

#define NOX
#include "xplay.h"

static   FNODE  DATA[MAX_DATA + 1];  /**  File Information            **/
static   String LIST[MAX_DATA + 1];  /**  File List                   **/
static   String SLIST[MAX_DATA+ 1];  /**  Short type (only filename)  **/
static   String names[BUFSIZ];       /**  File name strings for list  **/
static   String currentdir;          /**  Current working directory   **/
/*
 *  Prototypes
 */

int     MakeNewNode ();
int     filestat ();
void    FileManager ();
void    Set_Mark_Label ();
void    PlayFileMark ();
extern  void    Reset_List ();
extern  char *STRINGS();
extern  int STATE();
extern  int CHSTATE();
String * NAMESPOINTER();
String * LISTPOINTER();
char *  OptimizeDirname ();

/*****  Pointer to names  *****/
String * NAMESPOINTER()
{
    return names;
}

/*****  Pointer to LIST  *****/
String * LISTPOINTER()
{
    String *listptr, *slistptr, *ANSWER = LIST, cp;

    if (STATE(SHORTLIST)) {
      listptr = LIST;
      slistptr = SLIST;
      while (*listptr != NULL) {
        if ((cp = (char*)strrchr (*listptr, '/')) != NULL) {
          *slistptr = cp + 1;
        } else {
          *slistptr = *listptr;
        }
        listptr++;
        slistptr++;
      }
      ANSWER = SLIST;
    }
    return ANSWER;
}

/*****  Return Current full path   *****/
String CPATHNAME()
{
    if (STATE(CURTLST) >= 0)
      return LIST[STATE(CURTLST)];
    else
      return " ";
}

/*****  Return Current full path list pointer  *****/
String * CLISTPTR()
{
    return &(LIST[STATE(CURTLST)]);
}

/*****  Return Current File Name  *****/
char * CFILENAME()
{
    char *cp;

    if (LIST[STATE(CURTLST)] != NULL && STATE(CURTLST) >= 0)
      if ((cp = (char *)strrchr (LIST[STATE(CURTLST)], '/')) == NULL)
        return LIST[STATE(CURTLST)];
      else
        return cp + 1;
    else
      return " ";
}

/*****  Forward List  *****/
int Forward_List ()
{
    if (STATE(CURTLST) < STATE(LISTNUM) - 1)
      return CHSTATE(CURTLST, STATE(CURTLST) + 1);
    return -1;
}

/*****  Backward List  *****/
int Backward_List ()
{
    if (STATE(CURTLST) >= 0) {
      int secs = Play_Seconds ();
      if (secs < 0) return -1;
      if (secs < SKIPBACKTIME)
        return CHSTATE(CURTLST, STATE(CURTLST) - 1);
      else
        return STATE(CURTLST);
    }
    return -1;
}

/*****  Rewind List  *****/
int Rewind_List ()
{
    if (STATE(LISTNUM) > 0) 
      return CHSTATE(CURTLST, 0);
    return -1;
}

/*****  End_of_List  *****/
int End_of_List ()
{
    if (STATE(LISTNUM) > 0)
      return CHSTATE(CURTLST, STATE(LISTNUM) - 1);
    return -1;
}

/*****  Add List  *****/
int Add_List (indexes)
int indexes;
{
    FNODE node;
    char buf[BUFSIZ];

    if (DATA[indexes].fst)
      node = DATA[indexes];
    else
      return -1;
    if (node.fst == False || STATE(LISTNUM) >= MAX_DATA) return -1;
    if ((LIST[STATE(LISTNUM)]=(String)calloc(sizeof(char),MAXFILENAME))==NULL){
      Put_Message ("Not enough memory.", "  Add_List");
      return -1;
    }
    sprintf (buf, "%s/%s", STRINGS(DIRNAME), node.label);
    OptimizeDirname (LIST[STATE(LISTNUM)], buf);
    LIST[CHSTATE(LISTNUM, STATE(LISTNUM) + 1)] = Null;
    if (STATE(CURTLST) == -1) CHSTATE(CURTLST, 0);
    return STATE(LISTNUM);
}

/*****  Del List  *****/
int Del_List (indexes)
int indexes;
{
    int i;

    if (indexes > STATE(LISTNUM) - 1 || STATE(LISTNUM) <= 0) return -1;

    free (LIST[indexes]);
    for (i = indexes; i < STATE(LISTNUM) - 1; i++)
      LIST[i] = LIST[i + 1];
    CHSTATE(LISTNUM, STATE(LISTNUM) - 1);
    if (STATE(LISTNUM) == 0)
      CHSTATE(CURTLST, -1);
    if (STATE(CURTLST) > STATE(LISTNUM) - 1)
      CHSTATE(CURTLST, STATE(LISTNUM) - 1);
    Reset_List ();
}

/*****  Add All Files  *****/
void Add_All_Files ()
{
    int i;

    for (i = 0;i < STATE(FILENUM);i++)
      Add_List (i);
    Reset_List ();
}

/*****  Delete All Lists  *****/
void Delete_All ()
{
    int i;

    for (i = 0; i < STATE(LISTNUM); i++) {
      free (LIST[i]);
      LIST[i] = Null;
    }
    CHSTATE(LISTNUM, 0);
    CHSTATE(CURTLST, -1);
    Reset_List ();
}

/*****  File Stat  *****/
int filestat (filepath)
char *filepath;
{
    struct stat st;

    if (stat (filepath, &st) != -1) {
      if (S_ISDIR(st.st_mode))
        return S_IFDIR;
      if (S_ISREG(st.st_mode))
        return S_IFREG;
      if (S_ISLNK(st.st_mode))
        return S_IFREG;
      return 0;
    } else
      return -1;
}

/***** Get Current Directory *****/
String GetCurrentDirectory (dummy, str)
char *dummy;
String str;
{
    char buff[BUFSIZ];

    if (strncmp (str, "../", 3) == 0 || strcmp (str, "..") == 0 ||
        strncmp (str, "./", 2)  == 0 || strcmp (str, ".")  == 0) {
      sprintf (buff, "%s/%s", currentdir, str);
      return (String) OptimizeDirname (dummy, buff);
    } else
      return str;
}

/*****  Read File Name and Sorting  *****/
void Read_Sort_Files (dfd)
DIR *dfd;
{
    struct dirent *dp;
    int    i, k, s, l, name_num = 0, dir_num = 0, mode;
    char   *fname, temp[BUFSIZ], card[100];
    String dirs[BUFSIZ], files[BUFSIZ], buf, buf2, str;

    memset(dirs, Null, sizeof(dirs));
    str = (String) GetPattern ();
    if (strlen (str) < 1) str = STRINGS(PATTERN);
    strncpy (card, str, sizeof (card) - 1);
    memset (DATA, 0, sizeof(DATA));
    for (i = 0;MAX_DATA > i && NULL != (dp = readdir (dfd));) {
       if (dp -> d_ino == 0)
         continue;
       sprintf (temp, "%.100s/%.100s", STRINGS(DIRNAME), dp -> d_name);
       mode = filestat(temp);
       if (mode != -1) {
         i++;
         if (mode == S_IFREG) {
           if (wildcard (dp -> d_name, card)) {
             files[name_num] = dp -> d_name;
             if (name_num < MAX_DATA) name_num ++;
             else break;
           }
         } else
         if (mode == S_IFDIR) {
           dirs[dir_num] = (char *)strdup(dp -> d_name);
           if (dir_num  < MAX_DATA) dir_num ++;
           else break;
         }
       }
    }
    for (k = 0;k < dir_num - 1;k++) {
       buf = dirs[k];
       s = k;
       for (i = k + 1; i < dir_num; i++)
         if (strcmp ((char *)dirs[i], (char *)buf) < 0 ) {
           buf = dirs[i];
           s = i;
         }
         buf2 = dirs[s];
         dirs[s] = dirs[k];
         dirs[k] = buf2;
    }

    for (k = 0;k < name_num - 1;k++) {
       buf = files[k];
       s = k;
       for (i = k + 1; i < name_num; i++)
         if (strcmp (files[i], buf) < 0 ) {
           buf = files[i];
           s = i;
         }
         buf2 = files[s];
         files[s] = files[k];
         files[k] = buf2;
    }
    CHSTATE(FILENUM, 0);
    for (i = 0;i < dir_num;i++) {
      sprintf (DATA[i].label, "%.100s/", dirs[i]);
      DATA[i].fst = False;
      names [i] = DATA[i].label;
      free (dirs[i]);
    }

    CHSTATE(FILENUM, (k = i));
    for (i = 0;i < name_num && (k < MAX_DATA);i++) {
      sprintf (DATA[k].label, "%.100s", files[i]);
      DATA[k].fst = True;
      names [k] = DATA[k].label;
      CHSTATE(FILENUM, ++k);
    }
    names[k] = NULL;
}

/*****  Set file name or change directory  *****/
int Set_File_or_Directory (indexes, str)
int indexes;
char *str;
{
    DIR *dfd;
    static struct dirent *dp;
    int i;
    char copied[BUFSIZ], *fname, newname[BUFSIZ], buf[BUFSIZ];

    if (indexes >= 0) {
      fname = str; 
      if (fname [strlen (fname) - 1] == '/' && strcmp (fname, "/") != 0) 
        fname [strlen (fname) - 1] = '\0';
      strcpy (copied, STRINGS(DIRNAME));

      if (0 == strcmp (".", fname)) {/**/}
      else
      if (0 == strcmp ("..", fname)) {
        i = strlen (STRINGS(DIRNAME));
        i = (STRINGS(DIRNAME)[i - 1] == '/' ? i - 2: i - 1);
        for (; 0 <= i; i--) {
           if ('/' == STRINGS(DIRNAME)[i]) {
             if (0 == i) 
               strcpy (STRINGS(DIRNAME), "/");
             else 
               STRINGS(DIRNAME)[i] = '\0';
             break;
           }
        }
      } else {
        if ('/' != STRINGS(DIRNAME)[strlen (STRINGS(DIRNAME)) - 1]) 
          strcat (STRINGS(DIRNAME), "/");
        strcat (STRINGS(DIRNAME), fname);
      }

      if (NULL == (dfd = opendir (STRINGS(DIRNAME)))){
        strcpy (STRINGS(DIRNAME), copied);
        Add_List (indexes);
        return SETFILE;
      } else {
        if (!(STRINGS(DIRNAME) [strlen (STRINGS(DIRNAME)) - 1] == '/' &&
            strcmp (STRINGS(DIRNAME), "/") != 0)) 
          strcat (STRINGS(DIRNAME) ,"/");
        memset (DATA, 0, sizeof (DATA));
        Read_Sort_Files (dfd);
        closedir (dfd);
        return SETDIR;
      }
    }
    return SETNON;
}

/*****  Check Directory  *****/
int CheckDirectory (dirn)
String dirn;
{
    if (filestat (dirn) == S_IFDIR)
      return True;
    else
      return False;
}

/********  Get Argment and Set Current Path  ********/
void FileManager (argc, argv)
int argc;
char **argv;
{
    char temp [BUFSIZ + 1];
    int  i;
    char * str;

    currentdir = (char*)getcwd (NULL, BUFSIZ);

    if (STRINGS(DIRNAME) [0] != NULL)
      if ( ! CheckDirectory (STRINGS(DIRNAME))) {
        STRINGS(DIRNAME)[0] = NULL;
        CHSTATE(DIRF, False);
        CHSTATE(RECDIRF, False);
      } else {
        CHSTATE(DIRF, True);
        CHSTATE(RECDIRF, True);
      }

    if (argc == 2)
      if (CheckDirectory (*(argv + 1)))
        sprintf (STRINGS(DIRNAME), "%s", *(argv + 1));

    strcpy (STRINGS(RECDIR), STRINGS(DIRNAME));
    if (NULL == currentdir) {
      fprintf (stderr, "getwd failed. use root instead\n");
      strcpy (temp, "/");
    } else {
      strcpy (temp, currentdir);
    }

    str = ".";

    if (STRINGS(DIRNAME)[0] == NULL || strcmp (STRINGS(DIRNAME), ".") == 0) {
      CHSTATE(DIRF, True);
      CHSTATE(RECDIRF, True);
      strcpy (STRINGS(DIRNAME), temp);
    }
    if (STRINGS(DIRNAME)[0] == NULL || strcmp (STRINGS(DIRNAME), "..") == 0) {
      CHSTATE(DIRF, True);
      CHSTATE(RECDIRF, True);
      strcpy (STRINGS(DIRNAME), temp);
      str = "..";
    }
    LIST[0] = NULL;
    InitList (str);
}

/*****  Get Current Marked File Configuration  *****/
int Get_Current_File (AO)
Audio_obj *AO;
{
    int  fd, err;
    char *infos, buff[BUFSIZ];
    int  indexes = (int) Current_File_Index ();

    memset (AO, 0, sizeof (Audio_obj));
    sprintf (AO -> filename,"%s/%s", STRINGS(DIRNAME), DATA[indexes].label);

    if (DATA[indexes].fst == False) {
      OptimizeDirname (buff, AO -> filename);
      strcpy (AO -> filename, buff);
      return DIRECTORY;
    }

    if ((fd = open (AO -> filename, O_RDONLY)) == -1) {
      return CANT;
    }
    AO -> ifd = fd;
    err = GetFileConfiguration(AO);
    AO -> ifd = -1;

    if (err != True) {
      close (fd);
      return NOTVALID;
    }

    close (fd);
    return SUCCESS;
}

/***** Return Base name *****/
char * Basename (fullpath)
char * fullpath;
{
    char *bp;

    if ((bp = (char *)strrchr (fullpath, '/')) == NULL)
      return fullpath;
    return bp + 1;
}


/******  Get Home Directory  ******/
String GetHomeDirectory(dirname, str)
char *dirname, *str;
{
    struct passwd *pas = NULL;
    char *cp, username[BUFSIZ];

/*
 *   str may be ..  "~/sounds" or "~user" or "~" ,etc
 */
    if (str[1] == '/' || str[1] == Null) {     /* "~/sounds" or "~" */
      cp = &str[1];
      pas = getpwuid (getuid ());
    } else {                                   
      strcpy (username, &str[1]);
      if ((cp = (char *) strchr (username, '/')) != NULL) { /* "~user/..." */
        *cp = Null;
        pas = getpwnam (username);
        *cp = '/';
      } else {
        cp = "/\0";
        pas = getpwnam (username);
      }
    }
    if (pas == NULL)
      sprintf (dirname, "%s", str);
    else
      sprintf (dirname, "%s%s", pas -> pw_dir, cp);

    return dirname;
} 

/*****  Get Super directory name  *****/
void GetSuperDir (superdirname, notcomplete, dirname)
char *superdirname, *notcomplete;
String dirname;
{
    char *cp;

    strcpy (superdirname, dirname);
    if ((cp = (char *)strrchr (superdirname, '/')) == NULL) {
      strcpy (notcomplete, dirname);
      strcpy (superdirname, "");
      return;
    }
    *cp = Null;
    strcpy (notcomplete, (cp + 1));
}

/***** Search file in the directory *****/
int FindDirectory (finddir, superdirname)
char **finddir;
char *superdirname;
{
    DIR *dfd;
    struct dirent *dp;
    int i, dir_num = 0, file_num = 0, mode;
    char temp[BUFSIZ];

    finddir[0] = NULL;
    if (strlen (superdirname) == 0) strcpy (superdirname, "/");
    if (NULL == (dfd = opendir (superdirname))){
      return False;
    }
    for (i = 0;MAX_DATA > i && NULL != (dp = readdir (dfd));) {
       if (dp -> d_ino == 0)
         continue;
       sprintf (temp, "%.100s/%.100s", superdirname, dp -> d_name);
       mode = filestat(temp);
       if (mode != -1) {
         i++;
         if (mode == S_IFDIR) {
           finddir[dir_num++] = (char *)strdup(dp -> d_name);
           finddir[dir_num] = NULL;
         }
       }
    }
    closedir (dfd);
    return True;
}

/***** Fill strings till match (for file completion) *****/
int FillString (filled, finddir, notcomplete)
char *filled, **finddir, *notcomplete;
{
    char notcomp[BUFSIZ];
    char *args[BUFSIZ];
    int  i, num = 0, fnum = 0, flag = True, fills = 0;
    int  len = 0, mlen = 0, backnum = 0;

    while (finddir[num] != NULL) {
      if (strncmp (finddir[num], notcomplete, strlen (notcomplete)) == 0) {
        args[fnum] = finddir[num];
        if (mlen > (len = strlen (args[fnum]))) {
          backnum = fnum;
          mlen = len;
        }
        fnum++;
      }
      num++;
    }
    args[fnum] = NULL;
    strcpy (filled, notcomplete);
    if (fnum == 0 || strlen(notcomplete) == 0) return False;

    strcpy (notcomp, args[backnum]);
    len = strlen (notcomp);
    fills = strlen (notcomplete);
    while (fills < len && flag) {
       for (i = 0;i < fnum && flag; i++) {
          if (i == backnum) continue;
          if (notcomp[fills] != args[i][fills])
            flag = False;
       }
       fills++;
    }
    strncpy (filled, notcomp, fills);
    if (flag) 
      filled[fills] = Null;
    else
      filled[fills - 1] = Null;
    return flag;
}

/*****  Change 'a/b/../c' -> 'a/c'  *****/
char * parent_dir (bp, cp, optimized)
char * bp, *cp, *optimized;
{
     int len = strlen (optimized);
     char *dp;

     if (cp == optimized) {
       memcpy (bp, bp + 3, len);
     } else {
       *cp = Null;
       if ((dp = (char *)strrchr (optimized, '/')) != NULL) {
         memcpy (dp, cp + 3, len);
         bp = dp;
       } else {
         if (cp - 1 == optimized && *(cp - 1)== '.') { /* ./../ -> ../ */
           memcpy (optimized, cp + 1, len);
           bp = cp + 3;
	 } else {                                 /* a/../b/ -> b/ */
           memcpy (optimized, cp + 4, len);
           bp = cp + 3;
	 }
      }
    }
    return bp;
}

/***** Optimize Directory name ******/
char * OptimizeDirname (optimized, src)
char *optimized, *src;
{
    int i, num = 0, len, flag = False;
    char *cp, *bp, *dp;

    len = strlen (src);
    for (i = 0; i < len; i++) {
      if (src[i] == '/') {
        if (flag)
          continue;
        else {
          flag = True;
          optimized[num++] = src[i];
          continue;
        }
      }
      optimized[num++] = src[i];
      flag = False;
    }
        
    optimized [num] = Null;
    strcpy (optimized, optimized);
    len = strlen (optimized);
    bp = optimized;
    while (*bp != Null) {                      /* delete '/./' */
      if ((cp = (char *)strstr (bp, "/./")) != NULL) {
        memcpy (cp, cp + 2, len);
        len -= 2;
        bp = cp;
      } else break;
    }

    bp = optimized;
    while (*bp != Null) {                      /* Change /../ */
      if ((cp = (char *)strstr (bp, "/../")) != NULL) {
        bp = parent_dir (bp, cp, optimized);
      } else break;
    }

    len = strlen (optimized);
    if (strcmp (optimized + len - 3, "/..") == 0) {
      bp = parent_dir (optimized, optimized + len - 3, optimized);
      strcat (optimized, "/");
    }
    if (strcmp (optimized + len - 2, "/.") == 0)
      *(optimized + len - 1) = Null;

    return optimized;
}

