/*****************************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    filemgr.c  (File Selector)
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 *****************************************************************/

#include "xplay.h"
#include "bitmaps/dismiss.xbm"

Widget  all, view, viewbox, list, rescan;
Widget  dir_l, dir_str, pattern_l, pattern_str, listmark;
Widget  fdismiss, FileBox;

extern Widget TopFile, FilePanel, PopSpecial;
extern Widget rec_dir_s, clist;
static int listClick = -1, fileClick = -1;

/*
 * Prototype
 */
extern void Popdown_Panel ();
extern void Popup_Panel ();
extern void set_pattern ();
extern char *STRINGS();
extern int  STATE();
extern int  CHSTATE();
extern String * NAMESPOINTER();
extern void CheckTextandMark ();
extern String GetHomeDirectory ();
void set_dirname ();
void rescan_proc ();
void all_proc ();
void Play_in_List ();
void SetFileName ();
void ClickFileList ();
static void DoubleClick ();
String GetPattern ();
extern void FileCompletion ();
extern  XtAppContext APPCONTE ();

/*****  Translations  Directory Name text  *****/
static char dirnameTranslations [] =
       "Ctrl<Key>J:    set_dirname() \n\
        Ctrl<Key>O:    set_dirname() \n\
        Ctrl<Key>M:    set_dirname() \n\
        <Key>Return:   set_dirname() \n\
        <Enter>: blink-caret(DIR,True) \n\
        <Leave>: blink-caret(DIR,False) \n\
        <Key>Tab:      FileCompletion() \n\
        Ctrl<Key>I:    FileCompletion()";

/*****  Translations Browser File Name  *****/
static char patternTranslations [] =
       "Ctrl<Key>J:    set_pattern() \n\
        Ctrl<Key>O:    set_pattern() \n\
        Ctrl<Key>M:    set_pattern() \n\
        <Key>Return:   set_pattern() \n\
        <Enter>: blink-caret(BROWSER,True) \n\
        <Leave>: blink-caret(BROWSER,False) ";

/********  File Manager Panel Widgets  ********/
Widget Create_File_Panel (parent)
Widget parent;
{
    Dimension h;

    FileBox   = XtVaCMW ("FileBox",   BoxW,      parent,       NULL);
    FilePanel = XtVaCMW ("FilePanel", FormW,     FileBox, farg,NULL);

    all       = XtVaCMW ("all",       CommandW,  FilePanel, NULL);
    rescan    = XtVaCMW ("rescan",    CommandW,  FilePanel, NULL);
    PopSpecial= XtVaCMW ("special",   ToggleW,   FilePanel, NULL);

    view      = XtVaCMW ("view",      ViewPW,    FilePanel,    NULL);
    viewbox   = XtVaCMW ("viewbox",   BoxW,      view,         NULL);

    list      = XtVaCMW ("list",      ListW,     viewbox,      NULL);

    dir_l     = XtVaCMW ("dir_l",     ToggleW,   FilePanel,    NULL);
    dir_str   = XtVaCMW ("dir_str",   AsciiW,    FilePanel,    NULL);
    pattern_l = XtVaCMW ("pattern_l", ToggleW,   FilePanel,    NULL);
    pattern_str=XtVaCMW ("pattern_str",  AsciiW, FilePanel,    NULL);

    fdismiss = XtVaCMW ("fdismiss", CommandW, FilePanel, cntarg, NULL);
    XtAddCallback (fdismiss,   XtNcallback,   Popdown_Panel, (caddr_t)PFILE);
    XtAddCallback (dir_l,      XtNcallback,   set_dirname,   NULL);
    XtAddCallback (pattern_l,  XtNcallback,   set_pattern,   NULL);
    XtAddCallback (rescan,     XtNcallback,   rescan_proc,   NULL);
    XtAddCallback (all,        XtNcallback,   all_proc,      NULL);
    XtAddCallback (list,       XtNcallback,   ClickFileList, (caddr_t)dir_str);
    XtAddCallback (PopSpecial, XtNcallback,   Popup_Panel,   (caddr_t)PSPEC);

    XtVaGetValues (view, XtNheight, &h, NULL);

    MyOverrideTranslations (dir_str,     dirnameTranslations);
    MyOverrideTranslations (pattern_str, patternTranslations);

    Bitmap (fdismiss, dismiss_bits, dismiss_width, dismiss_height);

    XtAddEventHandler (dir_str, TEXTMASK, False,
                                   CheckTextandMark, (caddr_t)DIRNAME);
    XtAddEventHandler (pattern_str, TEXTMASK,False,
                                   CheckTextandMark, (caddr_t)PATTERN);
    XtVaSetValues (pattern_str, XtNstring, STRINGS(PATTERN), NULL);
    if (strlen (STRINGS(PATTERN)) > 0)
      CHSTATE(PATTERNF, True);
    else
      CHSTATE(PATTERNF, False);
    return FileBox;
}

/********  Rescan Directory  ********/
void rescan_proc (w, params, num_params)
Widget  w;
String  *params;
int     num_params;
{
    XawListReturnStruct  lrs;            /*  List Widget struct     */

    set_dirname (w, NULL);
    set_pattern (w, NULL);
/*
    lrs.string = ".";
    lrs.list_index = 0;
    SetFileName (&lrs);
*/
}

/********  Select All Files  ********/
void all_proc (w, params, num_params)
Widget  w;
String  *params;
int     num_params;
{
    Add_All_Files ();
}

/*****  Callback of List widget  ********/
void ClickFileList (llist, label, lrss)
Widget llist, label;
XawListReturnStruct *lrss;
{
    if (fileClick == lrss -> list_index) {
      SetFileName (lrss);
    }
    fileClick = lrss -> list_index;
    Set_Special ();
    XtAppAddTimeOut (APPCONTE(),DOUBLECLICKINT,DoubleClick,(XtPointer)SETFILE);
}

/********  Set File or Change Directory  *****/
void SetFileName (lrss)
XawListReturnStruct *lrss;
{
    char buff[BUFSIZ];

    change_cur (True, list);
    switch (Set_File_or_Directory (lrss -> list_index, lrss -> string)){
      case SETFILE:
                   XawListHighlight(list, lrss -> list_index);
                   break;
      case SETDIR :
                   OptimizeDirname (buff, STRINGS(DIRNAME));
                   strcpy (STRINGS(DIRNAME), buff);
                   XtVaSetValues(dir_str, XtNstring, STRINGS(DIRNAME), NULL);
                   XawTextSetInsertionPoint(dir_str, strlen(STRINGS(DIRNAME)));
                   XawListChange(list, (String *)NAMESPOINTER(), 0, 0, True);
                   if (STATE(FILENUM) > 0)
                     XawListHighlight(list, 0);
                   fileClick = 0;
                   break;
    }
    Reset_List ();
    Set_View_Location ();
    change_cur (False, list);
}

/*****  Initialize List Widget for File Browser  *****/
void InitList (str)
String str;
{
    XawListReturnStruct  lrs;
    char *buf, i;

    lrs.string = str;
    CHSTATE(CURTLST, -1);
    lrs.list_index = 0;
    buf = STRINGS(RECDIR);
    if ((i = strlen(buf)) > 0)
      if (buf[i - 1] != '/') {
        buf[i] = '/';
        buf[i+1] = Null;
      }
    XtVaSetValues (rec_dir_s, XtNstring, STRINGS(RECDIR), NULL);
    XawTextSetInsertionPoint (rec_dir_s, strlen(STRINGS(RECDIR)));
    SetFileName (&lrs);
}

/********  Set Directroy  ********/
void set_dirname (w, event)
Widget w;
XEvent event;
{
    struct stat sbuf;
    String str;
    int    flag = False;
    char   buf[BUFSIZ], dummy[BUFSIZ];
    XawListReturnStruct  lrs;

    CHSTATE(DIRF, False);
    XtVaGetValues (dir_str, XtNstring, &str, NULL);

    if (str[0] == '~'){                      /* Convert '~' */
      str = (String)GetHomeDirectory (dummy, str);
    } else
    if (str[0] == '.') {
      str = (String)GetCurrentDirectory (dummy, str);
    }
/*
    if (str[strlen(str) - 1] == '/' && strcmp (str, "/") != 0)
       str[strlen(str) - 1] = '\0';
*/
    if (access (str, R_OK) != -1 && strlen(str) > 0) {
      if (stat (str, &sbuf) < 0) {
        strcpy (buf, "Can't stat.");
      } else
      if (S_ISDIR(sbuf.st_mode)) {
        strcpy (STRINGS(DIRNAME), (char *)OptimizeDirname (buf, str));
        lrs.string = ".";
        lrs.list_index = 0;
        SetFileName (&lrs);
        flag = True;
      } else {
        sprintf (buf, "Not a Directory\n %s", str);
      }
    } else {
      sprintf (buf, "No such Directory\n %s", str);
    }

    if (flag == False) 
       popup_errbox (buf, "OK", NULL, NULL, TopFile);
    else
       CHSTATE(DIRF, True);
    strcpy (dummy, STRINGS(DIRNAME));
    OptimizeDirname (STRINGS(DIRNAME), dummy);
    XtVaSetValues (dir_str, XtNstring, STRINGS(DIRNAME), NULL);
    XawTextSetInsertionPoint (dir_str, strlen(STRINGS(DIRNAME)));
    CheckTextandMark (dir_str, DIRNAME, 0);
}

/******/
void set_pattern (w, event)
Widget w;
XEvent event;
{
    XawListReturnStruct  lrs;

    lrs.string = ".";
    lrs.list_index = 0;
    SetFileName (&lrs);
    if (strlen (STRINGS(PATTERN)) > 0)
      CHSTATE(PATTERNF, True);
    else
      CHSTATE(PATTERNF, False);
    strcpy (STRINGS(PATTERN), GetPattern());
    CheckTextandMark (w, PATTERN, 0);
}

/***** Direct Delete a file in the list *****/
void Delete_Mark_File ( w , event, params, nparams)
Widget w;
XEvent *event;
String *params;
int    nparams;
{
    delete_proc (w, THISONE, 0);
}

/*****  Double Click  *****/
static void DoubleClick (TYPE, id)
int TYPE;
XtIntervalId  *id;
{
    if (TYPE == SETLIST)
      listClick = -1;
    else
    if (TYPE == SETFILE)
      fileClick = -1;
}

/***** Direct Play a file in the list *****/
void Mark_File_in_List (llist, label, lrss)
Widget llist, label;
XawListReturnStruct *lrss;
{
    int list_index = lrss -> list_index;

    if (STATE(PRFLAG) & PLAY && STATE(PSTATE) == DOPLAY) {
      if (list_index == STATE(CURTLST))
        stop_proc (0, 0, 0);
      else {
        CHSTATE(CURTLST, list_index);
        FlushOutPut ();
        CHSTATE(PSTATE, NOPLAY);
      };
    }
    CHSTATE(CURTLST, list_index);
    if (listClick == list_index)
      Play_in_List ();
    listClick = list_index;
    Reset_List ();
    XtAppAddTimeOut (APPCONTE(),DOUBLECLICKINT,DoubleClick,(XtPointer)SETLIST);
}

/***** Direct Play a file in the list *****/
void Play_in_List( w , event, params, nparams)
Widget w;
XEvent *event;
String *params;
int    nparams;
{
    int list_index = STATE(CURTLST);

    if (STATE(PRFLAG) & PLAY && STATE(PSTATE) == DOPLAY) {
      if (list_index == STATE(CURTLST))
        stop_proc (0, 0, 0);
      else {
        CHSTATE(CURTLST, list_index);
        FlushOutPut ();
        CHSTATE(PSTATE, NOPLAY);
      };
    } else
    if (!(STATE(PRFLAG) & PLAY)) {
      play_proc (0, 0, 0);
    }
    Reset_List ();
    Set_View_Location ();
}

/*****  Returen File Search Pattern  *****/
String GetPattern ()
{
    String str;

    XtVaGetValues (pattern_str, XtNstring, &str, NULL);
    if (str == NULL || *str == Null) {
      str = "*";
      XtVaSetValues (pattern_str, XtNstring, str, NULL);
    }
    return str;
}

/*****  Return Current File Index  *****/
int Current_File_Index ()
{
    int inde = XawListShowCurrent(list) -> list_index;

    if (inde != fileClick)
      if (inde > 0)
        return inde;
    return inde;
}


