/*************************************************************
 *
 * XPlay: Audio Control Panel            version 2.1
 *
 * File:  gainp.c (Gain Panel: Gain, Balance, Speed, etc)
 *
 * Date:  Sep, 1, 1993
 *
 * By:    Atsushi Hirahara
 *        School of Science and Engineering, Waseda University
 *        hirahara@fuka.info.waseda.ac.jp
 *
 *************************************************************/

#include    "xplay.h"
#include    "bitmaps/thumb.xbm"
#include    "bitmaps/slider.xbm"
#include    "bitmaps/dismiss.xbm"

Widget       play_bar, mon_bar, spd_bar, play_bal;
Widget       play_l, mon_l, spd_l, pbala_l, gdismiss;
Widget       cur_bar, cur_l, GainBox;

extern Widget GainPanel, rec_bar, rec_l, rec_bal, rbala_l;
extern Widget scale_l, scale_bar, trig_bar, trig_l, stedel_bar, stedel_l;

extern void    play_volume_proc ();
extern void    play_balance_proc ();
extern void    record_volume_proc ();
extern void    record_balance_proc ();
extern void    monitor_volume_proc ();
extern void    stereo_delay_proc ();
extern void    play_speed_proc ();
extern void    play_pointer_proc ();
extern void    gain_scale_proc ();

extern void    Popdown_Panel ();
extern void    Thumb ();
extern void    Scroll ();

void    SetBarShowd ();
void    SetScrollbars ();
void    Pointer_Type ();
Widget  Create_Gain_Panel ();

/*******  Gain Slider Panel Widgets  ********/
Widget Create_Gain_Panel (parent)
Widget parent;
{
    GainBox   = XtVaCMW ("GainBox",   BoxW,      parent,          NULL);
    GainPanel = XtVaCMW ("GainPanel", FormW,     GainBox  ,       NULL);
    play_l    = XtVaCMW ("play_l",    LabelW,    GainPanel, larg, NULL);
    play_bar  = XtVaCMW ("play_bar",  ScrollW,   GainPanel, barg, NULL);
#ifndef NOBALANCE
    pbala_l   = XtVaCMW ("pbala_l",   LabelW,    GainPanel, larg, NULL);
    play_bal  = XtVaCMW ("play_bal",  ScrollW,   GainPanel, barg, NULL);
    mon_l     = XtVaCMW ("mon_l",     LabelW,    GainPanel, larg, NULL);
    mon_bar   = XtVaCMW ("mon_bar",   ScrollW,   GainPanel, barg, NULL);
#else
    if (STATE(PROCESS) == DBRI) {
      pbala_l   = XtVaCMW ("pbala_l",   LabelW,    GainPanel, 
                           XtNfromVert, play_l,   larg, NULL);
      play_bal  = XtVaCMW ("play_bal",  ScrollW,   GainPanel,
                           XtNfromVert, play_bar, barg, NULL);
      mon_l     = XtVaCMW ("mon_l",     LabelW,    GainPanel, 
                           XtNfromVert, pbala_l, larg, NULL);
      mon_bar   = XtVaCMW ("mon_bar",   ScrollW,   GainPanel, 
                           XtNfromVert, play_bal,barg, NULL);
      XtAddCallback (play_bal, XtNjumpProc,   Thumb,    PLAYB);
      XtAddCallback (play_bal, XtNscrollProc, Scroll,   PLAYB);
    } else {
      mon_l     = XtVaCMW ("mon_l",     LabelW,    GainPanel,
                           XtNfromVert, play_l,   larg, NULL);
      mon_bar   = XtVaCMW ("mon_bar",   ScrollW,   GainPanel,
                           XtNfromVert, play_bar, barg, NULL);
    }
#endif
    spd_l     = XtVaCMW ("spd_l",     LabelW,    GainPanel, larg, NULL);
    spd_bar   = XtVaCMW ("spd_bar",   ScrollW,   GainPanel, barg, NULL);
    cur_l     = XtVaCMW ("cur_l",     CommandW,  GainPanel, larg, NULL);
    cur_bar   = XtVaCMW ("cur_bar",   ScrollW,   GainPanel, barg, NULL);
    if (STATE(SPLIT)){
      gdismiss  = XtVaCMW ("gdismiss",  CommandW,  GainPanel, cntarg, NULL);
      XtAddCallback (gdismiss, XtNcallback,   Popdown_Panel,  (caddr_t)PGAIN);
    } else {
      XtVaSetValues (play_bar, XtNhorizDistance, BARDISTANCE, NULL);
      XtVaSetValues (play_bal, XtNhorizDistance, BARDISTANCE, NULL);
      XtVaSetValues (mon_bar,  XtNhorizDistance, BARDISTANCE, NULL);
      XtVaSetValues (spd_bar,  XtNhorizDistance, BARDISTANCE, NULL);
      XtVaSetValues (cur_bar,  XtNhorizDistance, BARDISTANCE, NULL);
    }
    if (STATE(PROCESS) == DBRI) {
      XtAddCallback (play_bal, XtNjumpProc,   Thumb,    (caddr_t)PLAYB);
      XtAddCallback (play_bal, XtNscrollProc, Scroll,   (caddr_t)PLAYB);
    }
    XtAddCallback (play_bar, XtNjumpProc,   Thumb,    (caddr_t)PLAYV);
    XtAddCallback (mon_bar,  XtNjumpProc,   Thumb,    (caddr_t)MONV );
    XtAddCallback (spd_bar,  XtNjumpProc,   Thumb,    (caddr_t)PLAYS);
    XtAddCallback (cur_bar,  XtNjumpProc,   Thumb,    (caddr_t)PLAYP);
    XtAddCallback (play_bar, XtNscrollProc, Scroll,   (caddr_t)PLAYV);
    XtAddCallback (mon_bar,  XtNscrollProc, Scroll,   (caddr_t)MONV );
    XtAddCallback (spd_bar,  XtNscrollProc, Scroll,   (caddr_t)PLAYS);
    XtAddCallback (cur_bar,  XtNscrollProc, Scroll,   (caddr_t)PLAYP);
    XtAddCallback (cur_l,    XtNcallback,   Pointer_Type, NULL);

    if (STATE(SPLIT)) {
      Bitmap (gdismiss, dismiss_bits, dismiss_width, dismiss_height);
    }

    return GainBox;
}

/*****  Create pixmap From Bitmap Data   *****/
void CreatePFBD (w)
Widget w;
{
    unsigned long  fg, bg;

    XtVaGetValues (w, XtNforeground, &fg, XtNbackground, &bg, NULL);
    XtVaSetValues (w, XtNbackgroundPixmap, 
                      XCreatePixmapFromBitmapData (XtDisplay (w), 
                          XtScreen (w) -> root, (char *) slider_bits,
                          slider_width, slider_height, fg, bg,
                          XDefaultDepth (XtDisplay (w), 0)), NULL);
    XtVaSetValues (w, XtNthumb, 
                      XCreateBitmapFromData (XtDisplay (w), 
                          XtScreen (w) -> root, (char *) thumb_bits,
                          thumb_width, thumb_height) ,NULL);
/*
                      XCreatePixmapFromBitmapData (XtDisplay (w), 
                          XtScreen (w) -> root, (char *) thumb_bits,
                          thumb_width, thumb_height, fg, bg,
                          XDefaultDepth (XtDisplay (w), 0)), NULL);
*/
}

/*****  Create slider pixmap  *****/
void Create_Slider_Pixmap (type)
int type;
{
    static int virg = PNON;

    if ((type & PGAIN) && ! (virg & PGAIN)) {
      CreatePFBD (play_bar);
      CreatePFBD (play_bal);
      CreatePFBD (mon_bar);
      CreatePFBD (spd_bar);
      CreatePFBD (cur_bar);
      virg |= PGAIN;
    }
    if ((type & PREC) && ! (virg & PREC)){
      CreatePFBD (trig_bar);
      CreatePFBD (rec_bar);
      CreatePFBD (rec_bal);
      virg |= PREC;
    }
    if ((type & PSPEC) && ! (virg & PSPEC)){
      CreatePFBD (scale_bar);
      virg |= PSPEC;
    }
    if ((type & PCONF) && ! (virg & PCONF)){
      CreatePFBD (stedel_bar);
      virg |= PCONF;
    }
}

/******************************************************************/
/********  Scroll Proc (Left or Right mouse button pushed) ********/
/******************************************************************/
void Scroll (w, Type, position)
Widget  w;
int     Type;
int     position;
{
    int    cpos;
    static float per;

    cpos = STATE(Type) / (float) MAX_GAIN * END;

    if (position <= 0) {
      cpos += SKIP;
    } else {
      cpos -= SKIP;
    }

    if (cpos < 0)
      cpos = BEGIN;
    else 
    if (cpos > END)
      cpos = END;

    per = (float )cpos / END;

    switch (Type) {
      case PLAYV : play_volume_proc (per);       break;
      case PLAYB : play_balance_proc (per);      break;
      case RECV  : record_volume_proc (per);     break;
      case RECB  : record_balance_proc (per);    break;
      case MONV  : monitor_volume_proc (per);    break;
      case PLAYS : play_speed_proc (per);        break;
      case PLAYP : play_pointer_proc (per);      break;
      case SCALE : gain_scale_proc (per);        break;
      case TRIGL : trigger_level_proc (per);     break;
      case STEREODEL: stereo_delay_proc (per);      break;
    };
}

/********  Grab Thumb (Middle mouse button pushed)  ********/
void Thumb (w, Type, ratio)
Widget   w;
int      Type;
float    *ratio;
{
    switch (Type) {
      case PLAYV : play_volume_proc ((*ratio)*HOSEI);          break;
      case RECV  : record_volume_proc ((*ratio)*HOSEI);        break;
      case MONV  : monitor_volume_proc ((*ratio)*HOSEI);       break;
      case PLAYS : play_speed_proc ((*ratio)*HOSEI);           break;
      case PLAYB : play_balance_proc ((*ratio)*HOSEI);         break;
      case PLAYP : play_pointer_proc ((*ratio)*HOSEI);         break;
      case RECB  : record_balance_proc ((*ratio)*HOSEI);       break;
      case SCALE : gain_scale_proc ((*ratio)*HOSEI);           break;
      case TRIGL : trigger_level_proc ((*ratio)*HOSEI);        break;
      case STEREODEL: stereo_delay_proc ((*ratio)*HOSEI);         break;
    }
}

/*******  Set Slider Position and label  ********/
void SetBarShowd (barw, labw, pos, str)
Widget barw, labw;
int  pos;
char *str;
{
    SetThumb (barw, pos);
    XtVaSetValues (labw, XtNlabel, str, NULL);
}

/*****  Set Scrollbar Thumbs and Label   *****/
SetDefaultScrollbars ()
{
    SetThumb (play_bar, 50);
    SetThumb (rec_bar , 50);
    SetThumb (mon_bar , 50);
    SetThumb (spd_bar , 50);
    SetThumb (cur_bar ,  0);
    SetThumb (scale_bar,45);
    SetThumb (trig_bar, 10);
#ifndef NOBALANCE
    SetThumb (play_bal, 45);
    SetThumb (rec_bal , 45);
    SetThumb (stedel_bar, 60);
    if (STATE(PROCESS) == AMD) {
      XtVaSetValues (pbala_l,   XtNsensitive, False, NULL);
      XtVaSetValues (play_bal,  XtNsensitive, False, NULL);
      XtVaSetValues (rbala_l,   XtNsensitive, False, NULL);
      XtVaSetValues (rec_bal,   XtNsensitive, False, NULL);
      XtVaSetValues (stedel_l,  XtNsensitive, False, NULL);
      XtVaSetValues (stedel_bar,XtNsensitive, False, NULL);
    }
#endif
    SetScrollbars (TRIGL,  STATE(TRIGL));
    SetScrollbars (STEREODEL, STATE(STEREODEL));
/*
    XawScrollbarSetThumb (play_bar, 0.50, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (rec_bar , 0.50, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (mon_bar , 0.50, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (spd_bar , 0.50, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (cur_bar , 0.00, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (scale_bar,0.45, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (trig_bar, 0.10, (float) SHOWN / LENGTH / 2);
#ifndef NOBALANCE
    XawScrollbarSetThumb (play_bal, 0.455, (float) SHOWN / LENGTH / 2);
    XawScrollbarSetThumb (rec_bal , 0.455, (float) SHOWN / LENGTH / 2);
#endif
*/
}

/*****   Set Scrollbar Thumb   *****/
void SetScrollbars (type, value)
int type;
int value;
{
    char buff[50];

    switch (type) {
      case PLAYV: 
                  sprintf (buff, "Play    Gain:   %3d ", value);
                  SetBarShowd (play_bar, play_l, STATE(PLAYV), buff);
                  break;
      case RECV :
                  sprintf (buff, "Record  Gain:  %3d", value);
                  SetBarShowd (rec_bar, rec_l, STATE(RECV), buff);
                  break;
      case PLAYB:
                  sprintf (buff, "Balance     :   %3d ", value);
                  SetBarShowd (play_bal, pbala_l, STATE(PLAYB), buff);
                  break;
      case RECB :
                  sprintf (buff, "Balance     :  %3d", value);
                  SetBarShowd (rec_bal, rbala_l, STATE(RECB), buff);
                  break;
      case MONV :
                  sprintf (buff, "Monitor Gain:   %3d ", value);
                  SetBarShowd (mon_bar, mon_l, STATE(MONV), buff);
                  break;
      case PLAYS: {
                  float per;
                  int scale = GetScale (value);

                  if (scale < 0)
                    per = 2.0 / (float)(-scale + 2);
                  else
                    per = (float)(scale + 2) / 2.0;
                  sprintf (buff, "Play   Speed:  %3.2f ", per);
                  SetBarShowd (spd_bar, spd_l, STATE(PLAYS), buff);
	          };
                  break;
      case PLAYP:
                  if (STATE(PTYPE) == PERCENT)
                    sprintf (buff, "Play Pointer:   %3d ", value);
                  else {
                    char str[20];
                    millsecs_to_time_str ( value * 1000, str);
                    sprintf (buff, "Play    Time: %5.4s ", str);
		  }
                  SetBarShowd (cur_bar, cur_l, STATE(PLAYP), buff);
                  break;
      case SCALE:
                  sprintf (buff, "Play Gain Scale: %3d %%", 
                                       (int)(value*MAX_SCALE/(float)MAX_GAIN));
                  SetBarShowd (scale_bar, scale_l, STATE(SCALE), buff);
                  break;
      case TRIGL:
                  sprintf (buff, "Trigger Gain:  %3d", value);
                  SetBarShowd (trig_bar, trig_l, STATE(TRIGL), buff);
                  break;
      case STEREODEL:
                  sprintf (buff, "Semi Stereo Delay  : %+4d", 
                                (value - MAX_GAIN/2) * MAX_STEREODELAY / 50);
                  SetBarShowd (stedel_bar, stedel_l, STATE(STEREODEL), buff);
                  break;
    }
}

/***** Set Pointer Type (Pointer or Time format) *****/
void Pointer_Type (w, data, num_data)
Widget w;
caddr_t data, num_data;
{
    CHSTATE(PTYPE,(STATE(PTYPE) == PERCENT ? TIME : PERCENT));
    SetScrollbars (PLAYP, STATE(PLAYP));
}
