/*************************************************************
 *
 * XPlay: Audio Control Panel            version 2.1
 *
 * File:  recordp.c (Record Panel)
 *
 * Date:  Sep, 1, 1993
 *
 * By:    Atsushi Hirahara
 *        School of Science and EnGineering, Waseda University
 *        hirahara@fuka.info.waseda.ac.jp
 *
 *************************************************************/

#include     "xplay.h"

#include "bitmaps/record.xbm"
#include "bitmaps/pause.xbm"
#include "bitmaps/stop.xbm"
#include "bitmaps/trigger.xbm"
#include "bitmaps/popdown.xbm"

Widget rec_pause,  rec_start,  rec_stop, rec_down, trigger;
Widget rec_samp_l, rec_samp_s, rec_in_l,   rec_in_s;
Widget rec_chnl_l, rec_chnl_s, rec_size_l, rec_size_s;
Widget rec_dir_l, rec_dir_s;
Widget rec_file_l, rec_file_s, rec_info_l, rec_info_s;
Widget rec_time_l, rec_time_s, rec_limit_l, rec_limit_s;
Widget rec_bar, rec_l, rec_bal, rbala_l, trig_bar, trig_l, RecordBox;

extern Widget RecPanel, PopRecord, TopRecord;

/* 
 * Prototypes
 */

extern void Popdown_Panel ();
extern void Thumb ();
extern void Scroll ();
extern void pause_proc ();
extern int  filestat ();
extern void CheckTextandMark ();
extern String GetHomeDirectory ();
void   SetRecTriggerButton ();
void   Put_File_Size ();
void   set_reclimit ();
void   set_recfilename ();
void   set_recdirname ();
void   set_recinfo ();
void   record_proc ();
void   rec_stop_proc ();
void   trigger_proc ();
void  CheckAllTextMarking ();
Boolean CheckLimitTime ();
Boolean IsDirectory ();
Boolean CheckRecordFilename ();
extern  void FileCompletion();
extern  XtAppContext APPCONTE ();
extern  char * STRINGS ();

/*****  Translations Record Directory Name text  *****/
static char recdirnameTranslations [] =
       "Ctrl<Key>J:    set_recdirname() \n\
        Ctrl<Key>O:    set_recdirname() \n\
        Ctrl<Key>M:    set_recdirname() \n\
        <Key>Return:   set_recdirname() \n\
        <Enter>: blink-caret(RECDIR,True) \n\
        <Leave>: blink-caret(RECDIR,False) \n\
        <Key>Tab:      FileCompletion() \n\
        Ctrl<Key>I:    FileCompletion()";

/*****  Translations  Save File Name text  *****/
static char filenameTranslations [] =
       "Ctrl<Key>J:    set_recfilename() \n\
        Ctrl<Key>O:    set_recfilename() \n\
        Ctrl<Key>M:    set_recfilename() \n\
        <Key>Return:   set_recfilename() \n\
        <Enter>: blink-caret(RFILE,True) \n\
        <Leave>: blink-caret(RFILE,False) ";

/*****  Translations  Save File Name text  *****/
#ifdef MULTI                            /** For multi lingual **/
static char fileinfoTranslations [] =
       "Ctrl<Key>J:    set_fileinfo() \n\
        Ctrl<Key>O:    set_fileinfo() \n\
        Ctrl<Key>M:    set_fileinfo() \n\
        <Key>Return:   set_fileinfo() \n\
        <Enter>:  blink-caret(RINFO,True) focus-in() \n\
        <Leave>:  blink-caret(RINFO,False) focus-out()";
#else
static char fileinfoTranslations [] =
       "Ctrl<Key>J:    set_fileinfo() \n\
        Ctrl<Key>O:    set_fileinfo() \n\
        Ctrl<Key>M:    set_fileinfo() \n\
        <Key>Return:   set_fileinfo() \n\
        <Enter>:  blink-caret(RINFO,True)  \n\
        <Leave>:  blink-caret(RINFO,False)";
#endif

/*****  Translations  Save limit Time  *****/
static char reclimitTranslations [] =
       "Ctrl<Key>J:    set_reclimit() \n\
        Ctrl<Key>O:    set_reclimit() \n\
        Ctrl<Key>M:    set_reclimit() \n\
        <Key>Return:   set_reclimit() \n\
        <Enter>: blink-caret(RLIMIT,True) \n\
        <Leave>: blink-caret(RLIMIT,False) ";

/********  Recoding Configration Panel  ********/
Widget Create_Record_Panel (parent)
Widget parent;
{
    RecordBox    = XtVaCMW ("RecordBox",  BoxW,      parent,             NULL);
    RecPanel     = XtVaCMW ("RecPanel",   FormW,     RecordBox,  farg,   NULL);
    rec_start    = XtVaCMW ("rec_start",  ToggleW,   RecPanel,   cntarg, NULL);
    trigger      = XtVaCMW ("trigger",    ToggleW,   RecPanel,   cntarg, NULL);
    rec_stop     = XtVaCMW ("rec_stop",   CommandW,  RecPanel,   cntarg, NULL);
    rec_pause    = XtVaCMW ("rec_pause",  ToggleW,   RecPanel,   cntarg, NULL);
    rec_down     = XtVaCMW ("rec_down",   CommandW,  RecPanel,   cntarg, NULL);
    trig_l       = XtVaCMW ("trig_l",     LabelW,    RecPanel,   recarg, NULL);
    trig_bar     = XtVaCMW ("trig_bar",   ScrollW,   RecPanel,   barg,   NULL);
    rec_l        = XtVaCMW ("rec_l",      LabelW,    RecPanel,   recarg, NULL);
    rec_bar      = XtVaCMW ("rec_bar",    ScrollW,   RecPanel,   barg,   NULL);
#ifndef NOBALANCE
    rbala_l    = XtVaCMW ("rbala_l",    LabelW,    RecPanel,   recarg, NULL);
    rec_bal    = XtVaCMW ("rec_bal",    ScrollW,   RecPanel,   barg,   NULL);
    rec_samp_l = XtVaCMW ("rec_samp_l", LabelW,    RecPanel,   recarg, NULL);
    rec_samp_s = XtVaCMW ("rec_samp_s", LabelW,    RecPanel,   recarg, NULL);
#else
    if (STATE(PROCESS) == DBRI) {
      rbala_l    = XtVaCMW ("rbala_l",    LabelW,    RecPanel,
                            XtNfromVert,  rec_l,     recarg,   NULL);
      rec_bal    = XtVaCMW ("rec_bal",    ScrollW,   RecPanel,
                            XtNfromVert,  rec_bar,   barg,   NULL);
      rec_samp_l = XtVaCMW ("rec_samp_l", LabelW,    RecPanel,
                            XtNfromVert,  rbala_l,   recarg, NULL);
      rec_samp_s = XtVaCMW ("rec_samp_s", LabelW,    RecPanel,
                            XtNfromVert,  rec_bal,   recarg, NULL);
    } else {
      rec_samp_l = XtVaCMW ("rec_samp_l", LabelW,    RecPanel,
                            XtNfromVert,  rec_l,     recarg, NULL);
      rec_samp_s = XtVaCMW ("rec_samp_s", LabelW,    RecPanel,
                            XtNfromVert,  rec_bar,   recarg, NULL);
    }
#endif
    rec_in_l     = XtVaCMW ("rec_in_l",   LabelW,    RecPanel,   recarg, NULL);
    rec_in_s     = XtVaCMW ("rec_in_s",   LabelW,    RecPanel,   recarg, NULL);
    rec_chnl_l   = XtVaCMW ("rec_chnl_l", LabelW,    RecPanel,   recarg, NULL);
    rec_chnl_s   = XtVaCMW ("rec_chnl_s", LabelW,    RecPanel,   recarg, NULL);
    rec_size_l   = XtVaCMW ("rec_size_l", LabelW,    RecPanel,   recarg, NULL);
    rec_size_s   = XtVaCMW ("rec_size_s", LabelW,    RecPanel,   recarg, NULL);
    rec_time_l   = XtVaCMW ("rec_time_l", LabelW,    RecPanel,   recarg, NULL);
    rec_time_s   = XtVaCMW ("rec_time_s", LabelW,    RecPanel,   recarg, NULL);
    rec_limit_l  = XtVaCMW ("rec_limit_l",ToggleW,   RecPanel,   NULL);
    rec_limit_s  = XtVaCMW ("rec_limit_s",AsciiW,    RecPanel,   NULL);
    rec_dir_l    = XtVaCMW ("rec_dir_l",  ToggleW,   RecPanel,   NULL);
    rec_dir_s    = XtVaCMW ("rec_dir_s",  AsciiW,    RecPanel,   NULL);
    rec_file_l   = XtVaCMW ("rec_file_l", ToggleW,   RecPanel,   NULL);
    rec_file_s   = XtVaCMW ("rec_file_s", AsciiW,    RecPanel,   NULL);
    rec_info_l   = XtVaCMW ("rec_info_l", ToggleW,   RecPanel,   NULL);
#ifdef MULTI
    if (STATE(LANGF))
      rec_info_s   = XtVaCMW ("rec_info_s", MultiW,    RecPanel,   NULL);
    else
      rec_info_s   = XtVaCMW ("rec_info_s", AsciiW,    RecPanel,   NULL);
#else
    rec_info_s   = XtVaCMW ("rec_info_s", AsciiW,    RecPanel,   NULL);
#endif
    if (STATE(PROCESS) == DBRI) {
      XtAddCallback (rec_bal,    XtNjumpProc,   Thumb,    (caddr_t)RECB);
      XtAddCallback (rec_bal,    XtNscrollProc, Scroll,   (caddr_t)RECB);
    }
    XtAddCallback(rec_file_l, XtNcallback,set_recfilename,(caddr_t)rec_file_l);
    XtAddCallback(rec_limit_l,XtNcallback,set_reclimit,  (caddr_t)rec_limit_l);
    XtAddCallback(rec_dir_l,  XtNcallback,set_recdirname, NULL);
    XtAddCallback(rec_info_l, XtNcallback,set_recinfo,    NULL);
    XtAddCallback(rec_down,   XtNcallback,Popdown_Panel,  (caddr_t)PREC);
    XtAddCallback(rec_start,  XtNcallback,record_proc,    (caddr_t)TopRecord);
    XtAddCallback(rec_stop,   XtNcallback,rec_stop_proc,  (caddr_t)TopRecord);
    XtAddCallback(rec_pause,  XtNcallback,pause_proc,     (caddr_t)RECORD);
    XtAddCallback(trigger,    XtNcallback,trigger_proc,   (caddr_t)True);
    XtAddCallback(rec_bar,    XtNjumpProc,Thumb,          (caddr_t)RECV);
    XtAddCallback(rec_bar,    XtNscrollProc, Scroll,      (caddr_t)RECV);
    XtAddCallback(trig_bar,   XtNjumpProc,Thumb,          (caddr_t)TRIGL);
    XtAddCallback(trig_bar,   XtNscrollProc, Scroll,      (caddr_t)TRIGL);

    Bitmap (rec_down, popdown_bits, popdown_width, popdown_height);
    Bitmap (rec_start, record_bits, record_width, record_height);
    Bitmap (rec_stop, stop_bits, stop_width, stop_height);
    Bitmap (rec_pause, pause_bits, pause_width, pause_height);
    Bitmap (trigger, trigger_bits, trigger_width, trigger_height);

    MyOverrideTranslations (rec_dir_s,   recdirnameTranslations);
    MyOverrideTranslations (rec_file_s,  filenameTranslations);
    MyOverrideTranslations (rec_limit_s, reclimitTranslations);
    MyOverrideTranslations (rec_info_s,  fileinfoTranslations);

    XtVaSetValues (rec_dir_s, XtNstring, STRINGS(RECDIR), NULL);

    XtAddEventHandler (rec_dir_s, TEXTMASK, False,
                                  CheckTextandMark, (caddr_t)RECDIR);
    XtAddEventHandler (rec_file_s, TEXTMASK, False,
                                  CheckTextandMark, (caddr_t)RECFILE);
    XtAddEventHandler (rec_info_s, TEXTMASK, False, 
                                  CheckTextandMark, (caddr_t)INFOSTR);
    XtAddEventHandler (rec_limit_s, TEXTMASK, False,
                                  CheckTextandMark, (caddr_t)LIMITSTR);
    return RecordBox;
}

/*****  Popup error box on TopRecord  *****/
void record_errbox (str)
char *str;
{
    popup_errbox (str, "OK", NULL, NULL, TopRecord);
}

/******  Get Information String  ******/
String GetInfoString ()
{
    String Str;

    XtVaGetValues (rec_info_s, XtNstring, &Str, NULL);
    Str [MAX_INFO] = Null;
    return Str;
}

/********  Put Trigger Waiting  *******/
void Put_Trigger_Waiting ()
{
    String mesg = "   Trigger Waiting... ";
    XtVaSetValues (rec_size_s, XtNlabel, mesg, NULL);
    XtVaSetValues (rec_time_s, XtNlabel, mesg, NULL);
}

/********  Put Recording Current File Size  ********/
void Put_File_Size (size, timec, err1)
int size, timec, err1;
{
    char buff[80];
    unsigned long int tm, ts, tms, total;

    total = size / timec;
    tm    = total / 60;
    ts    = total - tm * 60;
    tms   = ((size * 100) / timec) % 100;
    sprintf (buff, "%d Bytes  ", size);
    XtVaSetValues (rec_size_s, XtNlabel, buff, NULL);
    sprintf (buff, "%02u:%02u.%02u  ", tm, ts, tms);
    if (err1 == True) {
      strcat (buff, " Data overflow!");
      XBell(XtDisplay (rec_time_s), 0);
    }
    XtVaSetValues (rec_time_s, XtNlabel, buff, NULL);
}

/********  Record Start  ********/
void record_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    char buff[BUFSIZ];
    String str;
    int  flag = False;

    if ( ! (STATE(PRFLAG) & RECORD) ) {
      if (IsDirectory (buff) == True) {
         if (CheckRecordFilename (buff) == True) {
            if (STATE(RFNAME)) {
               if (CheckLimitTime(buff) == True)
                 CHSTATE(PRFLAG, (STATE(PRFLAG) | RECORD));
              strcpy (STRINGS(INFOSTR), GetInfoString ());
            }
         } else
           popup_errbox (buff, "OK", NULL, NULL, w);
      } else
        popup_errbox (buff, "OK", NULL, NULL, w);
    }

    if (STATE(PRFLAG) & RECORD) {
      if (STATE(RSTATE) == NOREC) {
        if (STATE(PAUSEF) & RECORD) {
          Put_Message (" Record pausing :", STRINGS(RECFILE));
          set_recpause (True);
        }
        XtVaSetValues (rec_start, XtNstate, True, NULL);
      } else
       XtVaSetValues (rec_start, XtNstate, True, NULL);
    } else 
      XtVaSetValues (rec_start, XtNstate, False, NULL);
}

/*****  Record Stop  *****/
void rec_stop_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    XtVaSetValues (rec_start, XtNstate, False, NULL);
    CHSTATE(RECTRIGGER, NOUSE);
    SetRecTriggerButton ();
    if (STATE(RSTATE) == DOREC) {
      CHSTATE(RSTATE, ENDREC);
      RecordMain ();
      return;
    }
    CHSTATE(RFNAME, False);
    if ( ! STATE(PRFLAG) )
      Put_Message (DEMO, Non);
    CHSTATE(PRFLAG, (STATE(PRFLAG) & ~RECORD));
    FlushInPut (); 
    if ( STATE(PAUSEF) & RECORD ) {
      set_recpause (False);
    }
    CheckAllTextMarking ();
}

/***** Set Record Trigger Button  *****/
void SetRecTriggerButton ()
{
    if (STATE(RECTRIGGER) == WAITING)
      XtVaSetValues (trigger, XtNstate, True, NULL);
    else
      XtVaSetValues (trigger, XtNstate, False, NULL);
}

/*****  Record Trigger Porc  *****/
void trigger_proc (w, params, num_params)
Widget  w;
int     params;
int     num_params;
{
    if (STATE(RECTRIGGER) == WAITING) {
      CHSTATE(RECTRIGGER, NOUSE);
    } else
    if (STATE(RECTRIGGER) == NOUSE) {
      CHSTATE(RECTRIGGER, WAITING);
    }
    SetRecTriggerButton ();
}

/******  Check Record Limit Time  ******/
Boolean CheckLimitTime (mesg)
char *mesg;
{
    String str;
    char buff[100];
    int flag;
    long limit_time = STATE(LIMTTIM);

    XtVaGetValues (rec_limit_s, XtNstring, &str, NULL);
    if (strlen (str) > 0)
      if (strncmp (str, NONSET, sizeof (NONSET)) == 0)
        limit_time = 0;                        /** Unset string **/
      else                                     /** Try to convert str **/
        limit_time = time_str_to_millsecs (str);
    else
      limit_time = 0;                     /** Null string **/

    if (limit_time < 0) {
      popup_errbox ("  Invalid value for limit time. ",
                    "OK", NULL, NULL, TopRecord);
      strcpy (buff, NONSET);
      flag = False;
    } else {
      if (limit_time == 0)
        sprintf (buff, NONSET);
      else
        millsecs_to_time_str (limit_time, buff);
      XtVaSetValues (rec_limit_s, XtNstring, buff, NULL);
      flag = True;
    }
    if (strcmp (buff, NONSET) == 0)
      CHSTATE(RLIMITF, False);
    else
      CHSTATE(RLIMITF, True);
    CHSTATE(LIMTTIM, limit_time);
    strcpy (STRINGS(LIMITSTR), buff);
    return flag;
}

/******  Set Record Limit Time  ******/
void set_reclimit (w, event)
Widget w;
XEvent event;
{
    char buff[BUFSIZ];

    if (CheckLimitTime(buff) == True)
      Move_Cursor (TopRecord, rec_dir_s,  Widget_Width (rec_dir_s) - 10, 5);
    CheckTextandMark (w, LIMITSTR, 0);
}

/********  Is this directory?  ********/
Boolean IsDirectory (mesg)
char *mesg;
{
    String str;
    int    flag = False, mode, i;
    char   dummy[BUFSIZ], buff[BUFSIZ];

    XtVaGetValues (rec_dir_s, XtNstring, &str, NULL);
    strcpy (buff, str);
    if (strlen (buff) == 0)
      strcpy (buff, ".");
    if (buff[0] == '~'){                      /* Convert '~' */
      strcpy (buff, GetHomeDirectory (dummy, buff));
    }
    if ((i = strlen(buff)) > 0)
      if (buff[i-1] != '/') {
        buff[i] = '/';
        buff[i+1] = Null;
      }
    mode = filestat(buff);
    if (mode == S_IFDIR) {
      strcpy (STRINGS(RECDIR), buff);
      XtVaSetValues (rec_dir_s, XtNstring, STRINGS(RECDIR), NULL);
      XawTextSetInsertionPoint (rec_dir_s, strlen(STRINGS(RECDIR)));
      flag = True;
    } else
    if (mode == S_IFREG)
      sprintf (mesg, "Not a Directory\n %s", buff);
    else
      sprintf (mesg, "No such Directory\n %s", buff);

    CHSTATE(RECDIRF, flag);
    CheckTextandMark (NULL, RECDIR, 0);
    return flag;
}

/******  Set Record Information  *******/
void set_recinfo (w, event)
Widget w;
XEvent event;
{
    CheckTextandMark (w, INFOSTR, 0);
}

/********  Set Record Directroy  ********/
void set_recdirname (w, event)
Widget w;
XEvent event;
{
    char buf[BUFSIZ];

    if (IsDirectory (buf) == False) {
      popup_errbox (buf, "OK", NULL, NULL, TopRecord);
    } else {
       strcpy (buf, STRINGS(RECDIR));
       OptimizeDirname (STRINGS(RECDIR), buf);
       XtVaSetValues (rec_dir_s, XtNstring, STRINGS(RECDIR), NULL);
       XawTextSetInsertionPoint (rec_dir_s, strlen(STRINGS(RECDIR)));
       Move_Cursor (TopRecord, rec_file_s,  Widget_Width (rec_file_s) - 10, 5);
    }
    CheckTextandMark (w, RECDIR, 0);
}

/******  Overwrite ?  ******/
Boolean CheckRecordFilename (mesg)
char *mesg;
{
    String str;
    int    flag = False;
    char   buf[BUFSIZ], buf2[BUFSIZ];

    sprintf (buf2, "%s/%s", STRINGS(RECDIR), STRINGS(RECFILE));
    XtVaGetValues (rec_file_s, XtNstring, &str, NULL);

    if (strlen (str) == 0) {
        sprintf (mesg, "File name is required.");
        flag = False;
    } else
    if (IsDirectory (mesg) == True) {
      sprintf (buf, "%s/%s", STRINGS(RECDIR), str);

      if (STATE(RFNAME)) {                     /** Record file name flag  **/
        if (strcmp (buf, buf2) == 0)
          flag = True;                         /** Already checked        **/
      } 
      if (flag == False) {
        int mode = filestat (buf);

        if (mode == -1) {                      /** File does not exist.   **/
          sprintf (STRINGS(RECFILE), "%s", str);
          XtVaSetValues (rec_file_s, XtNstring, STRINGS(RECFILE), NULL);
          CHSTATE(RFNAME, True);
          flag = True;
        } else
        if (mode == S_IFREG) {                 /** File exists.          **/
          sprintf (buf, "File already exists. \n    \"%s\" \nOverwrite?", str);
          if (popup_errbox (buf, "OK", "Cancel", NULL, TopRecord) == VOK) {
            sprintf (STRINGS(RECFILE), "%s", str);  /**   Overwrite OK   **/
            XtVaSetValues (rec_file_s, XtNstring, STRINGS(RECFILE), NULL);
            CHSTATE(RFNAME, True);
          }
          CheckTextandMark (rec_file_l, RECFILE, 0);
          return True;
        } else
        if (mode == S_IFDIR) {                 /** This is a directory   **/
          sprintf (mesg, "This is a directory.\n %s", buf);
        } else {                               /** I don't know??        **/
          sprintf (mesg, "Invalid file name.\n %s", buf);
        }
      }
    } else {
      XtVaSetValues (rec_dir_s, XtNstring, STRINGS(RECDIR), NULL);
    }
    CHSTATE(RFNAME, flag);
    CheckTextandMark (rec_file_l, RECFILE, 0);
    return flag;
}

/******  Check and Set File Name  ******/
void set_recfilename (w, event)
Widget w;
XEvent event;
{
    char buf[BUFSIZ];

    if (CheckRecordFilename(buf) == False) {
      popup_errbox (buf, "OK", NULL, NULL, TopRecord);
    } else {
      Move_Cursor (TopRecord, rec_info_s,  Widget_Width (rec_info_s) - 10, 5);
    }
    CheckTextandMark (w, RECFILE, 0);
}

/******  Set File Info  ******/
void set_fileinfo (w, event)
Widget w;
XEvent event;
{
    XtVaSetValues (rec_info_s, XtNstring, GetInfoString(), NULL);
    Move_Cursor (TopRecord, rec_limit_s, Widget_Width (rec_limit_s) - 10, 5);
    CheckTextandMark (w, INFOSTR, 0);
}

/***** Check All marking of Text string fileds *****/
void  CheckAllTextMarking ()
{
    CheckTextandMark (NULL, LIMITSTR, NULL); 
    CheckTextandMark (NULL, RECDIR,   NULL);
    CheckTextandMark (NULL, RECFILE,  NULL);
    CheckTextandMark (NULL, DIRNAME,  NULL);
    CheckTextandMark (NULL, PATTERN,  NULL);
}
