/*****************************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    special.c  (special functions)
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 *****************************************************************/

#include "xplay.h"

Widget  SpeBox, SpePanel;
Widget  spetitle, spefile_l, spefile_s, speinfo_l, speinfo_s;
Widget  scale_l, scale_bar, speconfig, spefsize;
Widget  speOkay, specancel;

extern  Widget SpePanel,  PopSpecial, TopSpecial;

Audio_obj  AO;
static  int        flag;

/*
 * Prototype
 */

extern void Popdown_Panel ();
extern void Thumb ();
extern void Scroll ();
extern void Ring ();
extern char * Encoding_to_String ();

void   Set_Special ();
void   rewrite_proc ();

/*****  Translations Save File Name text  *****/
static char spe_fnameTranslations [] =
       "Ctrl<Key>J:    Ring() \n\
        Ctrl<Key>O:    Ring() \n\
        Ctrl<Key>M:    Ring() \n\
        <Key>Return:   Ring() \n\
        <Enter>: blink-caret(SPEFILE,True) \n\
        <Leave>: blink-caret(SPEFILE,False) ";

/*****  Translations Save File Information text  *****/
#ifdef MULTI
static char spe_infoTranslations [] =
       "Ctrl<Key>J:    Ring() \n\
        Ctrl<Key>O:    Ring() \n\
        Ctrl<Key>M:    Ring() \n\
        <Key>Return:   Ring() \n\
        <Enter>:  blink-caret(SPEINFO,True) focus-in()\n\
        <Leave>:  blink-caret(SPEINFO,False) focus-out()";
#else
static char spe_infoTranslations [] =
       "Ctrl<Key>J:    Ring() \n\
        Ctrl<Key>O:    Ring() \n\
        Ctrl<Key>M:    Ring() \n\
        <Key>Return:   Ring() \n\
        <Enter>:  blink-caret(SPEINFO,True) \n\
        <Leave>:  blink-caret(SPEINFO,False) ";
#endif


/********  File Manager Panel Widgets  ********/
Widget Create_Special_Panel (parent)
Widget parent;
{
    Dimension h;

    SpeBox    = XtVaCMW ("SpeBox",   BoxW,       parent,         NULL);
    SpePanel  = XtVaCMW ("SpePanel", FormW,      SpeBox,   farg, NULL);

    spetitle  = XtVaCMW ("spetitle",  LabelW,    SpePanel, NULL);
    spefile_l = XtVaCMW ("spefile_l", LabelW,    SpePanel, NULL);
    spefile_s = XtVaCMW ("spefile_s", AsciiW,    SpePanel, NULL);
    speinfo_l = XtVaCMW ("speinfo_l", LabelW,    SpePanel, NULL);
#ifdef MULTI
    if (STATE(LANGF))
      speinfo_s = XtVaCMW ("speinfo_s", MultiW,    SpePanel, NULL);
    else
      speinfo_s = XtVaCMW ("speinfo_s", AsciiW,    SpePanel, NULL);
#else
    speinfo_s = XtVaCMW ("speinfo_s", AsciiW,    SpePanel, NULL);
#endif
    scale_l   = XtVaCMW ("scale_l",   LabelW,    SpePanel, NULL);
    scale_bar = XtVaCMW ("scale_bar", ScrollW,   SpePanel, barg,  NULL);
    speconfig = XtVaCMW ("speconfig", LabelW,    SpePanel, NULL);
    spefsize  = XtVaCMW ("spefsize",  LabelW,    SpePanel, NULL);

    speOkay   = XtVaCMW ("speOkay",   CommandW,  SpePanel, NULL);
    specancel = XtVaCMW ("specancel", CommandW,  SpePanel, NULL);

    XtAddCallback (specancel, XtNcallback,    Popdown_Panel, (caddr_t)PSPEC);
    XtAddCallback (speOkay,   XtNcallback,    rewrite_proc,           NULL);
    XtAddCallback (scale_bar, XtNjumpProc,    Thumb,         (caddr_t)SCALE);
    XtAddCallback (scale_bar, XtNscrollProc,  Scroll,        (caddr_t)SCALE);

    MyOverrideTranslations (spefile_s, spe_fnameTranslations);
/*
    MyAugmentTranslations (speinfo_s, spe_infoTranslations);
*/
    MyOverrideTranslations (speinfo_s, spe_infoTranslations);
    return SpeBox;
}

/**********/
void rewrite_proc ()
{
    String newfilename, newfileinfo;
    int volume_scale, fd, err, header_size, newfilesize;
    char newfilepath[BUFSIZ];
    Audio_obj special_obj;

    if (flag != SUCCESS) {
      popup_errbox (" This is not an audio file.", 
                    "OK", NULL, NULL, TopSpecial);
      return;
    }
    XtVaGetValues (spefile_s, XtNstring, &newfilename, NULL);
    XtVaGetValues (speinfo_s, XtNstring, &newfileinfo, NULL);
    sprintf (newfilepath, "%s/%s", STRINGS(DIRNAME), newfilename);
    volume_scale = STATE(SCALE);
    if ((strcmp (newfilepath, AO.filename) == 0) &&
        (strcmp (newfileinfo, AO.Infostr)  == 0) &&
        (volume_scale == AO.volume_scale)) {
      return;
    }

    if ((fd = open (AO.filename, O_RDWR, 644)) == -1) {
      char buff[100];

      sprintf (buff, "Can't open %s", AO.filename);
      popup_errbox (buff, "OK", NULL, NULL, TopSpecial);
      return;
    }

    err = lseek (fd, 0, SEEK_SET);
    AO.ofd = fd;
    AO.Dev_hdr = AO.File_hdr;
    strcpy (AO.Infostr, newfileinfo);
    AO.volume_scale = volume_scale*MAX_SCALE/MAX_GAIN;
    if (Write_File_Header (&AO) < 0) {
      popup_errbox (" Error in writing header.", "OK", NULL, NULL, TopSpecial);
      close (fd); 
      return ;
    }

    AO.rw_size = AO.File_hdr.data_size;
    if (Rewrite_File_Size (AO) < 0) {
      popup_errbox (" Error in rewriting header.", 
                    "OK", NULL, NULL, TopSpecial);
      close (fd); 
      return ;
    }
    close (fd);

    if (strcmp (AO.filename, newfilepath) != 0) {
      if (rename (AO.filename, newfilepath) == -1)
        popup_errbox ("Can't rename file name.", 
                      "OK", NULL, NULL, TopSpecial);
        return;
    }
    Popdown_Panel (NULL, PSPEC, 0);
}

/*****  Reset Value of Special Panel  *****/
void   Set_Special ()
{
    switch (flag = Get_Current_File (&AO)) {
      case SUCCESS:
           switch (AO.suffix){
              case SUN: XtVaSetValues (spetitle, XtNlabel,             /* */
                     "          This is a Sun/NeXT .au file.          ", NULL);
                        break;
              case ULRAW: XtVaSetValues (spetitle, XtNlabel,             /* */
                     "          This is a u-law byte raw file.        ", NULL);
                        break;
              case WAV: XtVaSetValues (spetitle, XtNlabel, 
                     "     This is a Windows 3.1 .WAV RIFF file.      ", NULL);
                        break;
              default : XtVaSetValues (spetitle, XtNlabel, 
                     "                Unknown  suffix                 ", NULL);
                        break;
	   }
           break;
      case DIRECTORY:
           XtVaSetValues (spetitle, XtNlabel, 
                  "              This is a directory.              ", NULL);
           break;
      case NOTVALID:
           XtVaSetValues (spetitle, XtNlabel, 
                  "        This is not a valid audio file.         ", NULL);
           break;
      default:
           XtVaSetValues (spetitle, XtNlabel, 
                  "              Unknown file type.                ", NULL);
           break;
    };

    if (flag  == SUCCESS) {
      char buff[80];
      char *cp;
      Cardinal enc = AO.File_hdr.encoding;

      if ((cp = (char *)strrchr (AO.filename, '/')) == NULL)
        cp = AO.filename;
      else
        cp++;
      XtVaSetValues (spefile_s, XtNstring, cp, NULL);
      XtVaSetValues (speinfo_s, XtNstring, AO.Infostr,  NULL);
      CHSTATE(SCALE, AO.volume_scale * MAX_GAIN / MAX_SCALE);
      SetScrollbars (SCALE, AO.volume_scale * MAX_GAIN / MAX_SCALE);
      sprintf (buff, "Configuration  : %dHz, %s, %s",
             AO.File_hdr.sample_rate, Encoding_to_String (enc),
            (AO.File_hdr.channels == MONO ? "Monaural" : "Stereo"));
      XtVaSetValues (speconfig, XtNlabel, buff, NULL);
      sprintf (buff, "File Size      : %d Bytes           ",
            AO.header_size + AO.File_hdr.data_size);
      XtVaSetValues (spefsize, XtNlabel, buff, NULL);
    } else {
      char *cp;
      int  len = strlen (AO.filename);
      if (AO.filename[len - 1] == '/')
        AO.filename[len - 1] = Null;
      if ((cp = (char *)strrchr (AO.filename, '/')) == NULL)
        cp = AO.filename;
      else
        cp++;

      XtVaSetValues (spefile_s, XtNstring, cp, NULL);
      XtVaSetValues (speinfo_s, XtNstring, "                         ", NULL);
      CHSTATE(SCALE, 50);
      SetScrollbars (SCALE, 50);
      XtVaSetValues (speconfig, XtNlabel, 
                    "Configuration  :                                ", NULL);
      XtVaSetValues (spefsize, XtNlabel,  
                    "File Size      :                                ", NULL);
    }
}
