/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    state.c    (State Manager)
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#include "xplay.h"

static int   play_volume = -1;               /*  Playing Gain            */
static int   rec_volume = -1;                /*  Recording Gain          */
static int   mon_volume = -1;                /*  Monitor gain            */
static int   play_balance = -1;              /*  Balance of Playing      */
static int   rec_balance = -1;               /*  Balance of Recording    */
static int   play_speed = MAX_GAIN / 2;      /*  Playing speed           */
static int   play_position = 0;              /*  File posistion for play */
static int   gain_scale = 100;               /*  Gain scale for play     */
static int   rfname_flag;                    /*  Record file name flag   */
static int   loopingflag = False;            /*  Looping flag            */
static int   pauseflag = NOTHING;            /*  Pause flag              */
static int   play_state = NOPLAY;            /*  State of Playing        */
static int   rec_state = NOREC;              /*  State of Recording      */
static int   prflag = NOTHING;               /*  Play or Record flag     */
static int   popup_panels = PNON;            /*  Popup Panel flag        */
static int   splitflag = False;              /*  Split Window Model      */
static int   output_muted = False;           /*  Output Mute Flag        */
static int   Processor = DBRI;               /*  Audio Processor         */
static int   rec_limit = 20000;              /*  Record File Limit (KB)  */
static int   dirname_flag = False;           /*  Play Directory flag     */
static int   rec_dirname_flag = False;       /*  Record Directory flag   */
static int   file_number = 0;                /*  File number in the dir  */
static int   list_number = 0;                /*  File number in the list */
static int   current_list = -1;              /*  Current list pointer    */
static int   caret_blink_flag = False;       /*  Caret blink flag        */
static int   rec_overflow_flag = False;      /*  Overflow flag at rec    */
static int   direction  = FORWARD;           /*  Direction for play      */
static int   adjust = True;                  /*  Volume adjust for play  */
static int   stereodelay = 50;               /*  Semi Hi-Fi delay units  */
static int   rectrigger = NOUSE;             /*  Trigger of recording    */
static int   rectriglevel = 10;              /*  Trigger level of rec    */
static int   rlimit_flag = False;            /*  Record Limit Flag       */
static int   pattern_flag = False;           /*  Pattern Flag            */
static int   recinfo_flag = False;           /*  Record information flag */
static int   pointer_type = PERCENT;         /*  Type of play pointer    */
static int   shortlist_flag = False;         /*  LIST display type       */
static int   langflag = False;               /*  LANG environment value f*/
static int   samplefix = False;              /*  Sample rate fix mode    */
static int   limit_time = 0;                 /*  Limit time (millsec)    */
static int   effect=ENCODING|SAMPLE|CHANNEL; /*  Play effects            */
static int   config = NOCONFIG;              /*  Effect Configuration    */
static int   device_endian = BIGENDIAN;      /*  Device Endian           */

static unsigned limitsize = UNKNOWN_SIZE;    /*  Recording limit size    */
static char dirname[BUFSIZ];                 /*  Directory name string   */
static char recfile[BUFSIZ];                 /*  Record file name string */
static char recdir [BUFSIZ];                 /*  Directory for Recording */
static char pattern[BUFSIZ];                 /*  Search pattern for File */
static char infostr[BUFSIZ];                 /*  Information string      */
static char limitstr[BUFSIZ];                /*  Record limit string     */

static XtAppContext app_con;                 /*  application context     */

int  STATE();
int  CHSTATE();
char * STRINGS();

/*****  Read Global states  *****/
int STATE(type)
int type;
{
    switch (type) {
    case PLAYV  : return play_volume;
    case RECV   : return rec_volume;
    case MONV   : return mon_volume;
    case PLAYS  : return play_speed;
    case PLAYB  : return play_balance;
    case RECB   : return rec_balance;
    case PLAYP  : return play_position;
    case SCALE  : return gain_scale;
    case RFNAME : return rfname_flag;
    case LOOPING: return loopingflag;
    case PAUSEF : return pauseflag;
    case PSTATE : return play_state;
    case RSTATE : return rec_state;
    case PRFLAG : return prflag;
    case POPUP  : return popup_panels;
    case SPLIT  : return splitflag;
    case MUTED  : return output_muted;
    case PROCESS: return Processor;
    case RLIMIT : return rec_limit;
    case DIRF   : return dirname_flag;
    case RECDIRF: return rec_dirname_flag;
    case FILENUM: return file_number;
    case LISTNUM: return list_number;
    case CURTLST: return current_list;
    case CARETB : return caret_blink_flag;
    case OVERFLW: return rec_overflow_flag;
    case DIRECTI: return direction;
    case ADJUST : return adjust;
    case LIMTSIZ: return limitsize;
    case STEREODEL:return stereodelay;
    case RECTRIGGER:return rectrigger;
    case TRIGL  : return rectriglevel;
    case RLIMITF: return rlimit_flag;
    case PATTERNF:return pattern_flag;
    case RECINFOF:return recinfo_flag;
    case PTYPE:   return pointer_type;
    case SHORTLIST:return shortlist_flag;
    case LANGF  : return langflag;
    case SAMPLEFIX:return samplefix;
    case LIMTTIM: return limit_time;
    case EFFECT : return effect;
    case CONFIG : return config;
    case DEVENDIAN:return device_endian;
    defalut     : return 0;
    }
}

/*****  Change Global states  *****/
int CHSTATE(type,value)
int type;
int value;
{
    switch (type) {
    case PLAYV  : return play_volume = value;
    case RECV   : return rec_volume = value;
    case MONV   : return mon_volume = value;
    case PLAYS  : return play_speed = value;
    case PLAYB  : return play_balance = value;
    case RECB   : return rec_balance = value;
    case PLAYP  : return play_position = value;
    case SCALE  : return gain_scale = value;
    case RFNAME : return rfname_flag = value;
    case LOOPING: return loopingflag = value;
    case PAUSEF : return pauseflag = value;
    case PSTATE : return play_state = value;
    case RSTATE : return rec_state = value;
    case PRFLAG : return prflag = value;
    case POPUP  : return popup_panels = value;
    case SPLIT  : return splitflag = value;
    case MUTED  : return output_muted = value;
    case PROCESS: return Processor = value;
    case RLIMIT : return rec_limit = value;
    case DIRF   : return dirname_flag = value;
    case RECDIRF: return rec_dirname_flag = value;
    case FILENUM: return file_number = value;
    case LISTNUM: return list_number = value;
    case CURTLST: return current_list = value;
    case CARETB : return caret_blink_flag =value;
    case OVERFLW: return rec_overflow_flag = value;
    case DIRECTI: return direction = value;
    case ADJUST : return adjust = value;
    case LIMTSIZ: return limitsize = value;
    case STEREODEL:return stereodelay = value;
    case RECTRIGGER:return rectrigger = value;
    case TRIGL  : return rectriglevel = value;
    case RLIMITF: return rlimit_flag =  value;
    case PATTERNF:return pattern_flag = value;
    case RECINFOF:return recinfo_flag = value;
    case PTYPE  : return pointer_type = value;
    case SHORTLIST:return shortlist_flag = value;
    case LANGF  : return langflag = value;
    case SAMPLEFIX:return samplefix = value;
    case LIMTTIM: return limit_time = value;
    case EFFECT : return effect = value;
    case CONFIG : return config = value;
    case DEVENDIAN:return device_endian = value;
    defalut     : return 0;
    }
}

/*****  Read Global Strings  *****/
char *STRINGS(type)
int type;
{
    switch (type) {
    case DIRNAME: return dirname;
    case RECFILE: return recfile;
    case RECDIR : return recdir;
    case PATTERN: return pattern;
    case INFOSTR: return infostr;
    case LIMITSTR:return limitstr;
    defalut     : return Null;
    }
}

/*****  Read Application Context  *****/
XtAppContext APPCONTE()
{
    return app_con;
}

/*****  Write Application Context  *****/
XtAppContext CHAPPCONTE(value)
XtAppContext value;
{
    return app_con = value;
}
