/**********************************************************
 *
 * XPlay:  Audio Control Panel            version 2.1
 *
 * File:   tools.c
 *
 * Date:   Sep, 1, 1993
 *
 * By:     Atsushi Hirahara
 *         School of Science and EnGineering, Waseda University
 *         hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#include     "xplay.h"
#include     "menu.h"
#include     "bitmaps/icon.xbm"

extern Widget toplevel, msg_str;
extern Widget dir_str, rec_dir_s, rec_limit_s;
extern Widget rec_file_s, rec_info_s, pattern_str;
extern Widget dir_l, rec_dir_l, rec_limit_l, rec_file_l;
extern Widget rec_info_l, pattern_l;
extern Widget spefile_s, speinfo_s;
extern Widget TopFile;

/*
 *  Prototypes
 */

extern  void    Down ();
extern  void    pause_proc ();
extern  void    rec_stop_proc ();
extern  void    record_proc ();
extern  void    rescan_proc ();
extern  void    all_proc ();
extern  void    Popdown_Panel ();
extern  void    Put_Message ();
extern  void    Popup_Panel ();
extern  XtAppContext APPCONTE ();
extern  char *  STRINGS ();
extern  char *  OptimizeDirname ();

Widget  Create_File_Panel ();
Widget  Create_Message_Panel ();
Widget  Create_Record_Panel ();
void    Create_Error_Panel ();
void    Put_Message ();
void    Quit ();
void    Abort ();
void    FileCompletion ();

/*****  Print Usage  *****/
void usage (progname)
char *progname;
{
    Error(stderr, "%s [-options] [directroy]\n\n", progname);
    Error(stderr, "where options include:\n"); 
    Error(stderr, "     -help                 ... Print this usage.\n");
    Error(stderr, "     -split    (!-normal)  ... Panel split type.\n");
    Error(stderr, "     -normal   (!-split)   ... Normal type (default).\n");
    Error(stderr, "     -maxsize filesize     ... Maximum file size (KB).\n");
    Error(stderr, "     -pattern \"pattern\"    ... Search pattern.");
    Error(stderr, "(wildcard) : default \"%s\".\n", DEFPATTERN);
    Error(stderr, "     -unadjust             ... Un-adjust play gain.\n");
    Error(stderr, "     -stereo delay         ... Semi-Stereo delay.\n");
    Error(stderr, "     -percent  (!-time)    ... Display file pointer.\n");
    Error(stderr, "     -time     (!-percent) ... Display playing time.\n");
    Error(stderr, "     -fullpath (!-basename)... Display full path name.\n");
    Error(stderr, "     -basename (!-fullpath)... Display base name.\n");
    Error(stderr, "     -directory directory \n");
    Error(stderr, "     directory             ... Directory of audio file.\n");
    Error(stderr, "                               If no directory ");
    Error(stderr, "specified, set current directory.\n\n");
    exit (1);
}

/********  Quit  ********/
void Quit (w, par, npar)
Widget  w;
int     par;
int     npar;
{
    if (STATE(RSTATE) == DOREC && (STATE(PRFLAG) & RECORD)) {
      CHSTATE(RSTATE, ENDREC);
      RecordMain ();
    }
    FlushOutPut ();
    FlushInPut ();
    Pop_Configuration ();
    exit (0);
}

/******  Abort (If ^Q then quit.)  ******/
void Abort ( w )
Widget   w;
{
    if (STATE(RSTATE) == DOREC && (STATE(PRFLAG) & RECORD)) {
      CHSTATE(RSTATE, ENDREC);
      RecordMain ();
    }
    FlushOutPut ();
    FlushInPut ();
#ifdef DEBUG
    fprintf (stderr, "XPlay : Aborted... \n");
#endif
    exit (0);
}

/******  Exit xplay and exec sun's play command  ******/
void Continue (w)
Widget w;
{
    int pid;

    if ((pid = fork ()) < 0) {
      perror ("fork");
      Abort (w);
    }

    if (pid == 0) {
      String cp, *namep, *namepbak;
      char buff[1024], *argument[MAX_DATA + 3], **args;

      FlushOutPut ();
      FlushInPut ();
      namep = namepbak = (String*)CLISTPTR();
      memset (buff, 0, sizeof (buff));
      args = argument;
      *args++ = PLAYCOMMAND;
      while (*namep != NULL) {
        strcat (buff, *namep);
        strcat (buff, " ");
        if (strlen (buff) > 1000) break;
        *args++ = *namep++;
      }
      *args = NULL;
      execvp (*argument, argument);
      perror (*args);
    }
    Abort (w);
}

/********   Create Sample Menu Pane    ********/
void Create_Menu_Pane (parent, mgrname, menuitem, func)
Widget     parent;
char       *mgrname;
void       (*func)();
MENU       *menuitem;
{
    Widget      mshell;
    char        label[255];
    int         i = 0;

    mshell =XtVaCreatePopupShell (mgrname, SMenuW, parent,NULL);

    while (menuitem[i].value > 0) {
       sprintf (label, " %s ", menuitem[i].label);
       menuitem[i].widget = XtVaCMW (label, smeBSBObjectClass, mshell, NULL);
       XtAddCallback (menuitem[i].widget, XtNcallback, func, (caddr_t)i);
       i++;
    }
}

/********  Create Bitmap  ********/
void Create_Bitmap (w, TYPE, bits, width, height)
Widget w;
String TYPE, bits;
Dimension width, height;
{
    Pixmap pixmap = None;

    XtVaGetValues (w, TYPE, &pixmap, NULL);
    if (pixmap == None) {
      XtVaSetValues (w, TYPE,
                 XCreateBitmapFromData (XtDisplay (toplevel),
                   XtScreen (toplevel) -> root,
                     (char *) bits, width, height), NULL);
    }
}

/*****  Create Popup Panels  *****/
Widget Create_Popup_Panel (name, str)
char *name, *str;
{
    Widget tempw;
    Arg    wargs[10];

    XtSetArg (wargs[0], XtNtitle, str);
    tempw = (Widget)XtCPS (name, TopLevelW, toplevel, wargs, 1);
    Icon (tempw, icon_bits, icon_width, icon_height);
    return tempw;
}

/***** Get Widget Width *****/
Dimension Widget_Width (w)
Widget w;
{
    Dimension wi;

    XtVaGetValues (w, XtNwidth, &wi, NULL);
    return wi;
}

/***** Get Widget Height ******/
Dimension Widget_Height (w)
Widget w;
{
    Dimension he;

    XtVaGetValues (w, XtNwidth, &he, NULL);
    return he;
}

/***** Move Mouse Cursor *****/
void Move_Cursor (top, offset, posx, posy)
Widget top, offset;
Dimension posx, posy;
{
#ifndef NOMOVECUR
    Dimension topx, topy, offx, offy;
    int setx, sety;

    XtVaGetValues (top,    XtNx, &topx, XtNy, &topy, NULL);
    XtVaGetValues (offset, XtNx, &offx, XtNy, &offy, NULL);
    setx = topx + offx + posx;
    sety = topy + offy + posy;

    XWarpPointer (XtDisplay (top), None, DefaultRootWindow (XtDisplay (top)),
                                    0, 0, 0, 0, setx, sety);
#endif
}

/*****  Caret Blink Timeout  *****/
void CaretBlink (w, id)
int w;
XtIntervalId  *id;
{
    static int state = True;
    int  bdir, brecdir, blimit, binfo, bfile;
    int  sfile, sinfo, browser;
    int caret_blink_flag = STATE(CARETB);

    if (caret_blink_flag == DIRS)   bdir   = state; else bdir   = True;
    if (caret_blink_flag == RECDIRS)brecdir= state; else brecdir= True;
    if (caret_blink_flag == RECLIMIT) blimit = state; else blimit = True;
    if (caret_blink_flag == RFILE)  bfile  = state; else bfile  = True;
    if (caret_blink_flag == RINFO)  binfo  = state; else binfo  = True;
    if (caret_blink_flag == SFILE)  sfile  = state; else sfile  = True;
    if (caret_blink_flag == SINFO)  sinfo  = state; else sinfo  = True;
    if (caret_blink_flag == BROWSER)browser= state; else browser= True;

    XtVaSetValues (dir_str,     XtNdisplayCaret, bdir,   NULL);
    XtVaSetValues (rec_dir_s,   XtNdisplayCaret, brecdir,NULL);
    XtVaSetValues (rec_limit_s, XtNdisplayCaret, blimit, NULL);
    XtVaSetValues (rec_file_s,  XtNdisplayCaret, bfile,  NULL);
    XtVaSetValues (rec_info_s,  XtNdisplayCaret, binfo,  NULL);
    XtVaSetValues (spefile_s,   XtNdisplayCaret, sfile,  NULL);
    XtVaSetValues (speinfo_s,   XtNdisplayCaret, sinfo,  NULL);
    XtVaSetValues (pattern_str, XtNdisplayCaret, browser,NULL);
    state = (state ? False : True);
    XtAppAddTimeOut (APPCONTE(), BLINKDELAY, CaretBlink, 0);
}

/*****  Blink or not Action Procedure  *****/
void blink_caret (w, event, params, nparams)
Widget w;
XEvent event;
String *params;
int    nparams;
{
    int flag;

    if (params[0] == NULL || params[1] == NULL) {
      fprintf (stderr, "Params mismatch (Caret_Name,Boolean)\n");
      return;
    };
    if (strcmp (params[1], "True") == 0)
      flag = True;
    else
      flag = False;
    if (strcmp (params[0], "DIR") == 0) {
      CHSTATE(CARETB, (flag ? DIRS : False));
    } else
    if (strcmp (params[0], "RECDIR") == 0) {
      CHSTATE(CARETB, (flag ? RECDIRS : False));
    } else
    if (strcmp (params[0], "RLIMIT") == 0) {
      CHSTATE(CARETB, (flag ? RECLIMIT : False));
    } else
    if (strcmp (params[0], "RFILE") == 0) {
      CHSTATE(CARETB, (flag ? RFILE : False));
    } else
    if (strcmp (params[0], "RINFO") == 0) {
      CHSTATE(CARETB, (flag ? RINFO : False));
    } else
    if (strcmp (params[0], "BROWSER") == 0) {
      CHSTATE(CARETB, (flag ? BROWSER : False));
    } else
    if (strcmp (params[0], "SPEFILE") == 0) {
      CHSTATE(CARETB, (flag ? SFILE : False));
    } else
    if (strcmp (params[0], "SPEINFO") == 0) {
      CHSTATE(CARETB, (flag ? SINFO : False));
    } else
    {
      fprintf (stderr, "Warning: Unknown caret name [%s]\n", params[0]);
    }
}

/***** Push Mouse Pointer *****/
void PushPopPointer (pp)
Boolean pp;
{
#define MAXPOINT 10
    Display     *d;
    Window      dummywin;
    static      unsigned    int xx[MAXPOINT], yy[MAXPOINT], pushc = 0;
    unsigned    int  dummy;

    d = XtDisplay (toplevel);
    if (pp == True) {
      if (pushc < MAXPOINT){
        XQueryPointer (d, DefaultRootWindow (d),  &dummywin, &dummywin, 
                           &dummy, &dummy, &xx[pushc], &yy[pushc], &dummy);
        pushc++;
      }
    } else {
      if (pushc > 0) {
        pushc--;
        XWarpPointer (d, None, DefaultRootWindow (d),
                                    0, 0, 0, 0, xx[pushc], yy[pushc]);
      }
    }
}

/********  Last Message Panel  ********/
void Put_Message (str1, str2)
char *str1, *str2;
{
    char buff[80];
    Dimension wi;

    XtVaGetValues (msg_str, XtNwidth, &wi, NULL);
    sprintf (buff, "%s%s", str1, str2);
    XtVaSetValues (msg_str, XtNlabel, buff, XtNwidth, wi, NULL);
}

/********   Wait cursor   *********/
void change_cur (flag, lists)
int  flag;
Widget lists;
{
    Cursor  cs;
    Display *d1, *d2;
    Window  win1, win2, rootwindow;
    Widget  wd1 = toplevel, wd2 = TopFile;

    if ( ! XtIsRealized (wd1) || ! XtIsRealized (wd2) || ! XtIsRealized (lists))
       return;
    d1 = XtDisplay (wd1);
    d2 = XtDisplay (wd2);
    win1 = XtWindow (wd1);
    win2 = XtWindow (wd2);
    rootwindow = XDefaultRootWindow (d1);
    if (flag == True) {
      XtVaSetValues (lists, XtNcursor, XCreateFontCursor (d1, XC_watch), NULL);
      cs = XCreateFontCursor (d1, XC_watch);
      XDefineCursor (d1, win1, cs);
      XDefineCursor (d2, win2, cs);
      XDefineCursor (d1, rootwindow, cs);
    } else {
      XtVaSetValues (lists, XtNcursor,XCreateFontCursor(d1,XC_left_ptr), NULL);
      XUndefineCursor (d1, win1);
      XUndefineCursor (d2, win2);
      XUndefineCursor (d1, rootwindow);
    }
    XFlush (d1);
}

/*****  Event from AsciiTextWidget   *****/
void CheckTextandMark (w, type, event)
Widget w;
int type;
XEvent *event;
{
    char buff[BUFSIZ];
    String str = NULL;
    Boolean flag;
    Widget strw, togglew;
    int flagtype;

    switch (type) {
      case LIMITSTR:
                     strw     = rec_limit_s;
                     togglew  = rec_limit_l;
                     flagtype = RLIMITF;
                     break;
      case RECDIR  :
                     strw     = rec_dir_s;
                     togglew  = rec_dir_l;
                     flagtype = RECDIRF;
                     break;
      case RECFILE :
                     strw     = rec_file_s;
                     togglew  = rec_file_l;
                     flagtype = RFNAME;
                     break;
      case DIRNAME :
                     strw     = dir_str;
                     togglew  = dir_l;
                     flagtype = DIRF;
                     break;
      case PATTERN :
                     strw     = pattern_str;
                     togglew  = pattern_l;
                     flagtype = PATTERNF;
                     break;
      case INFOSTR :
                     strw     = rec_info_s;
                     togglew  = rec_info_l;
                     flagtype = RECINFOF;
                     XtVaGetValues (strw, XtNstring, &str, NULL);
                     if (str == NULL || *str == Null) {
                       CHSTATE(RECINFOF, False);
		     } else {
                       strcpy (STRINGS(INFOSTR), str);
                       CHSTATE(RECINFOF, True);
		     }
                     break;
      default : return;
    };

    XtVaGetValues (strw, XtNstring, &str, NULL);
    if (strlen (str) == 0) {
      flag = False;
    } else 
    if (strcmp (STRINGS(type), str) != 0) {
      flag = CHSTATE(flagtype, False);
    } else
    if (STATE(flagtype))
      flag = True; else flag = False;
    XtVaSetValues (togglew, XtNstate, flag, NULL);
    if (flagtype == RECDIRF && flag == False) {
      CHSTATE(RFNAME, False);
      CheckTextandMark (w, RECFILE, event);
    }
}

/***** File Completion *****/
void  FileCompletion (w, event, params, nparams)
Widget w;
XEvent event;
String *params;
int    nparams;
{
    String str;
    char superdirname[BUFSIZ], notcomplete[BUFSIZ], *finddir[BUFSIZ];
    char filled[BUFSIZ], dummy[BUFSIZ], dirname[BUFSIZ], **args;
    Boolean flag = False, err = False;
    int len;

    memset (finddir, sizeof(char*) , BUFSIZ);    
    XtVaGetValues (w, XtNstring, &str, NULL);
    strcpy (dirname, str);

    if (dirname[0] == '~') {
      GetHomeDirectory (dummy, dirname); /* Convert ~ */
      if (strcmp (dummy, dirname) == 0)
        err = True;
      strcpy (dirname, dummy);
    }
    if (err == False) {
/*
 *     dirname = "/usr/local/soft/soun"
 *     superdirname -> "/usr/local/soft/"
 *     notcomplete  -> "soun"
 */
      GetSuperDir (superdirname, notcomplete, dirname);

      strcpy (filled, notcomplete);

      if (strlen (notcomplete) > 0) {
/** Search superdir into finddir list **/
        if (FindDirectory (finddir, superdirname) == True) {

/** Search "notcomplete" from finddir **/
          if ((flag = FillString (filled, finddir, notcomplete)) == False)
            XBell (XtDisplay (w), 0);
        } else
          XBell (XtDisplay (w), 0);
        if (finddir != NULL) {
          args = finddir;
          while (*args != NULL) {
             free (*args);
             args++;
          }
        }
      }
      sprintf (notcomplete, "%s/%s%c",
                 superdirname, filled, (flag ? '/': Null));
    } else {  /* Can't convert Home directory */
      strcpy (notcomplete, dirname);
      XBell (XtDisplay (w), 0);
    }
    str = (char *)OptimizeDirname (dummy, notcomplete);
    XtVaSetValues (w, XtNstring, dummy, NULL);
    XawTextSetInsertionPoint (w, strlen (dummy));
}


