/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    wildcard.c
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and EnGineering, Waseda University
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#include <stdio.h>
#ifndef True
#define True 1
#endif
#ifndef False
#define False 0
#endif
#define IF_RETURN(exp)     if(exp) return True; else return False
#define EOP(p)             ((p) == NULL ? True : False)

enum {EOB = NULL, Null = '\0', NORMAL = ' ', ASTERISK = '*', QUESTION = '?'};

typedef char * String;
typedef char   Boolean;

typedef struct _List {
        char   chr;
        Boolean (*func)();
        struct _List *next;
} List, *ListPtr;

static Boolean Asterisk ();
static Boolean Question ();
static Boolean Other ();
static void FreeNode ();
static void MakeNode ();
Boolean wildcard ();

/****************************/
static Boolean Asterisk (p, s)
ListPtr p;
String s;
{
    if (EOP (p -> next))
      return True;
    else
      if (EOP (*s))
        return False;
      else
        if (p -> next -> func (p -> next, s)) 
          return True;
        else
          return Asterisk (p, (s + 1));
}

/*******************/
static Boolean Question (p, s)
ListPtr p;
String s;
{
    if (EOP (*s)) return False;
    if (EOP (p -> next))
      IF_RETURN (EOP (*(s + 1)));
    else
      return (p -> next -> func (p -> next, (s + 1)));
}

/*******************/
static Boolean Other (p, s)
ListPtr p;
String s;
{
    if (EOP (*s)) return False;
    if (p -> chr == *s)
       if (EOP (p -> next))
          IF_RETURN (EOP (*(s + 1)));
       else
          return (p -> next -> func (p -> next, (s + 1)));
    else
       return False;
}

/******************************/
static void FreeNode (ptr)
ListPtr ptr;
{
    if (EOP (ptr)) return;
    FreeNode (ptr -> next);
    free (ptr);
}

/******************************/
static void MakeNode (node, rptr)
ListPtr node;
String  rptr;
{
    if ( ! EOP (*rptr)) {
      ListPtr new = (ListPtr) malloc (sizeof (List));
        node -> next = new;
        new -> chr = *(rptr++);
        if (new -> chr == ASTERISK) {
           while (*rptr == ASTERISK) rptr++;
           new -> func = Asterisk;
        } else
           if (new -> chr == QUESTION)
              new -> func = Question;
           else
              new -> func = Other;
        new -> next = NULL;
        MakeNode (new, rptr);
    }
}

/*******************/
Boolean wildcard (string, regular)
char * string;
char * regular;
{
    List header;
    Boolean flag = False;

    if (EOP (string) || EOP (regular)) return False;
    if (*regular == Null) return False;
    MakeNode (&header, regular);
    if (header.next -> func != NULL)
      flag = header.next -> func (header.next, string);
    FreeNode (header.next);
    return flag;
}

