/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    xplay.h    (xplay header)
 *
 * Date:    Aug, 3, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and Engineering, Waseda University,
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#include   "includes.h"
#define    CLASSNAME       "XPlay"
#define    XPLAYID         "xplay "
#define    DEMO            "XPlay ver.2.1 Copyright A.Hirahara"
#define    THICKNESS       14
#define    STEPBAR         9
#define    SHOWN           16
#define    LENGTH          (SHOWN*STEPBAR+SHOWN)
#define    SKIP            ((LENGTH-SHOWN)/STEPBAR)
#define    BEGIN           (0)
#define    END             (LENGTH-SHOWN+1)
#define    HOSEI           ((float)(LENGTH-SHOWN/2)/END)
#define    BARDISTANCE     12
#define    TIMEOUT         30
#define    BLINKDELAY      300
#define    DOUBLECLICKINT  300
#define    MARKCHAR        '*'
#define    AUDIO_IODEV     "/dev/audio"
#define    AUDIO_CTLDEV    "/dev/audioctl"
#define    LANG_ENV        "LANG"
#define    PLAYCOMMAND     "play"
#define    MAX_GAIN        100
#define    MAX_SCALE       200
#define    MAXFILENAME     125
#define    MAX_DATA        500
#define    MAX_STEREODELAY 500
#define    FILEBUFSIZ      (1024*64)
#define    AUDIOBUFSIZ     345600
#define    TRIGGERVALUE    500
#define    DEFPATTERN      "*"
#define    NONSET          "----"
#define    SAMPLE_WIDTH    0.05
#define    EXTRAHEADERSIZE 16
#define    SKIPBACKTIME    5                /* Seconds of Skip Backward   */
#define    MAXRECORDSIZE   (1024*1000*20)   /* Max Output File Size (20M) */

#define    OUTPUT_MUTE      (0x08)
#ifndef    AUDIO_SPEAKER
#define    AUDIO_SPEAKER    (0x01)
#endif
#ifndef    AUDIO_HEADPHONE
#define    AUDIO_HEADPHONE  (0x02)
#endif
#ifndef    AUDIO_LINE_OUT
#define    AUDIO_LINE_OUT   (0x04)
#endif
#ifndef    AUDIO_MICROPHONE
#define    AUDIO_MICROPHONE (0x01)
#endif
#ifndef    AUDIO_LINE_IN
#define    AUDIO_LINE_IN    (0x02)
#endif
#ifndef    AUDIO_MIN_GAIN
#define	   AUDIO_MIN_GAIN   (0)	        /* minimum gain value */
#endif
#ifndef	   AUDIO_MAX_GAIN
#define	   AUDIO_MAX_GAIN   (255)	/* maximum gain value */
#endif
#ifndef    AUDIO_LFET_BALANCE
#define	   AUDIO_LEFT_BALANCE	(0)	/* left channel only */
#endif
#ifndef    AUDIO_MID_BALANCE
#define    AUDIO_MID_BALANCE	(32)	/* equal left/right balance */
#endif
#ifndef    AUDIO_RIGHT_BALANCE
#define	   AUDIO_RIGHT_BALANCE	(64)	/* right channel only */
#endif

#ifndef	AUDIO_ENCODING_NONE
#define	AUDIO_ENCODING_NONE	(0)
#endif
#ifndef	AUDIO_ENCODING_ULAW
#define	AUDIO_ENCODING_ULAW	(1)
#endif
#ifndef	AUDIO_ENCODING_ALAW
#define	AUDIO_ENCODING_ALAW	(2)
#endif
#ifndef	AUDIO_ENCODING_LINEAR
#define	AUDIO_ENCODING_LINEAR	(3)
#endif

#define SPEAKER          AUDIO_SPEAKER
#define HEADPHONE        AUDIO_HEADPHONE
#define LINE_OUT         AUDIO_LINE_OUT
#define MUTING           OUTPUT_MUTE
#define LINE_IN          AUDIO_LINE_IN
#define MICROPHONE       AUDIO_MICROPHONE
#define LEFT_BALANCE     AUDIO_LFET_BALANCE
#define MID_BALANCE      AUDIO_MID_BALANCE
#define RIGHT_BALANCE    AUDIO_RIGHT_BALANCE

#define	ENCODING_NONE    AUDIO_ENCODING_NONE
#define	ENCODING_ULAW    AUDIO_ENCODING_ULAW 
#define	ENCODING_ALAW    AUDIO_ENCODING_ALAW
#define	ENCODING_LINEAR  AUDIO_ENCODING_LINEAR
#define	ENCODING_FLOAT   AUDIO_ENCODING_FLOAT

#define	UNKNOWN_SIZE	 ((unsigned)(~0))
#define MAX_INFO         80

#define    Error           (void) fprintf
#define    Non             "\0"
#define    Null            '\0'
#define    Free(s)         {if((s)!=NULL)free(s);s=NULL;}
#define    Close(d)        {if((d)>0)close(d);d=-1;}
#define    InitAudioObj(p) memset((p),0,sizeof(Audio_obj))
#define    TEXTMASK        (ButtonPressMask | KeyReleaseMask)
#define    max(a,b)        (a > b ? a : b)
#define    min(a,b)        (a > b ? b : a)

#define barg      XtNorientation, (XtArgVal) XtorientHorizontal, \
                  XtNlength, (XtArgVal) LENGTH, \
                  XtNthickness, (XtArgVal) THICKNESS, \
                  XtNborderWidth, 0

#define larg      XtNborderWidth, 0, \
                  XtNjustify, XtJustifyRight, \
                  XtNresizable, True

#define recarg    XtNjustify, XtJustifyLeft, XtNborderWidth, 0

#define cntarg    XtNinternalWidth, 0, XtNinternalHeight, 0, \
                  XtNborderWidth, 0, XtNhighlightThickness, 0

#define farg      XtNborderWidth, 0


enum CHANNELS{NOCHANNEL, MONO, STEREO};
enum FLIST   {SETNON, SETFILE, SETDIR, SETLIST};
enum STATE   {PLAYV, RECV, MONV, PLAYS, PLAYB, RECB, PLAYP, SCALE, RFNAME, 
              LOOPING, PAUSEF, PSTATE, RSTATE, PRFLAG, POPUP, SPLIT,
              MUTED, PROCESS, RLIMIT, DIRF, RECDIRF, FILENUM, LISTNUM,
              CURTLST, CARETB, OVERFLW, DIRECTI, ADJUST, LIMTSIZ, 
              STEREODEL, RECTRIGGER, TRIGL, RLIMITF, PATTERNF, RECINFOF, 
              PTYPE, SHORTLIST, LANGF, SAMPLEFIX, LIMTTIM, EFFECT, CONFIG,
              DEVENDIAN};
enum STRTAG  {DIRNAME, RECFILE, RECDIR, PATTERN, INFOSTR, LIMITSTR};
enum PLAYTAG {NOPLAY, DOPLAY, ENDPLAY, PLAYERROR};
enum PTYPES  {PERCENT, TIME};
enum RECTAG  {NOREC,  DOREC,  ENDREC,  RECERROR, RECOVERMAX};
enum TRIGGER {NOUSE, WAITING, START};
enum MARKTAG {NOMARK, MARK};
enum VALUEB  {VNOSELECT, VOK, VNO, VCANCEL};
enum RPFLAG  {NOTHING, RECORD, PLAY, FORWARD, BACKWARD};
enum METHOD  {UNKNOWNP, U_LAW, A_LAW, LINEAR};
enum PROCESS {UNKNOWN, AMD, DBRI};
enum POPUP   {PNON=0,PGAIN=1,PCONT=2,PFILE=4,
              PREC=8,PINFO=16,PSPEC=32,PABOUT=64,PSFF=128,PCONF=256};
enum BLINK   {BONO=0,DIRS=1,RECLIMIT=2,RFILE=4,RINFO=8,BROWSER=16,RECDIRS=32,
              SFILE=64,SINFO=128};
enum ETC     {ERROR = -1, TOP = 0, THISONE, ALL};
enum OPENS   {SUCCESS, CANT, NOTVALID, DIRECTORY};
enum SUFFIX  {UNKNOWNSUFFIX, ULRAW, SUN, WAV};
enum CONFIGS {NOCONFIG = 0, ULAW2STEREO = 1, LINEAR_M2ULAW = 2,
              LINEAR_M2STEREO = 4, LINEAR_S2ULAW = 8, 
              LINEAR_S2MONO = 16, CHANGESAMPLE = 32};
enum EFFECTS {NOEFFECT = 0, ENCODING = 1, SAMPLE = 2, CHANNEL = 4};
enum ENDIAN  {BIGENDIAN, SMALLENDIAN};

/*
 *  Define new type
 */

typedef char LISTS[MAXFILENAME];

typedef struct _FNODE {                 /**  File struct  **/
        char    label [MAXFILENAME];
        Boolean fst;
        int     index;
} FNODE;

typedef struct _LNODE {                 /**  List struct  **/
        char    path [MAXFILENAME];
        Boolean flag;
        int     index;
} LNODE, *LNODEPTR;

typedef struct _MENU {                  /**  Sample Rate Menu Struct  **/
        int      value;
        unsigned sample;
        unsigned channels;
        unsigned precision;
        unsigned encoding;
        String   label;
        Widget   widget;
} MENU;

typedef struct _File_obj {
    char   magic[16];           /* The magic number */
    int    magicid;             /* The magic number id */
    int    version;             /* Version          */
    int    extend;              /* Extend value     */
    int    format;              /* ADPCM format     */
    int    emphasis;            /* Emphasis         */
    int    endian;              /* Data endian (Big/Small) */

    int    header_size;         /* Header offset. not consist of info */
    int    data_size;           /* Real data size (Byte) */
    int    unit_size;           /* bytes_per_unit (unit = one channel data) */
    int    samples_per_unit;    /* Samples per unit */
    int    encoding;            /* Encoding         */
#define UNKNOWNENC 0 
#define MULAW8     1
#define PCM8       2
#define PCM16      3
#define PCM24      4
#define PCM32      5
#define FLOAT32    6
#define DOUBLE64   7
#define ALAW8      8
#define ADPCM4     9
#define ADPCM8     10
#define ADPCM16    11
    int    sample_rate;         /* Sample rate      */
    int    channels;            /* Monaural = 1, Stereo = 2 */
} File_obj;

/*
 *   Audio device configuration for play/record.
 */
typedef struct _Audio_Device {
     Cardinal sample_rate;             	/* sample rate               */
     Cardinal channel;                 	/* number of channel         */
     Cardinal bits;                    	/* number of bits per sample */
     Cardinal encoding;                	/* encoding method           */
/*#define AUDIO_ENCODING_NONE	(0)	/* no encoding assigned */
/*#define AUDIO_ENCODING_ULAW	(1)	/* u-law encoding       */
/*#define AUDIO_ENCODING_ALAW	(2)	/* A-law encoding       */
/*#define AUDIO_ENCODING_LINEAR	(3)	/* Linear PCM encoding  */

     Cardinal port;                     /* I/O port                  */
     Cardinal port_num;                 /* availabel I/O ports       */
     Cardinal pause;                    /* pause flag                */

     Cardinal samples;                  /* number of samples converted */
     Cardinal error;                    /* overflow/underflow flag   */
     Cardinal waiting;                  /* a process wants access    */
     Cardinal extend;                   /* extend use                */
} Audio_Device;

typedef struct _gain_obj {
    Cardinal play_gain;                 /* gain (volume) for play    */
    Cardinal play_balance;              /* stereo balance for play   */
    Cardinal record_gain;               /* gain (volume) for record  */
    Cardinal record_balance;            /* stereo balance for record */
    Cardinal monitor_gain;              /* gain (volume) for monitor */
    Boolean   mute;                     /* Output mute flag          */
    int save_gain;                      /* gain saved                */
    int adjust_gain;                    /* adjust gain               */
    int scale;                          /* gain scale for play       */
} Audio_Gain, Gain_obj;

typedef struct _AllDevice {
    Audio_Device Play_Dev;
    Audio_Device Record_Dev;
    Audio_Gain   Gain_Dev;
} AllDevice;

typedef struct _audio_obj {
    File_obj      File_hdr;                   /*  audio header for file   */
    File_obj      Dev_hdr;                    /*  audio header for device */
    char          filename[255];
    char          Infostr[MAX_INFO + 1];      /* File Infomation str */
    int           ifd;
    int           ofd;
    int           ready;
    int           direction;
    int           volume_scale;
    int           play_speed;
    int           header_size;
    int           rw_size;
    int           bufsize;
    int           bytes_per_units;
    int           samples;
    int           suffix;
    unsigned long file_position;
} Audio_obj;

