/**********************************************************
 *
 * XPlay:   Audio Control Panel            version 2.1
 *
 * File:    xplaym.c (Main)
 *
 * Date:    Sep, 1, 1993
 *
 * By:      Atsushi Hirahara
 *          School of Science and EnGineering, Waseda University
 *          hirahara@fuka.info.waseda.ac.jp
 *
 **********************************************************/

#include     "xplay.h"                       /*  include header files   */
#include     "resource.h"                    /*  fallback resources     */
#include     "bitmaps/dismiss.xbm"           /*  Dismiss bitmap         */
#include     "bitmaps/icon.xbm"              /*  Icon bitmap            */
#include     "action.h"                      /*  action table           */

Widget toplevel,  Paned;                     /*  toplevel widget        */
Widget GainPanel, PopGain,    TopGain;       /*  for Gain Panel         */
Widget Control,   PopControl, TopControl;    /*  for Control Panel      */
Widget FilePanel, PopFile,    TopFile;       /*  for File Panel         */
Widget RecPanel,  PopRecord,  TopRecord;     /*  for Record Panel       */
Widget InfoPanel, PopInfo,    TopInfo;       /*  for Information Panel  */
Widget TopAbout,  PopAbout;                  /*  for About Panel        */
Widget ConfigPanel, TopConfig, PopConfig;    /*  for Configuration Panel*/
Widget MessageP,  MessageBox, msg_str;       /*  for Message Panel      */
Widget SpePanel,  PopSpecial, TopSpecial;    /*  for Special Panel      */
Widget TopError, ErrOK, ErrNO, ErrCancel;    /*  for Error Panel        */
Widget ErrorBox, error_l, formerr;           /*  for Error Panel        */

Widget PopQuit,   SplitF;                    /*  for Split Model        */
extern Widget load, record;                  /*  for Pop Up/Down        */

Audio_Device Play_Dev;                       /* Device config for play  */
Audio_Device Record_Dev;                     /* Decice config for record*/
Audio_Gain   Gain_Dev;                       /* Gain config             */

/*
 *  Prototypes
 */

extern  Widget  Create_Gain_Panel ();
extern  Widget  Create_Control_Panel ();
extern  Widget  Create_File_Panel ();
extern  Widget  Create_Message_Panel ();
extern  Widget  Create_Record_Panel ();
extern  Widget  Create_Popup_Panel ();
extern  void    FileManager ();
extern  void    CheckAudioDevice ();
extern  void    GetAudioConfiguration ();
extern  void    FlushOutPut ();
extern  void    FlushInPut ();
extern  void    PlayMain ();
extern  void    RecordMain ();
extern  void    Put_Message ();
extern  void    usage ();
extern  void    Quit ();
extern  void    Abort ();
extern  void    Create_Error_Panel ();
extern  void    CaretBlink ();
extern  void    set_output_label ();
extern  void    set_input_label ();
extern  void    set_sample_rate_label ();
extern  void    set_channel_label ();
extern  void    set_pause_label ();
extern  char *  STRINGS();
extern  int     STATE();
extern  int     CHSTATE();
extern  void    Set_Special ();
extern  void    Popup_About ();
extern  XtAppContext APPCONTE ();

void    Ring ();
void    NewValue ();
void    Popup_Panel ();
void    Popdown_Panel ();
void    Down ();
void    Create_Split_Panel ();
Widget  Create_Info_Panel ();

/*****  Translations for List Widge *****/
static char infoTranslations [] =
       "Ctrl<Key>C : Down(Info) \n\
        Ctrl<Key>D : Down(Info) \n\
        Ctrl<Key>Q : Down(Info)";

/******  Ring  *****/
void Ring(w, event, params, nparams)
Widget w;
XEvent *event;
String *params;
int    nparams;
{
    XBell (XtDisplay (toplevel), 0);
}

/******  Popup Panel by translation ******/
void Popup ( w , event, params, nparams)
Widget w;
XEvent *event;
String *params;
int    nparams;
{
    if (strcmp (params[0], "File") == 0)
      Popup_Panel (w, PFILE, 1);
    else
    if (strcmp (params[0], "Record") == 0)
      Popup_Panel (w, PREC, 1);
    else
    if (strcmp (params[0], "Help") == 0)
      Popup_Panel (w, PINFO, 1);
    else
    if (strcmp (params[0], "Gain") == 0)
      Popup_Panel (w, PGAIN, 1);
    else
    if (strcmp (params[0], "Control") == 0)
      Popup_Panel (w, PCONT, 1);
    else
    if (strcmp (params[0], "Special") == 0)
      Popup_Panel (w, PSPEC, 1);
    else
    if (strcmp (params[0], "About") == 0)
      Popup_About (w, PABOUT, 1);
    else
    if (strcmp (params[0], "SFF") == 0)
      Popup_About (w, PSFF, 1);
    else
    if (strcmp (params[0], "Config") == 0)
      Popup_Panel (w, PCONF, 1);
}

/******  PopDown  (if ^C then popdown) ******/
void Down ( w , event, params, nparams)
Widget w;
XEvent *event;
String *params;
int    nparams;
{
    if (strcmp (params[0], "Gain") == 0)
      Popdown_Panel (NULL, PGAIN, 0);
    else
    if (strcmp (params[0], "Control") == 0)
      Popdown_Panel (NULL, PCONT, 0);
    else
    if (strcmp (params[0], "File") == 0)
      Popdown_Panel (NULL, PFILE, 0);
    else
    if (strcmp (params[0], "Record") == 0)
      Popdown_Panel (NULL, PREC, 0);
    else
    if (strcmp (params[0], "Info") == 0)
      Popdown_Panel (NULL, PINFO, 0);
    else
    if (strcmp (params[0], "Special") == 0)
      Popdown_Panel (NULL, PSPEC, 0);
    else
    if (strcmp (params[0], "About") == 0)
      Popdown_About (NULL, PABOUT, 0);
    else
    if (strcmp (params[0], "SFF") == 0)
      Popdown_About (NULL, PSFF, 0);
    else
    if (strcmp (params[0], "Config") == 0)
      Popdown_Panel (NULL, PCONF, 0);
}

/********  Create Normal Panel  *******/
void Create_Normal_Panel (parent)
Widget parent;
{
    TopGain    = (Widget)Create_Gain_Panel (parent);
    TopControl = (Widget)Create_Control_Panel (parent);
    Create_Message_Panel (parent);

    TopFile    = Create_Popup_Panel ("TopFile", "File Panel (XPlay)");
    TopRecord  = Create_Popup_Panel ("TopRecord", "Record Panel (XPlay)"); 
    TopInfo    = Create_Popup_Panel ("TopInfo", "Help Panel (XPlay)");
    TopSpecial = Create_Popup_Panel ("TopSpecial", "Special Panel (XPlay)");
    TopAbout   = Create_Popup_Panel ("TopAbout", "About... (XPlay)");
    TopConfig  = Create_Popup_Panel ("TopConfig", "Configuration (XPlay)");

    Create_File_Panel (TopFile);
    Create_Record_Panel (TopRecord);
    Create_Info_Panel (TopInfo);
    Create_Special_Panel (TopSpecial);
    Create_About_Panel (TopAbout);
    Create_Config_Panel (TopConfig);
}

/********  Create Popup Panel Panel  *******/
void Create_Split_Panel (parent)
Widget parent;
{
    SplitF      = XtVaCMW ("SplitF",    FormW,     parent, NULL);
    PopGain     = XtVaCMW ("PopGain",   ToggleW,   SplitF, NULL);
    PopControl  = XtVaCMW ("PopControl",ToggleW,   SplitF, NULL);
    PopFile     = XtVaCMW ("PopFile",   ToggleW,   SplitF, NULL);
    PopRecord   = XtVaCMW ("PopRecord", ToggleW,   SplitF, NULL);
    PopQuit     = XtVaCMW ("PopQuit",   CommandW,  SplitF, NULL);

    TopGain    = Create_Popup_Panel ("TopGain",  "Play Gain Panel (XPlay)");
    TopControl = Create_Popup_Panel ("TopControl", "Control Panel (XPlay)"); 
    TopFile    = Create_Popup_Panel ("TopFile", "File Panel (XPlay)");
    TopRecord  = Create_Popup_Panel ("TopRecord", "Record Panel (XPlay)"); 
    TopInfo    = Create_Popup_Panel ("TopInfo", "Help Panel (XPlay)");
    TopSpecial = Create_Popup_Panel ("TopSpecial", "Special Panel (XPlay)");
    TopAbout   = Create_Popup_Panel ("TopAbout", "About... (XPlay)");
    TopConfig  = Create_Popup_Panel ("TopConfig", "Configuration (XPlay)");

    XtAddCallback (PopGain,    XtNcallback,   Popup_Panel,   (XtPointer)PGAIN);
    XtAddCallback (PopControl, XtNcallback,   Popup_Panel,   (XtPointer)PCONT);
    XtAddCallback (PopFile,    XtNcallback,   Popup_Panel,   (XtPointer)PFILE);
    XtAddCallback (PopRecord,  XtNcallback,   Popup_Panel,   (XtPointer)PREC);
    XtAddCallback (PopQuit,    XtNcallback,   Quit,          (XtPointer)NULL);

    Create_Gain_Panel (TopGain);
    Create_Control_Panel (TopControl);
    Create_File_Panel (TopFile);
    Create_Record_Panel (TopRecord);
    Create_Info_Panel (TopInfo);
    Create_Special_Panel (TopSpecial);
    Create_About_Panel (TopAbout);
    Create_Config_Panel (TopConfig);
}

/*****  Popdown Info Event Handler  *****/
void Popdown_Info (w, data, event)
Widget w;
int data;
XEvent *event;
{
    Popdown_Panel (w, PINFO, 1);
}

/*******  Information Panel Widgets  ********/
Widget Create_Info_Panel (parent)
Widget parent;
{
    Widget information;
    String help_path, jhelp = "jhelp.dat", ehelp = "help.dat";
    int fd;
    char buff[10000];

    InfoPanel = XtVaCMW ("InfoPanel", FormW, parent, NULL);

#ifdef JHELP
    jhelp = JHELP;
#endif
#ifdef EHELP
    ehelp = EHELP;
#endif
    help_path = ehelp;

#ifdef MULTI
    if (STATE(LANGF)) {
      if (strncmp (getenv (LANG_ENV), "ja_JP", 5) == 0)
        help_path = jhelp;
      information = XtVaCMW ("Information", MultiW, InfoPanel, NULL);
    } else 
      information = XtVaCMW ("Information", AsciiW, InfoPanel, NULL);
#else
    information = XtVaCMW ("Information", AsciiW, InfoPanel, NULL);
#endif 

    if ((fd = open (help_path, O_RDONLY)) != -1) {
      read (fd, buff, sizeof (buff));
      close (fd);
    } else {
      perror ("Cannot open help file.");
      sprintf (buff, "Can't find help file (%s)\n", help_path);
    }

/*
    XtVaSetValues (information, XtNstring, help_str, NULL);
*/
    XtVaSetValues (information, XtNstring, buff, NULL);
    MyOverrideTranslations (information, infoTranslations);

    XtAddEventHandler(information, ButtonPressMask, False, 
                                   (XtEventHandler)Popdown_Info, NULL);
    return InfoPanel;
}

/********  Message Panel Widgets  ********/
Widget Create_Message_Panel (parent)
Widget parent;
{
    Widget quit, about;

    MessageBox = XtVaCMW ("MessageBox", BoxW,  parent,           NULL);
    MessageP   = XtVaCMW ("MessageP",   FormW, MessageBox,       NULL);

    if (STATE(SPLIT) == False) {
      quit     = XtVaCMW ("quit",      CommandW,  MessageP,       NULL);
      XtAddCallback (quit,        XtNcallback, Quit,   (XtPointer)NULL);
      about    = XtVaCMW ("about",     ToggleW,   MessageP,
                                       XtNfromHoriz, quit,        NULL);
      msg_str  = XtVaCMW ("msg_str",   LabelW,    MessageP,
                            XtNfromHoriz, about, XtNwidth, 243, NULL);
    } else {
      Widget cdismiss;
      about    = XtVaCMW ("about",     ToggleW,   MessageP,       NULL);
      msg_str  = XtVaCMW ("msg_str",   LabelW,    MessageP,
                            XtNfromHoriz, about,  XtNwidth, 265,  NULL);
      cdismiss = XtVaCMW ("cdismiss", CommandW, MessageP, cntarg, NULL);
      XtAddCallback (cdismiss, XtNcallback, Popdown_Panel, (XtPointer)PCONT);
      if (STATE(SPLIT)) {
        Bitmap (cdismiss, dismiss_bits, dismiss_width, dismiss_height);
      }
    }
    PopAbout = about;
    XtAddCallback (about, XtNcallback, Popup_About, (XtPointer)PABOUT);
    return MessageBox;
}

static void nop(){/**/};
/********  Error Message Pannel Widgets  ********/
void Create_Error_Panel (parent)
Widget parent;
{
/*
    TopError = XtVaCMW ("TopError",   TransshellW,  parent,    NULL);
*/
    TopError = XtVaCMW ("TopError",   OvershellW ,  parent,    NULL);
    ErrorBox = XtVaCMW ("ErrorBox",   BoxW,         TopError,  NULL);
    formerr  = XtVaCMW ("formerr",    FormW,        ErrorBox,  NULL);
    error_l  = XtVaCMW ("error_l",    LabelW,       formerr,   NULL);
    ErrOK    = XtVaCMW ("ErrOK",      CommandW,     formerr,   NULL);
    ErrNO    = XtVaCMW ("ErrNO",      CommandW,     formerr,   NULL);
    ErrCancel= XtVaCMW ("ErrCancel",  CommandW,     formerr,   NULL);
    XtAddEventHandler (formerr, ButtonPressMask, False, nop, NULL);
}

/********  Error Message Box  ********/
int popup_errbox (str, button1, button2, button3, wid)
char *str, *button1, *button2, *button3;
Widget wid;
{
#define MOX 50
#define MOY 80
    XEvent      e;
    Display     *d;
    Window      dummywin, root;
    Dimension   x, y, dx, dy, w, h, borderw;
    unsigned    int xx, yy, ww, hh, dummy;
    unsigned    int scrw, scrh, bw, dp;

    d = XtDisplay (wid);
    XQueryPointer (d, DefaultRootWindow (d), 
                  &dummywin, &dummywin, &dummy, &dummy, &xx, &yy, &dummy);
    XGetGeometry (d, DefaultRootWindow (d),
                             &root, &dummy, &dummy, &scrw, &scrh, &bw, &dp);
    XBell (XtDisplay (toplevel), 0);
    XtVaGetValues (formerr, XtNwidth, &w, XtNheight, &h, 
                            XtNborderWidth, &borderw, NULL);

    xx = (xx < MOX ? xx : xx - MOX);
    yy = (yy < MOY ? yy : yy - MOY);
    w += borderw * 2;
    h += borderw * 2;
    if (xx + w > scrw)
      xx = scrw - w;
    if (yy + h > scrh)
      yy = scrh - h;
    XtVaSetValues (TopError, XtNx, xx, XtNy, yy, 
                             XtNwidth, w, XtNheight, h, NULL);
    XtPopup (TopError, XtGrabExclusive);
    XtVaSetValues (error_l, XtNlabel, str, NULL);

    if (button1 == NULL) {
      XUnmapWindow (XtDisplay (ErrOK), XtWindow (ErrOK));
   } else {
      XMapWindow (XtDisplay (ErrOK), XtWindow (ErrOK));
      XtVaSetValues (ErrOK, XtNlabel, button1, NULL);
    }
    if (button2 == NULL) {
      XUnmapWindow (XtDisplay(ErrNO), XtWindow(ErrNO));
    } else {
      XMapWindow (XtDisplay(ErrNO), XtWindow(ErrNO));
      XtVaSetValues (ErrNO, XtNlabel, button2, NULL);
    }
    if (button3 == NULL) {
      XUnmapWindow (XtDisplay(ErrCancel), XtWindow(ErrCancel));
    } else {
      XMapWindow (XtDisplay(ErrCancel), XtWindow(ErrCancel));
      XtVaSetValues (ErrCancel, XtNlabel, button3, NULL);
    }
    while (True) {
        XNextEvent (d, &e);
        switch(e.type){
          case Expose:
                      XtDispatchEvent (&e);
                      break;
          case ButtonPress:
                      if (button1 != NULL)
                        if (e.xbutton.window == XtWindow (ErrOK)){
                          XtPopdown (TopError);
                          return VOK;
                        }
                      if (button2 != NULL)
                        if (e.xbutton.window == XtWindow (ErrNO)){
                          XtPopdown (TopError);
                          return VNO;
                        }
                      if (button3 != NULL)
                        if (e.xbutton.window == XtWindow (ErrCancel)) {
                          XtPopdown (TopError);
                          return VCANCEL;
                        }
                      XBell (XtDisplay (toplevel), 0);
         }
    }
}

/********  Popdown Panels  ********/
void Popdown_Panel (w, type, ntype)
Widget w;
int type, ntype;
{
    Widget top, pop;

    switch (type) {
      case PGAIN: 
                  if (!STATE(SPLIT)) return;
                  top = TopGain;
                  pop = PopGain;
                  break;
      case PCONT: 
                  if (!STATE(SPLIT)) return;
                  top = TopControl;
                  pop = PopControl;
                  break;
      case PFILE: 
                  top = TopFile;
                  if (STATE(SPLIT) == True){
                    pop = PopFile;  
                    XtVaSetValues (load, XtNstate, False, NULL);
                  } else
                    pop = load;
                  break;
      case PINFO: 
                  top = TopInfo;
                  pop = PopInfo;
                  break;
      case PSPEC: 
                  top = TopSpecial;
                  pop = PopSpecial;
                  break;
      case PREC : 
                  top = TopRecord;  
                  if (STATE(SPLIT) == True){
                    pop = PopRecord;  
                    XtVaSetValues (record, XtNstate, False, NULL);
                  } else
                    pop = record;
                  break;
      case PCONF:
                  top = TopConfig;
                  pop = PopConfig;
                  break;
      default   : return;
    }
    CHSTATE(POPUP, (STATE(POPUP) & ~type));
    XtPopdown (top);
    XtVaSetValues (pop, XtNstate, False, NULL);
}

/********  Popup Panels  ********/
void Popup_Panel (w, type, ntype)
Widget w;
int type, ntype;
{
    Dimension defw, defh;
    Widget top, pop, form;

    switch (type) {
      case PGAIN:
                 if (!STATE(SPLIT)) return;
                 top = TopGain;
                 pop = PopGain;
                 form = GainPanel;
                 break;
      case PCONT: 
                 if (!STATE(SPLIT)) return;
                 top = TopControl;
                 pop = PopControl;
                 form = Control;
                 break;
      case PFILE:
                 top = TopFile;
                 if (STATE(SPLIT) == True)
                   XtVaSetValues (PopFile, XtNstate, True, NULL);
                 pop = load;
                 form = FilePanel;
                 break;
      case PINFO:
                 top = TopInfo;
                 pop = PopInfo;
                 form = InfoPanel;
                 break;
      case PSPEC:
                 top = TopSpecial;
                 pop = PopSpecial;
                 form = SpePanel;
                 Set_Special ();
                 break;
      case PREC :
                 top = TopRecord;
                 if (STATE(SPLIT) == True)
                   XtVaSetValues (PopRecord, XtNstate, True, NULL);
                 pop = record;
                 form = RecPanel;
                 break;
      case PCONF:
                 top = TopConfig;
                 pop = PopConfig;
                 form = ConfigPanel;
                 break;
      default  : return;
    }
    if (STATE(POPUP) & type) {
      Popdown_Panel (top, type, 1);
    } else {
      XtVaGetValues (form, XtNwidth, &defw, XtNheight, &defh, NULL);
      if (defw > 0 && defh > 0)
        XtVaSetValues (top, XtNwidth, defw, XtNheight, defh, NULL);
      XtVaSetValues (pop, XtNstate, True, NULL);
      XtPopup (top, XtGrabNone);
      Create_Slider_Pixmap (type);
      CHSTATE(POPUP, (STATE(POPUP) | type));
    }
}

/*
 * Sliders procedures
 */
/*******  Play Volume  ********/
void play_volume_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    set_playgain (lab);
    Gain_Dev.play_gain = ~0;
}

/*******  Record Volume  ********/
void record_volume_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    set_recordgain (lab);
    Gain_Dev.record_gain = ~0;
}

/*******  Play  Balance  ********/
void play_balance_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    set_playbalance (lab);
    Gain_Dev.play_balance = ~0;
}

/*******  Record Balance  ********/
void record_balance_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    set_recordbalance (lab);
    Gain_Dev.record_balance = ~0;
}

/********  Monitor Volume  ********/
void monitor_volume_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    set_monitorgain (lab);
    Gain_Dev.monitor_gain = ~0;
}

/********  Play Speed  ********/
void play_speed_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    CHSTATE(PLAYS, lab);
}

/********  Play Pointer  ********/
void play_pointer_proc (value)
float   value;
{
    int lab;

    if ( ! (STATE(PRFLAG) & PLAY) ) return;
    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    move_file_pointer (lab);
}

/********  Play Gain Scale  ********/
void gain_scale_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    CHSTATE(SCALE, lab);
    SetScrollbars (SCALE, lab);
}

/********  Trigger Level Proc  *****/
void trigger_level_proc (value)
float   value;
{
    int lab;

    lab = value * MAX_GAIN;
    if (lab > MAX_GAIN) lab = MAX_GAIN;
    CHSTATE(TRIGL, lab);
    SetScrollbars (TRIGL, lab);
}

/********  Timeout Proc  ********/
void NewValue (cd, id)
caddr_t cd;
XtIntervalId  *id;
{
    Audio_Device newPlay;                      /* Device config for play  */
    Audio_Device newRecord;                    /* Decice config for record*/
    Audio_Gain   newGain;                      /* Gain config             */
    static int speed = -2, position = -1, stereo_delay = -1;
    char buff[50], *p;
    int  secs;

    GetAudioConfiguration (&newPlay, &newRecord, &newGain);

/***  Check Gain  ***/
    if (memcmp (&newGain, &Gain_Dev, sizeof (Audio_Gain)) != 0){
      if (STATE(PLAYV) != newGain.play_gain) {
        CHSTATE(PLAYV, newGain.play_gain);
        SetScrollbars (PLAYV, STATE(PLAYV));
      }
      if (STATE(RECV) != newGain.record_gain) {
        CHSTATE(RECV, newGain.record_gain);
        SetScrollbars (RECV, STATE(RECV));
      }
      if (STATE(PLAYB) != newGain.play_balance && (STATE(PROCESS) == DBRI)){
        int dum = newGain.play_balance;
        CHSTATE(PLAYB, dum);
        SetScrollbars (PLAYB, dum);
      }
      if (STATE(RECB) != newGain.record_balance && (STATE(PROCESS) == DBRI)){
        int dum = newGain.record_balance;
        CHSTATE(RECB, dum);
        SetScrollbars (RECB, dum);
      }
      if (STATE(MUTED) != newGain.mute && (STATE(PROCESS) == DBRI)) {
        int dum = newGain.mute;
        CHSTATE(MUTED, dum);
        set_output_label (newPlay.port);
      }
      if (STATE(MONV) != newGain.monitor_gain) {
        CHSTATE(MONV, newGain.monitor_gain);
        SetScrollbars (MONV, STATE(MONV));
      }
    }   /*  Gain */

/***  Check Play device configuration  ***/
    if (memcmp (&newPlay, &Play_Dev, sizeof (Audio_Device)) != 0){
      int set = False;
      if (Play_Dev.pause != newPlay.pause) {
        set_pause_label (PLAY, newPlay.pause);
        set = True;
      }
      if (Play_Dev.port != newPlay.port) {
        set_output_label (newPlay.port);
        set = True;
      }
      if ((Play_Dev.sample_rate != newPlay.sample_rate) ||
          (Play_Dev.encoding != newPlay.encoding)) {
        set_sample_rate_label (newPlay.sample_rate, newPlay.encoding);
        set = True;
      }
      if (Play_Dev.channel != newPlay.channel) {
        set_channel_label (newPlay.channel);
        set = True;
      }
      if (Play_Dev.waiting != newPlay.waiting) {
        if (newPlay.waiting)
          Put_Message ("Other process is waiting for play.", Non);
        else
          Put_Message (DEMO, Non);
        set = True;
      }
      if (set) Put_Device_Config (newPlay);
    } /*  End of Play Device */

/***  Check Record device configuration  ***/
    if (memcmp (&newRecord, &Record_Dev, sizeof (Audio_Device)) != 0) {
      if (Record_Dev.pause != newRecord.pause) {
        set_pause_label (RECORD, newRecord.pause);
      }
      if (Record_Dev.port != newRecord.port) {
        set_input_label((newRecord.port == AUDIO_MICROPHONE ? True : False));
      }
      if ((Record_Dev.sample_rate != newRecord.sample_rate) ||
          (Record_Dev.encoding != newRecord.encoding)) {
        set_sample_rate_label (newRecord.sample_rate, newRecord.encoding);
      }
      if (Record_Dev.channel != newRecord.channel) {
        set_channel_label (newRecord.channel);
      }
      if (Record_Dev.waiting != newRecord.waiting) {
        if (newRecord.waiting)
          Put_Message ("Other process is waiting for record.", Non);
        else
          Put_Message (DEMO, Non);
      }
    }  /***  Record  ***/ 

    if (speed != STATE(PLAYS)) {       /*  Play Speed Control  */
      if (STATE(PSTATE) == DOPLAY)
        Flush_Play ();
      speed =  STATE(PLAYS);
      SetScrollbars (PLAYS, speed);
    }

    if (STATE(PTYPE) == PERCENT) {     
      if (position != STATE(PLAYP)){   /*  Play File Pointer form  percent */
          SetScrollbars (PLAYP, STATE(PLAYP));
          position = STATE(PLAYP);
      }
    } else {
      if (position != (secs = Play_Seconds ())){  /* form time */
        SetScrollbars (PLAYP, (secs < 0 ? 0 : secs));
        position = secs;
      }
    }

    if (stereo_delay != STATE(STEREODEL)) {      /* Stereo Delay value */
      stereo_delay = STATE(STEREODEL);
      SetScrollbars (STEREODEL, stereo_delay);
    }

    if ( STATE(PRFLAG) & PLAY )   /* Do playing  */
      PlayMain (newPlay);
    if ( STATE(PRFLAG) & RECORD ) /* Do recoding */
      RecordMain (newRecord);

    Play_Dev   = newPlay;        /* Copy current device configurations */
    Record_Dev = newRecord;
    Gain_Dev   = newGain;

    XtAppAddTimeOut (APPCONTE(), TIMEOUT, NewValue, NULL);
    /* Call myself */
}

/********  Main  ********/
main (argc, argv)
int argc;
char **argv;
{
    char  *progname = *argv;
    XtAppContext app_con;

    CheckAudioDevice ();
    if (getenv (LANG_ENV) == NULL)
      CHSTATE(LANGF, False);
    else
      CHSTATE(LANGF, True);

#ifdef MULTI
    if (STATE(LANGF))
      XtSetLanguageProc (NULL, NULL, NULL);
#endif
    toplevel  = XtVaAppInitialize (&app_con, CLASSNAME, options,
                  XtNumber (options), &argc, argv, fallback_resources, NULL);

    CHAPPCONTE(app_con);
    XtVaGetApplicationResources (toplevel, (caddr_t)&app_resources,
                                      resources, XtNumber (resources), NULL);

    if (app_resources.help == True) usage (progname);
    CHSTATE(SPLIT, app_resources.split);
    if (app_resources.maxsize > 0)
      CHSTATE(RLIMIT, app_resources.maxsize);
    if (app_resources.stereodelay > 0)
      CHSTATE(STEREODEL, 
           (app_resources.stereodelay * 50 / MAX_STEREODELAY + MAX_GAIN/2));
    if (app_resources.direc != NULL) {
      strcpy (STRINGS(DIRNAME), app_resources.direc);
    }
    if (app_resources.pattern != NULL)
      strcpy (STRINGS(PATTERN), app_resources.pattern);
    else
      strcpy (STRINGS(PATTERN), DEFPATTERN);
    CHSTATE(ADJUST, (!app_resources.adjust));
    CHSTATE(SHORTLIST, app_resources.basename);
    CHSTATE(PTYPE, (app_resources.percent ? PERCENT:TIME));
    CHSTATE(PTYPE, (app_resources.time    ? TIME:PERCENT));

/*****  Creat Panels  *****/
    Paned     = XtVaCMW ("paned", PanedW,  toplevel,    NULL);

    if (STATE(SPLIT)) {
      Create_Split_Panel (Paned);
    } else {
      Create_Normal_Panel (Paned);
    }
    Create_Error_Panel (toplevel);

/********  Add Actions  ********/
    XtAppAddActions (APPCONTE(), actionsTable, XtNumber (actionsTable));

/********  Intializing  ********/
    Icon   (toplevel, icon_bits, icon_width, icon_height);

    SetDefaultScrollbars ();
    FileManager (argc, argv);
    NewValue (0, 0);
    CaretBlink (0, 0);
    XtRealizeWidget (toplevel);

/*****   Set File Panel geometry   *****/

    if (!STATE(SPLIT))
      Create_Slider_Pixmap (PGAIN);

    CheckAllTextMarking ();      /** Set toggle of String field **/
    SetScrollbars (PLAYP, 0);

    Put_File_Config (NULL);
    SetConfigMessage (NULL);
    XtAppMainLoop (APPCONTE());
}
